;+
; NAME:
;       AFTIME2DATE
;
; PURPOSE:
;       The purpose of this procedure is to convert
;       'aftime' in a TSD file to 'date' (yyyy/mm/dd hh:mm:ss UTC)
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.jaxa.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       aftime2date, GROUP_LEADER=wGroup
;
; MODIFICATION HISTORY:
;       Written by: Makiuti,S.
;       2007/08/15  Changed to consider a leap second
;-
;
pro Calcu_aftime, Event

  On_Error, 2

  ; Get Widget IDs
  WID_TEXT_AFTIME = widget_info(Event.top, find_by_uname='AFTIME_TEXT')
  WID_TEXT_AFDATE = widget_info(Event.top, find_by_uname='AFDATE_TEXT')

  ; Get value
  WIDGET_CONTROL, WID_TEXT_AFTIME, GET_VALUE=aftime

  aftime = double(aftime)

  ; Adjust a leap second at 2006/01/01 00:00:00 UTC
  adj_sec = 0.0d ; offset for a leap second; usually zero
  IF (aftime GE 189388801.0d) THEN BEGIN ; after 2006/01/01 00:00:00 UTC
    aftime = aftime - 1
  ENDIF ELSE IF ((aftime GE 189388800.0d) AND (aftime LT 189388801.0d)) THEN BEGIN ; at 2005/12/31 23:59:60.?
    aftime = aftime - 1
    adj_sec = 1.0d ; offset for a leap second
  ENDIF

  jd_aftime = (double(aftime)/60./60./24. + double(JULDAY(1,1,2000)) - 12.0/24.0)
  CALDAT, jd_aftime, Month, Day, Year, Hour, Minute, Second

  Second = Second + adj_sec ; for a leap second 2005/12/31 23:59:60.?

  afdate = '  '+strcompress(string(Year))+'/'+string(Month, format='(i2.2)')+'/'+string(Day, format='(i2.2)')+' '+$
         string(Hour, format='(i2.2)')+':'+string(Minute, format='(i2.2)')+':'+$
         strcompress(string(Second, format='(f6.3)'), /REMOVE_ALL)+' (UTC)'
  ; Set value
  WIDGET_CONTROL, WID_TEXT_AFDATE, SET_VALUE=afdate
end
;
; Empty stub procedure used for autoloading.
;
pro aftime2date_eventcb
end
;
pro AFTIME_BASE_0_event, Event

  wTarget = (widget_info(Event.id,/NAME) eq 'TREE' ?  $
      widget_info(Event.id, /tree_root) : event.id)


  wWidget =  Event.top

  case wTarget of

    Widget_Info(wWidget, FIND_BY_UNAME='AFTIME_TEXT'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_TEXT_CH' )then $
        Calcu_aftime, Event
    end
    else:
  endcase

end

pro AFTIME_BASE_0, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_

  Resolve_Routine, 'aftime2date_eventcb',/COMPILE_FULL_FILE  ; Load event callback routines

  AFTIME_BASE_0 = Widget_Base( GROUP_LEADER=wGroup,  $
      UNAME='AFTIME_BASE_0' ,XOFFSET=150 ,YOFFSET=200  $
      ,TITLE='Convert AFTIME', XPAD=4, YPAD=4, /COLUMN)


  AFTIMEBOX_BASE_1 = Widget_Base(AFTIME_BASE_0,  $
      XOFFSET=8 ,YOFFSET=8 ,UNAME='AFTIMEBOX_BASE_1' ,/COLUMN, FRAME=1)

  AFTIME_LABEL = Widget_Label(AFTIMEBOX_BASE_1, UNAME='AFTIME_LABEL'  $
      ,XOFFSET=8 ,YOFFSET=8 ,/ALIGN_LEFT  $
      ,VALUE='AFTIME (ASTRO-F Time) :')

  AFTIME_TEXT = Widget_Text(AFTIMEBOX_BASE_1, UNAME='AFTIME_TEXT'  $
      ,XOFFSET=8 ,YOFFSET=21 ,/EDITABLE  $
      ,/ALL_EVENTS ,XSIZE=25 ,YSIZE=1)


  AFDATE_TEXT = Widget_Text(AFTIME_BASE_0, UNAME='AFDATE_TEXT' ,XOFFSET=8  $
      ,YOFFSET=60 ,XSIZE=32 ,YSIZE=1)

  Widget_Control, /REALIZE, AFTIME_BASE_0

  XManager, 'AFTIME_BASE_0', AFTIME_BASE_0, /NO_BLOCK

end
;
; Empty stub procedure used for autoloading.
;
pro aftime2date, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
  AFTIME_BASE_0, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
end
