; $Id: det_image.pro,v 1.4 2007/07/05 10:40:04 makiuti Exp $
;+
; NAME:
;       Det_Image
;
; PURPOSE:
;       The purpose of this procedure is to display the detector
;       2D image and animate it.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.jaxa.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       det_image(, FISDET=FIS_det_arr, TSD=tsd_in, /TEST, GROUP_LEADER=Event.top)
;
;         FISDET : FIS SW or LW detector array ([100,*] or [75,*])
;         TSD    : TSD object; this keyword is ignored when FISDET is passed
;         /TEST  : for a test run, open a tsd file specified in this code
;         GROUP_LEADER : When called by FISv, parent widget is specified
;
;       When called from command-line, FISDET or TSD are ecpected.
;       When called by FISv, GROUP_LEADER is expected.
;
; OTHER PROGRAMS NEEDED:
;   The following programs are required to run this one:
;    - colorbar.pro
;
; MODIFICATION HISTORY:
;        Written by: Makiuti,S.
;        2003.05.17  Ver.2.0 - use IMAGE_CONT procedure
;        2004.02.14  Ver.3.0 - use CONGRID, BYTSCL, and TV
;        2004.05.17  Last modified (added surface(3D) plot)
;-
;

; Called at startup
pro ShowImage1st, widgetID
COMMON ShareFISv, tsd
COMMON ShareDet, det, surf_flag

   ;Find the ID of the top-level base
   parentID = widgetID
   WHILE Widget_Info(parentID, /Parent) NE 0 DO BEGIN
      parentID = Widget_Info(parentID, /Parent)
   ENDWHILE

  ; Get maximum and minimum values of detector image
  cr_max=max(det)
  cr_min=min(det)

  ; Get ID of a Color Range Text Box and set values
  WID_TEXT_CRmax = widget_info(parentID, find_by_uname='WID_TEXT_CRmax')
  WIDGET_CONTROL, WID_TEXT_CRmax, SET_VALUE=string(cr_max)
  WID_TEXT_CRmin = widget_info(parentID, find_by_uname='WID_TEXT_CRmin')
  WIDGET_CONTROL, WID_TEXT_CRmin, SET_VALUE=string(cr_min)



  ; Get ID of DATA selector in slider
  IMAGE_DRAW = widget_info(parentID, find_by_uname='IMAGE_DRAW')
  WIDGET_CONTROL, IMAGE_DRAW, GET_VALUE=wid

  DEVICE, decomposed=0
  LOADCT, 3
  wset, wid

  ; Get ID of the data selector slider
  SLIDER_smpl_num = widget_info(parentID, find_by_uname='SLIDER_smpl_num')
  ; Get the value selected in the slider now
  widget_control, SLIDER_smpl_num, get_value=slidevalue

  ; Find number of pixels
  det_ch_num = n_elements(det[*,0])

  IF (det_ch_num EQ 100) THEN BEGIN
    ; Reform detector signal array
    imarr=reform(det[*,slidevalue], 20, 5)
    title='SW Detector sample#'+string(slidevalue)
    bairitu=15.0
  ENDIF ELSE IF (det_ch_num EQ 75) THEN BEGIN
    ; Reform detector signal array
    imarr=reform(det[*,slidevalue], 15, 5)
    title='LW Detector sample#'+string(slidevalue)
    bairitu=20.0
  ENDIF

  ; Reverse detector image vertically
  imarr = reverse(imarr, 2)

  ; Get ID of DRAW_TITLE
  DRAW_TITLE = widget_info(parentID, find_by_uname='DRAW_TITLE')
  ; Set title value
  widget_control, DRAW_TITLE, set_value=title

  ; Scale range
  imarr2=bytscl(imarr, min=cr_min, max=cr_max)
  ; Display
  imarr2=congrid(imarr2, n_elements(imarr[*,0])*bairitu, $
                         n_elements(imarr[0,*])*bairitu )
  tv, imarr2[*, 0*bairitu:2*bairitu-1], 85,40
  tv, imarr2[*, 2*bairitu:5*bairitu-1], 85,110
  colorbar, /vertical, position=[0.01, 0.05, 0.05, 0.95] $
          , range=[double(cr_min), double(cr_max)], CHARSIZE=0.7, /right $
          , FORMAT='(E9.1)'


  DEVICE, decomposed=1

end
;
;-----------------------------------------------------------------
pro ShowImage, Event, NO_ERASE=no_erase, SURF=surf
COMMON ShareFISv, tsd
COMMON ShareDet, det, surf_flag

  ; Get ID of a Color Range Text Box and set values
  WID_TEXT_CRmax = widget_info(Event.top, find_by_uname='WID_TEXT_CRmax')
  WIDGET_CONTROL, WID_TEXT_CRmax, GET_VALUE=cr_max
  WID_TEXT_CRmin = widget_info(Event.top, find_by_uname='WID_TEXT_CRmin')
  WIDGET_CONTROL, WID_TEXT_CRmin, GET_VALUE=cr_min


  ; Get ID of DATA selector in slider
  IMAGE_DRAW = widget_info(Event.top, find_by_uname='IMAGE_DRAW')
  WIDGET_CONTROL, IMAGE_DRAW, GET_VALUE=wid

  DEVICE, decomposed=0
  LOADCT, 3, /SILENT
  wset, wid

  ; Erase the previous drawing
  If(NOT keyword_set(no_erase)) then erase

  ; Get ID of the data selector slider
  SLIDER_smpl_num = widget_info(Event.top, find_by_uname='SLIDER_smpl_num')
  ; Get the value selected in the slider now
  widget_control, SLIDER_smpl_num, get_value=slidevalue

  ; Find number of pixels
  det_ch_num = n_elements(det[*,0])

  IF (det_ch_num EQ 100) THEN BEGIN
    ; Reform detector signal array
    imarr=reform(det[*,slidevalue], 20, 5)
    title='SW Detector sample#'+string(slidevalue)
    bairitu=15.0
  ENDIF ELSE IF (det_ch_num EQ 75) THEN BEGIN
    ; Reform detector signal array
    imarr=reform(det[*,slidevalue], 15, 5)
    title='LW Detector sample#'+string(slidevalue)
    bairitu=20.0
  ENDIF

  ; Reverse detector image vertically
  imarr = reverse(imarr, 2)

  ; Get ID of DRAW_TITLE
  DRAW_TITLE = widget_info(Event.top, find_by_uname='DRAW_TITLE')
  ; Set title value
  widget_control, DRAW_TITLE, set_value=title

  ; Scale range
  imarr2=bytscl(imarr, min=double(cr_min), max=double(cr_max) )
  ; Display
  imarr2=congrid(imarr2, n_elements(imarr[*,0])*bairitu, $
                         n_elements(imarr[0,*])*bairitu )

  IF(NOT keyword_set(surf)) THEN BEGIN
    tv, imarr2[*, 0*bairitu:2*bairitu-1], 85,40
    tv, imarr2[*, 2*bairitu:5*bairitu-1], 85,110
    colorbar, /vertical, position=[0.01, 0.05, 0.05, 0.95] $
            , range=[double(cr_min), double(cr_max)], CHARSIZE=0.7, /right $
            , FORMAT='(E9.1)'
    ;image_cont, imarr, /ASPECT
  ENDIF

  ; Display surface image
  IF(keyword_set(surf)) THEN BEGIN
    xtname = [string(0+1, format='(i2)'), string(n_elements(imarr[*,0]), format='(i2)')]; for x lavel
    ytname = [string(0+1, format='(i2)'), string(n_elements(imarr[0,*]), format='(i2)')]; for y lavel
    shade_surf, imarr2, shade=imarr2, ax=80, az=10, $
                 xstyle=1, xticks=1, xtickv=[10,bairitu*(n_elements(imarr[*,0])-0.5)], xtickname=xtname, $
                 ystyle=1, yticks=1, ytickv=[bairitu*4.5, 10],   ytickname=ytname, $
                 zstyle=1, zticks=1, ztickname=[' ', ' '], NOERASE=no_erase
    surfr, ax=80, az=10
    contour, imarr2, nlevels=8, /t3d, /overplot
  ENDIF

  DEVICE, decomposed=1

end
;
;-----------------------------------------------------------------
pro AutoBtn, Event
COMMON ShareDet, det, surf_flag

  ; Get ID of the data selector slider
  SLIDER_smpl_num = widget_info(Event.top, find_by_uname='SLIDER_smpl_num')
  ; Get the value selected in the slider now
  widget_control, SLIDER_smpl_num, get_value=slidevalue
  ; Get min & max values of the slider
  slider_min_max = widget_info(SLIDER_smpl_num, /SLIDER_MIN_MAX)

  ; Get ID of the Auto Play Speed Text Box
  WID_TEXT_Speed = widget_info(Event.top, find_by_uname='WID_TEXT_Speed')
  ; Get speed value for auto play
  WIDGET_CONTROL, WID_TEXT_Speed, GET_VALUE=speed
  ; Set auto play speed (waiting time)
  speed=float(speed)


  FOR i = slidevalue, slider_min_max[1], 1 DO BEGIN

    ; Get value selected in slider now
    widget_control, SLIDER_smpl_num, get_value=slidevalue
    i = slidevalue

    slidevalue = i + 1
    ; Set value in slider
    widget_control, SLIDER_smpl_num, set_value=slidevalue

    ShowImage, Event, /no_erase, surf=surf_flag
    wait, speed

  ENDFOR

end
;
;-----------------------------------------------------------------
pro CRmax_ev, Event
COMMON ShareDet, det, surf_flag

  ; Get ID of a Color Range MAX Text Box
  WID_TEXT_CRmax = widget_info(Event.top, find_by_uname='WID_TEXT_CRmax')
  ; Get max value of color range
  WIDGET_CONTROL, WID_TEXT_CRmax, GET_VALUE=cr_max

  ; Get ID of a Color Range MIN Text Box
  WID_TEXT_CRmin = widget_info(Event.top, find_by_uname='WID_TEXT_CRmin')
  ; Get min value of color range
  WIDGET_CONTROL, WID_TEXT_CRmin, GET_VALUE=cr_min

  ; If min value is greater than max value
  IF (float(cr_min) GE float(cr_max)) THEN BEGIN
    return
  ENDIF

  ; Redraw image
  ShowImage, Event, surf=surf_flag

end
;-----------------------------------------------------------------
pro CRmin_ev, Event
COMMON ShareDet, det, surf_flag

  ; Get ID of a Color Range MAX Text Box
  WID_TEXT_CRmax = widget_info(Event.top, find_by_uname='WID_TEXT_CRmax')
  ; Get max value of color range
  WIDGET_CONTROL, WID_TEXT_CRmax, GET_VALUE=cr_max

  ; Get ID of a Color Range MIN Text Box
  WID_TEXT_CRmin = widget_info(Event.top, find_by_uname='WID_TEXT_CRmin')
  ; Get min value of color range
  WIDGET_CONTROL, WID_TEXT_CRmin, GET_VALUE=cr_min

  ; If min value is greater than max value
  IF (float(cr_min) GE float(cr_max)) THEN BEGIN
    ;cr_min = string(float(cr_max) - 256.0)
    ;WIDGET_CONTROL, WID_TEXT_CRmin, SET_VALUE=cr_min
    return
  ENDIF

  ; Redraw image
  ShowImage, Event, surf=surf_flag

end
;
;-----------------------------------------------------------------
pro Data_Det, Event
COMMON ShareFISv, tsd
COMMON ShareDet, det, surf_flag

  ; Make array of detector signals
  det=tsd->get_value_tsd('FIS_OBS', 'DET')


  ; Get maximum and minimum values of detector image
  cr_max=max(det)
  cr_min=min(det)

  IF (cr_max EQ cr_min) THEN BEGIN
    cr_max = cr_max + 1
    cr_min = cr_min - 1
  ENDIF

  ; Get ID of a Color Range Text Box and set values
  WID_TEXT_CRmax = widget_info(Event.top, find_by_uname='WID_TEXT_CRmax')
  WIDGET_CONTROL, WID_TEXT_CRmax, SET_VALUE=string(cr_max)
  WID_TEXT_CRmin = widget_info(Event.top, find_by_uname='WID_TEXT_CRmin')
  WIDGET_CONTROL, WID_TEXT_CRmin, SET_VALUE=string(cr_min)

  surf_flag = 0;; initial display is NOT 'surface' but 'tv'

  ; Get ID of the 'Surf' button and set value of the button as initial value
  Surf_IMG_Btn = widget_info(Event.top, find_by_uname='Surf_IMG_Btn')
  WIDGET_CONTROL, Surf_IMG_Btn, SET_VALUE='3D'

  ShowImage, Event

end
;-----------------------------------------------------------------
pro Data_Flux, Event
COMMON ShareFISv, tsd
COMMON ShareDet, det, surf_flag

  ; Make array of detector signals
  det=tsd->get_value_tsd('FIS_OBS', 'FLUX')


  ; Get maximum and minimum values of detector image
  cr_max=max(det)
  cr_min=min(det)

  IF (cr_max EQ cr_min) THEN BEGIN
    cr_max = cr_max + 1
    cr_min = cr_min - 1
  ENDIF

  ; Get ID of a Color Range Text Box and set values
  WID_TEXT_CRmax = widget_info(Event.top, find_by_uname='WID_TEXT_CRmax')
  WIDGET_CONTROL, WID_TEXT_CRmax, SET_VALUE=string(cr_max)
  WID_TEXT_CRmin = widget_info(Event.top, find_by_uname='WID_TEXT_CRmin')
  WIDGET_CONTROL, WID_TEXT_CRmin, SET_VALUE=string(cr_min)

  surf_flag = 0;; initial display is NOT 'surface' but 'tv'

  ; Get ID of the 'Surf' button and set value of the button as initial value
  Surf_IMG_Btn = widget_info(Event.top, find_by_uname='Surf_IMG_Btn')
  WIDGET_CONTROL, Surf_IMG_Btn, SET_VALUE='3D'

  ShowImage, Event

end

;-----------------------------------------------------------------
pro Surf_img, Event
COMMON ShareDet, det, surf_flag

  ; Get ID of the 'Surf' button
  Surf_IMG_Btn = widget_info(Event.top, find_by_uname='Surf_IMG_Btn')
  ; set 'sruf_flag' and change the value of the button
  IF (surf_flag EQ 0) THEN BEGIN
    surf_flag = 1 & WIDGET_CONTROL, Surf_IMG_Btn, SET_VALUE='2D'
  ENDIF ELSE BEGIN
    surf_flag = 0 & WIDGET_CONTROL, Surf_IMG_Btn, SET_VALUE='3D'
  ENDELSE

  ShowImage, Event, surf=surf_flag

end
;
;-----------------------------------------------------------------
pro CloseBtn, Event
  LOADCT, 0
  WIDGET_CONTROL, Event.top, /DESTROY
end

;-----------------------------------------------------------------
;
; Empty stub procedure used for autoloading.
pro det_image_eventcb
end

pro DETIMG_TOP_event, Event
COMMON ShareDet, det, surf_flag

  wTarget = (widget_info(Event.id,/NAME) eq 'TREE' ?  $
      widget_info(Event.id, /tree_root) : event.id)


  wWidget =  Event.top

  case wTarget of

    Widget_Info(wWidget, FIND_BY_UNAME='SLIDER_smpl_num'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_SLIDER' )then $
        ShowImage, Event, surf=surf_flag
    end
    Widget_Info(wWidget, FIND_BY_UNAME='WID_BUTTON_Auto'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        AutoBtn, Event
    end
    Widget_Info(wWidget, FIND_BY_UNAME='WID_BUTTON_0'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        CloseBtn, Event
    end
    Widget_Info(wWidget, FIND_BY_UNAME='WID_TEXT_CRmax'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_TEXT_CH' )then $
        CRmax_ev, Event
    end
    Widget_Info(wWidget, FIND_BY_UNAME='WID_TEXT_CRmin'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_TEXT_CH' )then $
        CRmin_ev, Event
    end
    Widget_Info(wWidget, FIND_BY_UNAME='Sect_DET_Btn'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        Data_Det, Event
    end
    Widget_Info(wWidget, FIND_BY_UNAME='Sect_FLUX_Btn'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        Data_Flux, Event
    end
    Widget_Info(wWidget, FIND_BY_UNAME='Surf_IMG_Btn'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        Surf_img, Event
    end
    else:
  endcase

end

pro DETIMG_TOP, FISDET=FIS_det_arr, TSD=tsd_in, TEST=TEST, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
COMMON ShareFISv, tsd
COMMON ShareDet, det, surf_flag

  surf_flag = 0

  ; When called from command-line (not from FISv) with existing tsd object
  IF Obj_Valid(tsd_in) NE 0 THEN tsd=tsd_in

  Resolve_Routine, 'det_image_eventcb',/COMPILE_FULL_FILE  ; Load event callback routines

  DETIMG_TOP = Widget_Base( GROUP_LEADER=wGroup, UNAME='DETIMG_TOP'  $
      ,TITLE='FIS Detector Image' ,XOFFSET=50 ,YOFFSET=50 ,/ROW)

  ; Left-hand base
  Left_BASE = Widget_Base(DETIMG_TOP, /column)

  ; Base for Color Range
  CRange_Base = Widget_Base(Left_BASE, /frame, /column)
  Wid_Lavel_CRtitle = Widget_Label(CRange_Base, value='Color range')
  WID_LABEL_CRmax = Widget_Label(CRange_Base, UNAME='WID_LABEL_CRmax'  $
      ,VALUE='Max:')
  WID_TEXT_CRmax = Widget_Text(CRange_Base, UNAME='WID_TEXT_CRmax'  $
       ,SCR_XSIZE=80, /ALL_EVENTS ,VALUE=['256'], /EDITABLE)

  WID_LABEL_CRmin = Widget_Label(CRange_Base, UNAME='WID_LABEL_CRmin'  $
      ,VALUE='Min:')
  WID_TEXT_CRmin = Widget_Text(CRange_Base, UNAME='WID_TEXT_CRmin'  $
       ,SCR_XSIZE=80, /ALL_EVENTS ,VALUE=['0'], /EDITABLE)

  ; Auto play speed
  Speed_Base = Widget_Base(Left_BASE, /frame, /column)
  Wid_Lavel_Speed = Widget_Label(Speed_Base, value='Speed [sec/frm]')
  WID_TEXT_Speed = Widget_Text(Speed_Base, UNAME='WID_TEXT_Speed'  $
       ,SCR_XSIZE=80, /ALL_EVENTS ,VALUE=['   0.1'], /EDITABLE)

  ; DET or FLUX, Select Button
  Slct_Btn_Base = Widget_Base(Left_BASE, /frame, /column, /EXCLUSIVE)
  Sect_DET_Btn = Widget_Button(Slct_Btn_Base, UNAME='Sect_DET_Btn', Value='DET')
  Sect_FLUX_Btn = Widget_Button(Slct_Btn_Base, UNAME='Sect_FLUX_Btn', Value='FLUX')
  Widget_Control, Sect_DET_Btn, /Set_Button

  ; Show profile
  Prol_DET_Btn = Widget_Button(Left_BASE, UNAME='Surf_IMG_Btn', Value='3D', Tooltip='Surface image')

  ; Right-hand base
  Right_BASE = Widget_Base(DETIMG_TOP, /column)

  ; Base for detector image display
  IMAGE_BASE = Widget_Base(Right_BASE, UNAME='IMAGE_BASE' ,/BASE_ALIGN_CENTER ,TITLE='Detector Image' $
      ,SPACE=3 ,XPAD=3 ,YPAD=3 ,COLUMN=1, /frame)

  DRAW_TITLE = Widget_Label(IMAGE_BASE, UNAME='DRAW_TITLE', Value='Detector Image' $
      , /DYNAMIC_RESIZE)

  IMAGE_DRAW = Widget_Draw(IMAGE_BASE, UNAME='IMAGE_DRAW' $
      ,SCR_XSIZE=400 ,SCR_YSIZE=200)


  SLIDER_smpl_num = Widget_Slider(IMAGE_BASE, UNAME='SLIDER_smpl_num'  $
      ,XSIZE=400)


  WID_BUTTON_Auto = Widget_Button(IMAGE_BASE, UNAME='WID_BUTTON_Auto'  $
      ,XOFFSET=148 ,YOFFSET=173 ,SCR_XSIZE=80 ,SCR_YSIZE=30  $
      ,/ALIGN_CENTER ,VALUE='Play', Tooltip='Auto Play')

  Wid_Lavel_note = Widget_Label(IMAGE_BASE, value='To stop playing, drag the slide bar to the right end.')

  ; Close button
  ClsBtn_BASE = Widget_Base(Right_BASE, UNAME='ClsBtn_BASE' ,/ALIGN_RIGHT $
      ,COLUMN=1)

  WID_BUTTON_0 = Widget_Button(ClsBtn_BASE, UNAME='WID_BUTTON_0'  $
      ,SCR_XSIZE=80 ,SCR_YSIZE=25  $
      ,/ALIGN_CENTER ,VALUE='Close', Tooltip='Close window')

  ; If FIS detector array is passed from command-line
  IF (size(FIS_det_arr, /n_dimensions) EQ 2) and $
    ((size(FIS_det_arr, /dimensions))[0] EQ 100) or $
    ((size(FIS_det_arr, /dimensions))[0] EQ 75 ) $
    THEN BEGIN

    det = FIS_det_arr

    ; Destroy the DET or FLUX select button
    Widget_Control, Slct_Btn_Base, /destroy

  ENDIF ELSE BEGIN

    ; If TSD object was not passed from command-line
    ; , or if this procedure was not called by FISv
    IF (Obj_Valid(tsd) EQ 0) THEN BEGIN
      IF (keyword_set(test)) THEN BEGIN
        ; Make TSD object for **TEST**
        tsd=Obj_New('tsd', INFILE='C:\RSI\sample_fits\TSD2\fis20030804_105659_01_lw.fits')
        ;tsd=Obj_New('tsd', INFILE='C:\RSI\sample_fits\TSD2\fis20030804_105659_01_sw.fits')
        ;tsd=Obj_New('tsd', INFILE='C:\RSI\IDL56\sample_fits\fis20021114_231444_06_sw.fits')
      ENDIF ELSE BEGIN
        print, '** USAGE:'
        print, '> det_image(, FISDET=FIS_det_arr, TSD=tsd_in, /TEST, GROUP_LEADER=Event.top)'
        Return
      ENDELSE
    ENDIF

    ; Make array of FIS detector signals
    det=tsd->get_value_tsd('FIS_OBS', 'DET')
  ENDELSE

  ;## Set Values of Slidebars ##
  ; Find number of array samples
  det_smp_max = n_elements(det[0,*])
  ; Set slider max to the number of samples
  widget_control, SLIDER_smpl_num, set_slider_max=det_smp_max-1


  Widget_Control, /REALIZE, DETIMG_TOP


  ; Display the detector image of the 1st sample
  ShowImage1st, IMAGE_DRAW ; can replace IMAGE_DRAW with any widgets


  XManager, 'DETIMG_TOP', DETIMG_TOP, /NO_BLOCK

end
;
; Empty stub procedure used for autoloading.
;
pro det_image, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
  DETIMG_TOP, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
end
