; $Id: lds_query.pro,v 1.7 2007/07/05 10:40:04 makiuti Exp $
;+
; NAME:
;       LDS_QUERY
;
; PURPOSE:
;       The purpose of this procedure is to query
;       the Local Data Server and get a TSD object.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.jaxa.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       lds_query, GROUP_LEADER=wGroup
;
; MODIFICATION HISTORY:
;        Written by: Makiuti,S.
;        07/07/05 Last modified
;-
;
;-----------------------------------------------------------------
PRO lds_query_ok, Event
COMMON ShareFISv
COMMON LDSq, lds_info

  ; Get Widget IDs
  WID_TEXT_1   = widget_info(Event.top, find_by_uname='WID_TEXT_1')
  WID_TEXT_2   = widget_info(Event.top, find_by_uname='WID_TEXT_2')
  Sect_SW_Btn  = widget_info(Event.top, find_by_uname='Sect_SW_Btn')
  Sect_DR1_Btn = widget_info(Event.top, find_by_uname='Sect_DR1_Btn')
  Sect_DR2_Btn = widget_info(Event.top, find_by_uname='Sect_DR2_Btn')
;  Sect_Fli_Btn = widget_info(Event.top, find_by_uname='Sect_Fli_Btn')
;  Sect_Lab_Btn = widget_info(Event.top, find_by_uname='Sect_Lab_Btn')
  Sect_GB0_Btn = widget_info(Event.top, find_by_uname='Sect_GB0_Btn')

  ; Get value
  WIDGET_CONTROL, WID_TEXT_1, GET_VALUE=stime
  WIDGET_CONTROL, WID_TEXT_2, GET_VALUE=span

  ; stime & span are one-element arrays. So, convert into scalar values.
  stime = stime[0]
  span  = span[0]

  ; Detector type (SW/LW)
  det_type = widget_info(Sect_SW_Btn, /BUTTON_SET)
  IF (det_type EQ 1) THEN det = 'FIS_SW' ELSE det = 'FIS_LW'

  ; Data query mode (DR1/DR2) ;;(Fli./Lab./Sim. in the past)
  query_mode = widget_info(Sect_DR1_Btn, /BUTTON_SET)
  IF (query_mode EQ 1) THEN BEGIN
    mode = 'DR1'
  ENDIF ELSE BEGIN
    query_mode = widget_info(Sect_DR2_Btn, /BUTTON_SET)
    IF (query_mode EQ 1) THEN mode = 'DR2' ELSE mode = 'Default'
  ENDELSE

  ; GB_processed (0/1)
  gb_proc0 = widget_info(Sect_GB0_Btn, /BUTTON_SET)
  IF (gb_proc0 EQ 1) THEN gb_p = '0' ELSE gb_p = '1'

  print, 'Query the Local Data Server.'
  print, '  time : '   + stime
  print, '  span : '   + span
  print, '  detector: '+ det
  print, '  mode : '   + mode
  print, '  GB   : '   + gb_p

  ; Information structure to keep the last parameters.
  lds_info = { $
    stime:stime, $
    span:span,   $
    det:det,     $
    mode:mode,   $
    gb_p:gb_p }

  ; First, destroy all existing pointer heap variables
  ;OBJ_DESTROY, OBJ_VALID()
  IF (OBJ_VALID(tsd)) THEN OBJ_DESTROY, tsd
  ; Query the LDS
  WIDGET_CONTROL, /HOURGLASS
  tsd = Obj_New('tsd')
  ret = tsd->read_tsd_from_lds(time=stime, span=span, format=det, mode=mode, $
                               gb_processed=gb_p, /COMPRESS)

  ; In the case that the LDS cannot find any data.
  IF (size(ret, /type) NE 7) THEN BEGIN
    Result = DIALOG_MESSAGE('LDS cannot find any data. TSD was not created.', $
        TITLE='LDS result', /INFORMATION, DIALOG_PARENT=Event.top)
    ;OBJ_DESTROY, OBJ_VALID()
    IF (OBJ_VALID(tsd)) THEN OBJ_DESTROY, tsd
  ENDIF

  WIDGET_CONTROL, Event.top, /DESTROY
END

;-----------------------------------------------------------------
pro lds_query_cancel, Event
  WIDGET_CONTROL, Event.top, /DESTROY
end
;-----------------------------------------------------------------
;
; Empty stub procedure used for autoloading.
;
pro lds_query_eventcb
end
;
;-----------------------------------------------------------------
PRO LDS_Q_BASE_event, Event

  wTarget = (widget_info(Event.id,/NAME) eq 'TREE' ?  $
      widget_info(Event.id, /tree_root) : event.id)

  wWidget =  Event.top

  case wTarget of

    Widget_Info(wWidget, FIND_BY_UNAME='WID_BUTTON_0'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        lds_query_ok, Event
    end
    Widget_Info(wWidget, FIND_BY_UNAME='WID_BUTTON_1'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        lds_query_cancel, Event
    end
    else:
  endcase

END
;-----------------------------------------------------------------
PRO LDS_Q_BASE, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
COMMON LDSq, lds_info

IF n_elements(lds_info) THEN BEGIN
  stime = lds_info.stime
  span  = lds_info.span
  det   = lds_info.det
  mode  = lds_info.mode
  gb_p  = lds_info.gb_p
ENDIF ELSE BEGIN ; Initial settings
  stime='2006-05-15 17:10:00'
  span ='60'
  det  ='FIS_SW'
  mode ='DR1'
  gb_p ='0'
ENDELSE

  Resolve_Routine, 'lds_query_eventcb',/COMPILE_FULL_FILE  ; Load event callback routines

  LDS_Q_BASE = Widget_Base( GROUP_LEADER=wGroup, UNAME='LDS_Q_BASE'  $
      ,XOFFSET=110 ,YOFFSET=190 ,TITLE='LDS Query' ,SPACE=3 ,XPAD=3 ,YPAD=3)

  WID_BASE_1 = Widget_Base(LDS_Q_BASE, UNAME='WID_BASE_1' ,XOFFSET=4  $
      ,YOFFSET=6 ,SPACE=3 ,XPAD=3 ,YPAD=3 , /COLUMN)
  ;
  ; # LDS location
  WID_BASE_TITLE = Widget_Base(WID_BASE_1, UNAME='WID_BASE_TITLE' ,XOFFSET=3  $
      ,YOFFSET=3, /ROW ,/ALIGN_CENTER)

  WID_LABEL_TITLE = Widget_Label(WID_BASE_TITLE, UNAME='WID_LABEL_TITLE'  $
      ;,/SUNKEN_FRAME ,/ALIGN_CENTER ,VALUE='LDS : http://afsvr02.ir.isas.jaxa.jp/LDS/')
      ,/SUNKEN_FRAME ,/ALIGN_CENTER ,VALUE='LDS : http://www.ir.isas.jaxa.jp/AKARI/iris_data/LDS/')
  ;
  ; # Query base
  WID_BASE_QUERY = Widget_Base(WID_BASE_1, UNAME='WID_BASE_QUERY' ,XOFFSET=3  $
      ,YOFFSET=3, /COLUMN, /FRAME)
  ;
  ; label
  WID_BASE_2 = Widget_Base(WID_BASE_QUERY, UNAME='WID_BASE_2' ,XOFFSET=3  $
      ,YOFFSET=3 ,SPACE=3 ,XPAD=3 ,YPAD=3)
  WID_LABEL_1 = Widget_Label(WID_BASE_2, UNAME='WID_LABEL_1'  $
      ,XOFFSET=8 ,YOFFSET=4 ,/ALIGN_LEFT ,VALUE='Query the Local Data Server')
  ;
  ; keyword : time
  WID_BASE_3 = Widget_Base(WID_BASE_QUERY, UNAME='WID_BASE_3' ,XOFFSET=3  $
      ,YOFFSET=24 ,SPACE=3 ,XPAD=3 ,YPAD=3)
  WID_LABEL_2 = Widget_Label(WID_BASE_3, UNAME='WID_LABEL_2'  $
      ,XOFFSET=8 ,YOFFSET=8 ,/ALIGN_LEFT ,VALUE='Time :')
  WID_TEXT_1 = Widget_Text(WID_BASE_3, UNAME='WID_TEXT_1' ,XOFFSET=80  $
      ,YOFFSET=4 ,XSIZE=30 ,YSIZE=1 ,/EDITABLE, VALUE=stime)
  ;
  ; keyword : span
  WID_BASE_4 = Widget_Base(WID_BASE_QUERY, UNAME='WID_BASE_4' ,XOFFSET=3  $
      ,YOFFSET=50 ,SPACE=3 ,XPAD=3 ,YPAD=3)
  WID_LABEL_3 = Widget_Label(WID_BASE_4, UNAME='WID_LABEL_3'  $
      ,XOFFSET=8 ,YOFFSET=8 ,/ALIGN_LEFT ,VALUE='Span(sec) :')
  WID_TEXT_2 = Widget_Text(WID_BASE_4, UNAME='WID_TEXT_2' ,XOFFSET=80  $
      ,YOFFSET=4 ,XSIZE=30 ,YSIZE=1 ,/EDITABLE, VALUE=span)
 ;
 ;  Select SW/LW
  WID_BASE_Det = Widget_Base(WID_BASE_QUERY, UNAME='WID_BASE_Det' ,XOFFSET=3  $
      ,YOFFSET=50 ,XPAD=3 ,YPAD=0, /row, /EXCLUSIVE)

  Sect_SW_Btn = Widget_Button(WID_BASE_Det, UNAME='Sect_SW_Btn', Value='SW')
  Sect_LW_Btn = Widget_Button(WID_BASE_Det, UNAME='Sect_LW_Btn', Value='LW')
  IF strcmp(det, 'FIS_SW') THEN $
    Widget_Control, Sect_SW_Btn, /Set_Button $
    ELSE Widget_Control, Sect_LW_Btn, /Set_Button
 ;
 ;  Select Lab/Sim
  WID_BASE_Mode = Widget_Base(WID_BASE_QUERY, UNAME='WID_BASE_Mode' ,XOFFSET=3  $
      ,YOFFSET=50 ,XPAD=3 ,YPAD=0, /row, /EXCLUSIVE)

  Sect_DR1_Btn = Widget_Button(WID_BASE_Mode, UNAME='Sect_DR1_Btn', Value='DR1')
  Sect_DR2_Btn = Widget_Button(WID_BASE_Mode, UNAME='Sect_DR2_Btn', Value='DR2')
;  Sect_Fli_Btn = Widget_Button(WID_BASE_Mode, UNAME='Sect_Fli_Btn', Value='Fli.')
;  Sect_Lab_Btn = Widget_Button(WID_BASE_Mode, UNAME='Sect_Lab_Btn', Value='Lab.')
;  Sect_Sim_Btn = Widget_Button(WID_BASE_Mode, UNAME='Sect_Sim_Btn', Value='Sim.')

  IF strcmp(mode, 'DR1') THEN BEGIN
    Widget_Control, Sect_DR1_Btn, /Set_Button
;  ENDIF ELSE IF strcmp(mode, 'Lab') THEN BEGIN
;    Widget_Control, Sect_Lab_Btn, /Set_Button
  ENDIF ELSE BEGIN
    Widget_Control, Sect_DR2_Btn, /Set_Button
  ENDELSE
 ;
 ;  Select GB_processed
  WID_BASE_GB_P_base = Widget_Base(WID_BASE_QUERY, UNAME='WID_BASE_5' ,XOFFSET=3  $
      ,YOFFSET=50 ,SPACE=3 ,XPAD=3 ,YPAD=3, /Row)

  WID_LABEL_GB_P = Widget_Label(WID_BASE_GB_P_base, UNAME='WID_LABEL_GB_P'  $
      ,XOFFSET=8 ,YOFFSET=8 ,/ALIGN_LEFT ,VALUE='GB processed :')

  WID_BASE_GB_P = Widget_Base(WID_BASE_GB_P_base, UNAME='WID_BASE_GB' ,XOFFSET=3  $
      ,YOFFSET=50 ,XPAD=3 ,YPAD=0, /row, /EXCLUSIVE)

  Sect_GB0_Btn = Widget_Button(WID_BASE_GB_P, UNAME='Sect_GB0_Btn', Value='0')
  Sect_GB1_Btn = Widget_Button(WID_BASE_GB_P, UNAME='Sect_GB1_Btn', Value='1')
  IF strcmp(gb_p, '0') THEN $
    Widget_Control, Sect_GB0_Btn, /Set_Button $
    ELSE Widget_Control, Sect_GB1_Btn, /Set_Button
  ;
  ;  Query Button
  WID_BASE_5 = Widget_Base(WID_BASE_1, UNAME='WID_BASE_5' ,XOFFSET=74  $
      ,YOFFSET=76 ,/ALIGN_CENTER ,/BASE_ALIGN_CENTER ,TITLE='IDL'  $
      ,SPACE=3 ,XPAD=3 ,YPAD=3 ,ROW=1)

  WID_BUTTON_0 = Widget_Button(WID_BASE_5, UNAME='WID_BUTTON_0'  $
      ,XOFFSET=3 ,YOFFSET=3 ,/ALIGN_CENTER ,VALUE='Query')
  WID_BUTTON_1 = Widget_Button(WID_BASE_5, UNAME='WID_BUTTON_1'  $
      ,XOFFSET=36 ,YOFFSET=3 ,/ALIGN_CENTER ,VALUE='Cancel')
;;;
;;;
  Widget_Control, /REALIZE, LDS_Q_BASE

  XManager, 'LDS_Q_BASE', LDS_Q_BASE;, /NO_BLOCK

END
;-----------------------------------------------------------------
;
; Empty stub procedure used for autoloading.
;
PRO lds_query, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_

  LDS_Q_BASE, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_

END
