; $Id: list_vdet_ps.pro,v 1.7 2007/07/05 10:40:04 makiuti Exp $
;+
; NAME:
;       LIST_VDET_PS
;
; PURPOSE:
;       The purpose of this procedure is to output a PS file
;       of the list of detector signals. This procedure is
;       called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.jaxa.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       list_vdet_ps[, xrange, yrange]
;
; MODIFICATION HISTORY:
;        Written by Makiuti,S.
;        06/03/08 Last modified
;-
;
pro list_vdet_ps, det_data, xrange, yrange
COMMON ShareFISv

IF n_elements(xrange) EQ 0 THEN xrange=[0,0]
IF n_elements(yrange) EQ 0 THEN yrange=[0,0]

IF n_elements(det_data) EQ 0 THEN det_data = tsd->get_value_tsd('FIS_OBS', 'DET')

x = indgen(n_elements(det_data[0,*]), /L64)

vdet_ch_num = n_elements(det_data[*,0])

; Using FSC_PSConfig program
psObject = Obj_New("FSC_PSConfig", /Landscape, /European, $
            XSize=25.0, YSize=17.0, XOffset=2.0, YOffset=27.0)
psObject->GUI, Cancel=cancelled

IF cancelled EQ 0 THEN BEGIN

  entry_device = !d.name
  set_plot, 'PS'
  ;device, filename="test.ps", /landscape
  device, _Extra=psObject->GetKeywords()

  !p.multi=[0,5,5,0,0]

FOR k = 0, vdet_ch_num/5/5-1, 1 DO BEGIN
  FOR j = 0, 4, 1 DO BEGIN
    FOR i = (5*k)+(vdet_ch_num/5)*j, (5*k)+(vdet_ch_num/5)*j+4, 1 DO BEGIN

      ch_num=i+1

      IF (vdet_ch_num EQ 100) THEN $
        ch = 'SW Detector ch'+string(ch_num, format='(i3)') ;pixel to plot
      IF (vdet_ch_num EQ 75) THEN $
        ch = 'LW Detector ch'+string(ch_num, format='(i3)') ;pixel to plot
      ; Plot
      plot, x, det_data[i,*], TITLE=ch, charsize=1.0, /ynozero, $
        xticks=2, XTITLE='Index', $
        xrange=xrange, yrange=yrange, xstyle=1, ystyle=1

    ENDFOR
  ENDFOR
  xyouts, 0.72, -0.01, 'plotted at '+systime(), /NORMAL
  xyouts, 0.0, -0.01, tsd->get_filename(), /NORMAL
  erase
ENDFOR

  device, /close_file
  set_plot, entry_device
  !p.multi=0

ENDIF
Obj_Destroy, psObject

end