; $Id: make_x_aocu_tree.pro,v 1.7 2007/07/03 08:15:05 makiuti Exp $
;+
; NAME:
;       MAKE_X_AOCU_TREE
;
; PURPOSE:
;       This procedure makes the AOCU extension branch
;       in the X data selector of the FISv.
;       This procedure is called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.jaxa.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       make_x_aocu_tree, widgetID
;
; MODIFICATION HISTORY:
;        Written by: Makiuti,S.
;        2007/07/03  Last modified
;-
pro make_x_aocu_tree, widgetID

  parentID = widgetID
  WHILE Widget_Info(parentID, /Parent) NE 0 DO BEGIN
     parentID = Widget_Info(parentID, /Parent)
  ENDWHILE

  Widget_Control, parentID, UPDATE=0

  ;  ## AOCU Extension Root
  X_TREENODE_AOCU = widget_info(parentID, find_by_uname='X_TREENODE_AOCU')

  ; (1) Time
  X_TREENODE_AOCU_1 = Widget_Tree(X_TREENODE_AOCU, UNAME='X_TREENODE_AOCU_1'  $
      ,/FOLDER ,VALUE='Time')
    X_TREE_AFTIME = Widget_Tree(X_TREENODE_AOCU_1, UNAME='X_TREE_AOCU_AFTIME'  $
      ,VALUE='AFTIME', UVALUE='AOCU_AFTIME')
    X_TREE_PIMTIORG = Widget_Tree(X_TREENODE_AOCU_1, UNAME='X_TREE_AOCU_PIMTIORG'  $
      ,VALUE='PIMTIORG', UVALUE='AOCU_PIM_TI_ORG')

  ; (2) AOCU Status
  X_TREENODE_AOCU_2 = Widget_Tree(X_TREENODE_AOCU, UNAME='X_TREENODE_AOCU_2'  $
      ,/FOLDER ,VALUE='Status')
    X_TREE_AOCU_BLANK = Widget_Tree(X_TREENODE_AOCU_2, UNAME='X_TREE_AOCU_BLANK'  $
      ,VALUE='Blank', UVALUE='AOCU_BLANK')
    X_TREE_AOCU_AO_ROM_RAM = Widget_Tree(X_TREENODE_AOCU_2, UNAME='X_TREE_AOCU_AO_ROM_RAM'  $
      ,VALUE='AO_ROM_RAM', UVALUE='AOCU_AO_ROM_RAM')
    X_TREE_AOCU_ADS_MODE_B1 = Widget_Tree(X_TREENODE_AOCU_2, UNAME='X_TREE_AOCU_ADS_MODE_B1'  $
      ,VALUE='ADS_Mode_B1', UVALUE='AOCU_ADS_MODE_B1')
    X_TREE_AOCU_ADS_MODE_B0 = Widget_Tree(X_TREENODE_AOCU_2, UNAME='X_TREE_AOCU_ADS_MODE_B0'  $
      ,VALUE='ADS_Mode_B0', UVALUE='AOCU_ADS_MODE_B0')
    X_TREE_AOCU_ADS_RATE_MODE_B1 = Widget_Tree(X_TREENODE_AOCU_2, UNAME='X_TREE_AOCU_ADS_RATE_MODE_B1'  $
      ,VALUE='ADS_RATE_MODE_B1', UVALUE='AOCU_ADS_RATE_MODE_B1')
    X_TREE_AOCU_ADS_RATE_MODE_B0 = Widget_Tree(X_TREENODE_AOCU_2, UNAME='X_TREE_AOCU_ADS_RATE_MODE_B0'  $
      ,VALUE='ADS_RATE_MODE_B0', UVALUE='AOCU_ADS_RATE_MODE_B0')

  ; (3) Analog Data
  X_TREENODE_AOCU_3 = Widget_Tree(X_TREENODE_AOCU, UNAME='X_TREENODE_AOCU_3'  $
      ,/FOLDER ,VALUE='Analg Data')

    X_TREENODE_AOCU_QUATERNION = Widget_Tree(X_TREENODE_AOCU_3, UNAME='X_TREENODE_AOCU_QUATERNION'  $
        ,/FOLDER ,VALUE='Quaternion')
      X_TREE_AOCU_ADS_Q1 = Widget_Tree(X_TREENODE_AOCU_QUATERNION, UNAME='X_TREE_AOCU_Q1'  $
      ,VALUE='Quaternion_1', UVALUE='AOCU_QUAT1')
      X_TREE_AOCU_ADS_Q2 = Widget_Tree(X_TREENODE_AOCU_QUATERNION, UNAME='X_TREE_AOCU_Q2'  $
      ,VALUE='Quaternion_2', UVALUE='AOCU_QUAT2')
      X_TREE_AOCU_ADS_Q3 = Widget_Tree(X_TREENODE_AOCU_QUATERNION, UNAME='X_TREE_AOCU_Q3'  $
      ,VALUE='Quaternion_3', UVALUE='AOCU_QUAT3')
      X_TREE_AOCU_ADS_Q4 = Widget_Tree(X_TREENODE_AOCU_QUATERNION, UNAME='X_TREE_AOCU_Q4'  $
      ,VALUE='Quaternion_4', UVALUE='AOCU_QUAT4')

    X_TREENODE_AOCU_BODY_RATE = Widget_Tree(X_TREENODE_AOCU_3, UNAME='X_TREENODE_AOCU_BODY_RATE'  $
        ,/FOLDER ,VALUE='Body_Rate')
      X_TREE_AOCU_BR1 = Widget_Tree(X_TREENODE_AOCU_BODY_RATE, UNAME='X_TREE_AOCU_BR1'  $
      ,VALUE='Body_Rate_1', UVALUE='AOCU_BR1')
      X_TREE_AOCU_BR2 = Widget_Tree(X_TREENODE_AOCU_BODY_RATE, UNAME='X_TREE_AOCU_BR2'  $
      ,VALUE='Body_Rate_2', UVALUE='AOCU_BR2')
      X_TREE_AOCU_BR3 = Widget_Tree(X_TREENODE_AOCU_BODY_RATE, UNAME='X_TREE_AOCU_BR3'  $
      ,VALUE='Body_Rate_3', UVALUE='AOCU_BR3')

    X_TREE_AOCU_BRA = Widget_Tree(X_TREENODE_AOCU_3, UNAME='X_TREE_AOCU_RA'  $
      ,VALUE='RA', UVALUE='AOCU_RA')
    X_TREE_AOCU_BDEC = Widget_Tree(X_TREENODE_AOCU_3, UNAME='X_TREE_AOCU_DEC'  $
      ,VALUE='DEC', UVALUE='AOCU_DEC')
    X_TREE_AOCU_ANG = Widget_Tree(X_TREENODE_AOCU_3, UNAME='X_TREE_AOCU_RANG'  $
      ,VALUE='Roll', UVALUE='AOCU_RANG')

    X_TREE_AOCU_DBRA = Widget_Tree(X_TREENODE_AOCU_3, UNAME='X_TREE_AOCU_DRA'  $
      ,VALUE='d_RA', UVALUE='AOCU_DRA')
    X_TREE_AOCU_DBDEC = Widget_Tree(X_TREENODE_AOCU_3, UNAME='X_TREE_AOCU_DDEC'  $
      ,VALUE='d_DEC', UVALUE='AOCU_DDEC')
    X_TREE_AOCU_DANG = Widget_Tree(X_TREENODE_AOCU_3, UNAME='X_TREE_AOCU_DRANG'  $
      ,VALUE='d_Roll', UVALUE='AOCU_DRANG')

    X_TREE_AOCU_CRS_OFF = Widget_Tree(X_TREENODE_AOCU_3, UNAME='X_TREE_AOCU_CRS_OFF'  $
      ,VALUE='Crs_Off', UVALUE='AOCU_CRS_OFF')

  ; (4) Quality
  X_TREENODE_AOCU_4 = Widget_Tree(X_TREENODE_AOCU, UNAME='X_TREENODE_AOCU_4'  $
      ,/FOLDER ,VALUE='Quality')
    X_TREE_AOCU_QUAL_INTP = Widget_Tree(X_TREENODE_AOCU_4, UNAME='X_TREE_AOCU_QUAL_INTP'  $
      ,VALUE='Interpolated', UVALUE='AOCU_QUAL_INTP')

  Widget_Control, parentID, /UPDATE

end