; $Id: make_x_pixflag_ch_tree.pro,v 1.5 2007/07/05 10:40:04 makiuti Exp $
;+
; NAME:
;       MAKE_X_PIXFLAG_CH_TREE
;
; PURPOSE:
;       This procedure makes the ch(pixel) number branches in each
;       pix_flag node in the X data selector of the FISv.
;       This procedure is called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.jaxa.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       make_x_pixflag_ch_tree, widgetID, pixflag_type
;         'pixflag_type' is pixel_flag type (BAD, DEAD, etc.)
;
; MODIFICATION HISTORY:
;        Written by: Makiuti,S.
;-
pro make_x_pixflag_ch_tree, widgetID, pixflag_type
COMMON ShareFISv

  parentID = widgetID
  WHILE Widget_Info(parentID, /Parent) NE 0 DO BEGIN
     parentID = Widget_Info(parentID, /Parent)
  ENDWHILE

  Widget_Control, parentID, UPDATE=0

  pixflag_type_member = ['BAD', 'UNDEF_ANOM', 'ARITH_ERR', 'DEAD', 'SATURATE', $
                         'RESET', 'RSTANOM', 'NO_DIFF', $
                         'NO_RP_CORR', 'NO_DK_CORR', 'NO_DCCAL_CORR', 'NO_TR_CORR', $
                         'NO_FLAT_FIELD', 'NO_GPGL', 'NO_MTGL', 'NO_ABSCAL', $
                         'GPGL_TYPE1', 'GPGL_TYPE2', 'GPGL_TYPE3', 'GPGL_TYPE4', 'GPGL_TAIL', $
                         'MTGL_TYPE1', 'MTGL_TYPE2', 'MTGL_TYPE3', 'MTGL_TYPE4', 'MTGL_TAIL', $
                         'NO_PERI_CORR', 'SX_PEAK', 'SX_SOURCE', 'SX_OBJ']
  pixflag_type_header_member = ['BAD_', 'UANM', 'AERR', 'DEAD', 'SATU', $
                                'REST', 'RANM', 'NDIF', $
                                'NRPC', 'NDKC', 'NDCC', 'NTRC', $
                                'NFLF', 'NGPG', 'NMTG', 'NABC', $
                                'GPG1', 'GPG2', 'GPG3', 'GPG4', 'GPGT', $
                                'MTG1', 'MTG2', 'MTG3', 'MTG4', 'MTGT', $
                                'NPER', 'SPAK', 'SRCE', 'SOBJ']
  pixflag_type_header = pixflag_type_header_member[where(pixflag_type EQ pixflag_type_member)]
  ; convert one-element array to scalar
  pixflag_type_header = pixflag_type_header[0]

  ; Parent branch ID in which child leaves are made
  temp_str1 = 'X_PIXFLAG_'+pixflag_type_header
  X_TREE_PIXFLAG_ID = widget_info(parentID, find_by_uname=temp_str1)

  ; Get detector type (SW or LW)
  pixmax = (STRMATCH(tsd->Get_Datatype(), '*SW*', /FOLD_CASE) EQ 1) ? 100:75

  FOR i = 1, pixmax DO BEGIN

    IF (i LE 9) THEN BEGIN
      pix_str = string(i, format='(I1)')
    ENDIF ELSE IF ((i GE 10) AND (i LE 99)) THEN BEGIN
      pix_str = string(i, format='(I2)')
    ENDIF ELSE IF (i EQ 100) THEN BEGIN
      pix_str = string(i, format='(I3)')
    ENDIF

      temp_str1 = pixflag_type_header + pix_str
      temp_id1  = Widget_Tree(X_TREE_PIXFLAG_ID, UNAME='X_PixFlag_'+pix_str $
        ,VALUE=pix_str, UVALUE=temp_str1)

  ENDFOR

  Widget_Control, parentID, /UPDATE

end