; $Id: make_y_gads_tree.pro,v 1.6 2007/07/03 08:15:05 makiuti Exp $
;+
; NAME:
;       MAKE_Y_GADS_TREE
;
; PURPOSE:
;       This procedure makes the GADS extension branch
;       in the Y data selector of the FISv.
;       This procedure is called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.jaxa.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       make_y_gads_tree, widgetID
;
; MODIFICATION HISTORY:
;        Written by: Makiuti,S.
;        2007/07/03  Last modified
;-
pro make_y_gads_tree, widgetID

  parentID = widgetID
  WHILE Widget_Info(parentID, /Parent) NE 0 DO BEGIN
     parentID = Widget_Info(parentID, /Parent)
  ENDWHILE

  Widget_Control, parentID, UPDATE=0

  ;  ## ADS Extension Root
  Y_TREENODE_GADS = widget_info(parentID, find_by_uname='Y_TREENODE_GADS')

  ; (1) Time
  Y_TREENODE_GADS_1 = Widget_Tree(Y_TREENODE_GADS, UNAME='Y_TREENODE_GADS_1'  $
      ,/FOLDER ,VALUE='Time')
    Y_TREE_AFTIME = Widget_Tree(Y_TREENODE_GADS_1, UNAME='Y_TREE_GADS_AFTIME'  $
      ,VALUE='AFTIME', UVALUE='GADS_AFTIME')
    Y_TREE_PIMTIORG = Widget_Tree(Y_TREENODE_GADS_1, UNAME='Y_TREE_GADS_PIMTIORG'  $
      ,VALUE='PIMTIORG', UVALUE='GADS_PIM_TI_ORG')

  ; (2) Status
  Y_TREENODE_GADS_2 = Widget_Tree(Y_TREENODE_GADS, UNAME='Y_TREENODE_GADS_2'  $
      ,/FOLDER ,VALUE='Status')
    Y_TREE_GADS_BLANK = Widget_Tree(Y_TREENODE_GADS_2, UNAME='Y_TREE_GADS_BLANK'  $
      ,VALUE='Blank', UVALUE='GADS_BLANK')
    Y_TREE_GADS_ATT_CNT_MD_B1 = Widget_Tree(Y_TREENODE_GADS_2, UNAME='Y_TREE_GADS_ATT_CNT_MD_B1'  $
      ,VALUE='ATT_CNT_MD_B1', UVALUE='GADS_ATT_CNT_MD_B1')
    Y_TREE_GADS_ATT_CNT_MD_B0 = Widget_Tree(Y_TREENODE_GADS_2, UNAME='Y_TREE_GADS_ATT_CNT_MD_B0'  $
      ,VALUE='ATT_CNT_MD_B0', UVALUE='GADS_ATT_CNT_MD_B0')

  ; (3) Analog Data
  Y_TREENODE_GADS_3 = Widget_Tree(Y_TREENODE_GADS, UNAME='Y_TREENODE_GADS_3'  $
      ,/FOLDER ,VALUE='Analg Data')

    Y_TREENODE_GADS_QUATERNION = Widget_Tree(Y_TREENODE_GADS_3, UNAME='Y_TREENODE_GADS_QUATERNION'  $
        ,/FOLDER ,VALUE='Quaternion')
      Y_TREE_GADS_Q1 = Widget_Tree(Y_TREENODE_GADS_QUATERNION, UNAME='Y_TREE_GADS_Q1'  $
      ,VALUE='Quaternion_1', UVALUE='GADS_QUAT1')
      Y_TREE_GADS_Q2 = Widget_Tree(Y_TREENODE_GADS_QUATERNION, UNAME='Y_TREE_GADS_Q2'  $
      ,VALUE='Quaternion_2', UVALUE='GADS_QUAT2')
      Y_TREE_GADS_Q3 = Widget_Tree(Y_TREENODE_GADS_QUATERNION, UNAME='Y_TREE_GADS_Q3'  $
      ,VALUE='Quaternion_3', UVALUE='GADS_QUAT3')
      Y_TREE_GADS_Q4 = Widget_Tree(Y_TREENODE_GADS_QUATERNION, UNAME='Y_TREE_GADS_Q4'  $
      ,VALUE='Quaternion_4', UVALUE='GADS_QUAT4')

    Y_TREENODE_GADS_POS_ERR = Widget_Tree(Y_TREENODE_GADS_3, UNAME='Y_TREENODE_GADS_POS_ERR'  $
        ,/FOLDER ,VALUE='Pos_Err')
      Y_TREE_GADS_POS_ERR1 = Widget_Tree(Y_TREENODE_GADS_POS_ERR, UNAME='Y_TREE_GADS_POS_ERR1'  $
      ,VALUE='Pos_Err_1', UVALUE='GADS_ERR1')
      Y_TREE_GADS_POS_ERR2 = Widget_Tree(Y_TREENODE_GADS_POS_ERR, UNAME='Y_TREE_GADS_POS_ERR2'  $
      ,VALUE='Pos_Err_2', UVALUE='GADS_ERR2')
      Y_TREE_GADS_POS_ERR3 = Widget_Tree(Y_TREENODE_GADS_POS_ERR, UNAME='Y_TREE_GADS_POS_ERR3'  $
      ,VALUE='Pos_Err_3', UVALUE='GADS_ERR3')

    Y_TREENODE_GADS_ANGL_VEL = Widget_Tree(Y_TREENODE_GADS_3, UNAME='Y_TREENODE_ANGL_VEL'  $
        ,/FOLDER ,VALUE='Body_Rate')
      Y_TREE_GADS_BR1 = Widget_Tree(Y_TREENODE_GADS_ANGL_VEL, UNAME='Y_TREE_GADS_BR1'  $
      ,VALUE='Body_Rate_1', UVALUE='GADS_BR1')
      Y_TREE_GADS_BR2 = Widget_Tree(Y_TREENODE_GADS_ANGL_VEL, UNAME='Y_TREE_GADS_BR2'  $
      ,VALUE='Body_Rate_2', UVALUE='GADS_BR2')
      Y_TREE_GADS_BR3 = Widget_Tree(Y_TREENODE_GADS_ANGL_VEL, UNAME='Y_TREE_GADS_BR3'  $
      ,VALUE='Body_Rate_3', UVALUE='GADS_BR3')

    Y_TREE_GADS_BRA = Widget_Tree(Y_TREENODE_GADS_3, UNAME='Y_TREE_GADS_RA'  $
      ,VALUE='RA', UVALUE='GADS_RA')
    Y_TREE_GADS_BDEC = Widget_Tree(Y_TREENODE_GADS_3, UNAME='Y_TREE_GADS_DEC'  $
      ,VALUE='DEC', UVALUE='GADS_DEC')
    Y_TREE_GADS_ANG = Widget_Tree(Y_TREENODE_GADS_3, UNAME='Y_TREE_GADS_RANG'  $
      ,VALUE='Roll', UVALUE='GADS_RANG')

    Y_TREE_GADS_DBRA = Widget_Tree(Y_TREENODE_GADS_3, UNAME='Y_TREE_GADS_DRA'  $
      ,VALUE='d_RA', UVALUE='GADS_DRA')
    Y_TREE_GADS_DBDEC = Widget_Tree(Y_TREENODE_GADS_3, UNAME='Y_TREE_GADS_DDEC'  $
      ,VALUE='d_DEC', UVALUE='GADS_DDEC')
    Y_TREE_GADS_DANG = Widget_Tree(Y_TREENODE_GADS_3, UNAME='Y_TREE_GADS_DRANG'  $
      ,VALUE='d_Roll', UVALUE='GADS_DRANG')

    Y_TREE_GADS_EL_SOL = Widget_Tree(Y_TREENODE_GADS_3, UNAME='Y_TREE_GADS_EL_SOL'  $
      ,VALUE='EL_Sol', UVALUE='GADS_EL_SOL')
    Y_TREE_GADS_AA_SOL = Widget_Tree(Y_TREENODE_GADS_3, UNAME='Y_TREE_GADS_AA_SOL'  $
      ,VALUE='AA_Sol', UVALUE='GADS_AA_SOL')
    Y_TREE_GADS_AA_EAR = Widget_Tree(Y_TREENODE_GADS_3, UNAME='Y_TREE_GADS_AA_EAR'  $
      ,VALUE='AA_Ear', UVALUE='GADS_AA_EAR')
    Y_TREE_GADS_AA_LUN = Widget_Tree(Y_TREENODE_GADS_3, UNAME='Y_TREE_GADS_AA_LUN'  $
      ,VALUE='AA_Lun', UVALUE='GADS_AA_LUN')

    Y_TREE_GADS_CRS_OFF = Widget_Tree(Y_TREENODE_GADS_3, UNAME='Y_TREE_GADS_CRS_OFF'  $
      ,VALUE='Crs_Off', UVALUE='GADS_CRS_OFF')

  ; (4) Quality
  Y_TREENODE_GADS_4 = Widget_Tree(Y_TREENODE_GADS, UNAME='Y_TREENODE_GADS_4'  $
      ,/FOLDER ,VALUE='Quality')
    Y_TREE_GADS_QUAL_INTP = Widget_Tree(Y_TREENODE_GADS_4, UNAME='Y_TREE_GADS_QUAL_INTP'  $
      ,VALUE='Interpolated', UVALUE='GADS_QUAL_INTP')

  Widget_Control, parentID, /UPDATE

end