; $Id: save_vdet.pro,v 1.10 2007/07/05 10:40:04 makiuti Exp $
;+
; NAME:
;       SAVE_VDET
;
; PURPOSE:
;       The purpose of this procedure is to extract the FIS detector
;       data DET or FLUX (of all pixels) and save them as a text format file.
;       This procedure is called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.jaxa.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       save_vdet, GROUP_LEADER=Event.top
;
; ;OTHER PROGRAMS NEEDED:
; ;  The following programs are required to run this one:
; ;
; ;    extrac_data_by_time.pro (kawamura_lib)
; ;
; MODIFICATION HISTORY:
;        Written by: Makiuti,S.
;        06/02/02 Last modified
;-

;-----------------------------------------------------------------
; Event CallBack Procedures
;-----------------------------------------------------------------
;
; Empty stub procedure used for autoloading.
;
pro save_vdet_eventcb
end

;-----------------------------------------------------------------
pro SaveBtn, Event
COMMON ShareFISv, tsd

  ; Get ID of DATA selector in slider0
  WID_SLIDER_0 = widget_info(Event.top, find_by_uname='WID_SLIDER_0')
  ; Get value selected in slider now
  widget_control, WID_SLIDER_0, get_value=slidevalue_start

  ; Get ID of DATA selector in slider2
  WID_SLIDER_2 = widget_info(Event.top, find_by_uname='WID_SLIDER_2')
  ; Get value selected in slider now
  widget_control, WID_SLIDER_2, get_value=slidevalue_end

  ;print, slidevalue_start, slidevalue_end

  IF (slidevalue_start GT slidevalue_end) THEN BEGIN
    Result = DIALOG_MESSAGE("End sample number must be more than Start sample number",$
               DIALOG_PARENT=Event.top, /ERROR)
  ENDIF ELSE BEGIN

    ; Find data type (DET or FLUX)
    Sect_FLUX_Btn = widget_info(Event.top, find_by_uname='Sect_FLUX_Btn')
    data_type = widget_info(Sect_FLUX_Btn, /BUTTON_SET)

    ;ex_vdet = extrac_data_by_time(tsd, slidevalue_start+1, slidevalue_end+1)
    det_data = (data_type eq 0) ? tsd->get_value_tsd('FIS_OBS', 'DET'):tsd->get_value_tsd('FIS_OBS', 'FLUX')
    ex_data = det_data[*, slidevalue_start: slidevalue_end]

    text_file = DIALOG_PICKFILE(FILTER = '*.txt', /WRITE,$
                  DIALOG_PARENT = Event.top)

    ; If any file was not selected
    IF (text_file EQ "") THEN Retall

    openw, lun, text_file, /get_lun
    printf, lun, ex_data, FORMAT='('+string(n_elements(ex_data[*,0]))+'e12.3)'
    free_lun, lun
    WIDGET_CONTROL, Event.top, /DESTROY
  ENDELSE

end

;-----------------------------------------------------------------
pro CancelBtn, Event
  WIDGET_CONTROL, Event.top, /DESTROY
end

;-----------------------------------------------------------------
pro WID_BASE_SV_event, Event

  wTarget = (widget_info(Event.id,/NAME) eq 'TREE' ?  $
      widget_info(Event.id, /tree_root) : event.id)
  wWidget =  Event.top

  case wTarget of

    Widget_Info(wWidget, FIND_BY_UNAME='WID_BUTTON_0'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        SaveBtn, Event
    end
    Widget_Info(wWidget, FIND_BY_UNAME='WID_BUTTON_1'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        CancelBtn, Event
    end
    else:
  endcase

end

pro WID_BASE_SV, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
COMMON ShareFISv, tsd

  ;Resolve_Routine, 'save_vdet_eventcb',/COMPILE_FULL_FILE  ; Load event callback routines

  device, get_screen_size=screen_size
  xoffset = screen_size[0]/4
  yoffset = screen_size[1]/3

  WID_BASE_SV = Widget_Base( GROUP_LEADER=wGroup, UNAME='WID_BASE_SV'  $
      ,XOFFSET=xoffset ,YOFFSET=yoffset ,TITLE='Extr. & Save DET and FLUX' ,SPACE=3 ,XPAD=3  $
      ,YPAD=3 ,COLUMN=1)

  ;
  WID_BASE_1 = Widget_Base(WID_BASE_SV, UNAME='WID_BASE_1' ,FRAME=1  $
      ,XOFFSET=3 ,YOFFSET=3 ,SCR_XSIZE=512 ,SCR_YSIZE=65 ,TITLE='IDL'  $
      ,SPACE=3 ,XPAD=7 ,YPAD=3 ,COLUMN=1)

  WID_LABEL_0 = Widget_Label(WID_BASE_1, UNAME='WID_LABEL_0'  $
      ,XOFFSET=98 ,YOFFSET=3 ,/ALIGN_CENTER ,VALUE='Start Sample Num.')

  WID_SLIDER_0 = Widget_Slider(WID_BASE_1, UNAME='WID_SLIDER_0'  $
      ,XOFFSET=7 ,YOFFSET=20 ,SCR_XSIZE=250 ,SCR_YSIZE=33)

  ;
  WID_BASE_2 = Widget_Base(WID_BASE_SV, UNAME='WID_BASE_2' ,FRAME=1  $
      ,XOFFSET=3 ,YOFFSET=71 ,SCR_XSIZE=512 ,SCR_YSIZE=65  $
      ,TITLE='IDL' ,SPACE=3 ,XPAD=7 ,YPAD=3 ,COLUMN=1)

  WID_LABEL_2 = Widget_Label(WID_BASE_2, UNAME='WID_LABEL_2'  $
      ,XOFFSET=101 ,YOFFSET=3 ,/ALIGN_CENTER ,VALUE='End Sample Num.')

  WID_SLIDER_2 = Widget_Slider(WID_BASE_2, UNAME='WID_SLIDER_2'  $
      ,XOFFSET=7 ,YOFFSET=20 ,SCR_XSIZE=250 ,SCR_YSIZE=33)

  ;
  WID_BASE_3 = Widget_Base(WID_BASE_SV, UNAME='WID_BASE_3' ,XOFFSET=50  $
      ,YOFFSET=139 ,/ALIGN_CENTER ,TITLE='IDL' ,SPACE=5 ,XPAD=3  $
      ,YPAD=3 ,ROW=1 ,/GRID_LAYOUT)

  ; DET or FLUX, Select Button
  Slct_Btn_Base = Widget_Base(WID_BASE_3, /frame, /row, /EXCLUSIVE)
  Sect_DET_Btn = Widget_Button(Slct_Btn_Base, UNAME='Sect_DET_Btn', Value='DET')
  Sect_FLUX_Btn = Widget_Button(Slct_Btn_Base, UNAME='Sect_FLUX_Btn', Value='FLUX')
  Widget_Control, Sect_DET_Btn, /Set_Button

  ; Save and Cancel Button
  Btns_Base = Widget_Base(WID_BASE_3, /row)
  WID_BUTTON_0 = Widget_Button(Btns_Base, UNAME='WID_BUTTON_0'  $
      ,XOFFSET=3 ,YOFFSET=3 ,SCR_XSIZE=55 ,SCR_YSIZE=23  $
      ,/ALIGN_CENTER ,VALUE='Save', Tooltip='Save as a text file')

  WID_BUTTON_1 = Widget_Button(Btns_Base, UNAME='WID_BUTTON_1'  $
      ,XOFFSET=68 ,YOFFSET=3 ,SCR_XSIZE=55 ,SCR_YSIZE=23  $
      ,/ALIGN_CENTER ,VALUE='Cancel', Tooltip='Cancel')


  ; Added code manually START
  ; Make TSD object for test
  ;tsd=Obj_New('tsd', INFILE='C:\RSI\IDL56\sample_fits\fis20020927_112819_03_lw.fits')

  ;## Set Values of Slidebars ##
  ; Make array of detector signals
  vdet=tsd->get_value_tsd('FIS_OBS', 'DET')
  ; Find number of array samples
  vdet_smp_max = n_elements(vdet[0,*])

  ; Set slider max to the number of samples
  widget_control, WID_SLIDER_0, set_slider_max=vdet_smp_max-1
  widget_control, WID_SLIDER_2, set_slider_max=vdet_smp_max-1
  widget_control, WID_SLIDER_2, set_value=vdet_smp_max-1
  ; Added code manually END

  Widget_Control, /REALIZE, WID_BASE_SV

  XManager, 'WID_BASE_SV', WID_BASE_SV, /NO_BLOCK

end

;
; Empty stub procedure used for autoloading.
;
pro save_vdet, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
  WID_BASE_SV, GROUP_LEADER=wGroup, _EXTRA=_VWBExtra_
end
