;+
; NAME:
;       STAT_PLOT
;
; PURPOSE:
;       The purpose of this program is to calculate some statistics
;       of time series data.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.ac.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;       Widgets program
;
; CALLING SEQUENCE:
;        This program is usually called from zplot_alt.pro
;        stat_plot, info, GROUP_LEADER=parent
;
; MODIFY
;       Remodeled version of ZPLOT_ALT
;       03/05/09 make this program
;       03/08/13 last modified
;
;#######
; below is the original header of ZPLOT
;#######
; PURPOSE:
;
;       The purpose of this program is to display a line plot in a resizeable
;       graphics window which can be zoomed by drawing an "arrow box" on top
;       of it. To return to the un-zoomed plot, click and release anywhere in the window.
;
; AUTHOR:
;
;   FANNING SOFTWARE CONSULTING
;   David Fanning, Ph.D.
;   1645 Sheely Drive
;   Fort Collins, CO 80526 USA
;   Phone: 970-221-0438
;   E-mail: davidf@dfanning.com
;   Coyote's Guide to IDL Programming: http://www.dfanning.com/
;
; CATEGORY:
;
;      Widgets.
;
; CALLING SEQUENCE:
;
;      ZPlot, x, y
;
; OPTIONAL INPUTS:
;
;      x: If only one positional parameter, this is assumed to be the
;         dependent data. If there are two positional parameters, this
;         is assumed to be the independent data in accordance with the
;         PLOT command.
;
;      y: The dependent data, if the X parameter is present.
;
; KEYWORD PARAMETERS:
;
;       Any valid PLOT keyword can be used with this program. In additon,
;       the following keywords are defined specifically.
;
;       GROUP_LEADER: This keyword is used to assign a group leader to this
;                 program. This program will be destroyed when the group
;                 leader is destroyed. Use this keyword if you are calling
;                 ZIMAGE from another widget program.
;
;       XLOG: Set this keyword if you wish the X axis to be logarthmic style.
;
;       YLOG: Set this keyword if you wish the Y axis to be logarthmic style.
;
;       ZOOM_XSIZE: The initial X size of the plot window. Default is 400 pixels.
;
;       ZOOM_YSIZE: The initial Y size of the plot window. Default is 400 pixels.
;
; COMMON BLOCKS:
;
;       None.
;
; SIDE EFFECTS:
;
;       Drawing colors are loaded into the color table.
;
; RESTRICTIONS:
;
;       Zooms only along the X axis.
;
; PROCEDURE:
;
;       Click and drag the cursor to create an "arrow box". The plot
;       is zoomed into the X coordinates of the box, when released.
;       To restore unzoomed plot, click and release anywhere in the
;       window.
;
; EXAMPLE:
;
;        To display an plot you can zoom into, type:
;
;        ZPLOT
;
; MODIFICATION HISTORY:
;
;        Written by: David Fanning, 15 February 2000.
;        Modified the original rubberband box to be an "arrow box". 1 April 2000. DWF.
;        Added the ability to produce log style plots. 20 November 2001. DWF.
;-
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; Copyright  2000-2001 Fanning Software Consulting
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################



PRO StatPLOT_CLEANUP, tlb

   ; The purpose of this module is to delete the pixmap window
   ; and perform other cleanup chores when the program StatPLOT is
   ; destroyed.

Widget_Control, tlb, Get_UValue=info
IF N_Elements(info) NE 0 THEN BEGIN
   WDelete, info.pixIndex
   Ptr_Free, info.extraKeywords
ENDIF
END ; of StatPLOT_CLEANUP **********************************************************

;-----------------------------------------------------------------
pro StatClsBtn, Event
  WIDGET_CONTROL, Event.top, /DESTROY
end
;-----------------------------------------------------------------

PRO StatPLOT_RESIZE, event

; This event handler reponds to TLB re-size events.
 Widget_Control, event.top, Get_UValue=info, /No_Copy

;#####Start of altered code#####
  wWidget = Event.top
  wTarget = Event.id
  ;help, event, /structure

  case wTarget of

    ; ##Close Plot Window##
    Widget_Info(wWidget, FIND_BY_UNAME='ClsBtn'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then $
        StatClsBtn, Event
      end

    ; ##Plot with Symbols##
    Widget_Info(wWidget, FIND_BY_UNAME='SymbBtn'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then begin

        ; Switch psym
        info.psym = (info.psym EQ 0) ? -7:0

        ; Destroy old pixmap and create a new one.
        WDelete, info.pixIndex
        Window, XSize=info.zxsize, YSize=info.zysize, /Free, /Pixmap
        info.pixIndex = !D. Window

        ;  Set up colors for line plot.
        backColor = GetColor('Dark Green', !D.Table_Size-2)
        dataColor = GetColor('yellow', !D.Table_Size-3)
        axisColor = GetColor('green', !D.Table_Size-4)
;#
        dataColor2 = GetColor('pink', !D.Table_Size-5)
        dataColor3 = GetColor('aqua', !D.Table_Size-6)
        dataColor4 = GetColor('magenta', !D.Table_Size-7)
        dataColor5 = GetColor('gray', !D.Table_Size-8)

        ; Draw the plot in both windows.
        Plot, info.indep, info.dep, XRange=info.xrange, XStyle=info.xstyle, YRange=info.yrange, $
           YStyle=info.ystyle, XLog=info.xlog, YLog=info.ylog, $
           Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords
        OPlot, info.indep, info.dep, Color=dataColor, psym=info.psym

;#
        title_pos = CONVERT_COORD([0.85, 1], /NORMAL, /TO_DATA)
        IF n_elements(info.dep2) EQ n_elements(info.dep) THEN BEGIN
          IF (info.dbly EQ 1) THEN BEGIN
            Plot, info.indep, info.dep, XRange=info.xrange, XStyle=info.xstyle, $
              YRange=info.yrange, Ystyle=4, xmargin=[10,10], $
              XLog=info.xlog, YLog=info.ylog, $
              Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords
            axis, yaxis=0, yrange=[Min(info.dep),Max(info.dep)], ytitle=info.y1title, Color=dataColor, /save
            OPlot, info.indep, info.dep, Color=dataColor, psym = (info.psym NE 0) ? -7:0
            axis, yaxis=1, yrange=[Min(info.dep2),Max(info.dep2)], ytitle=info.y2title, Color=dataColor2, /save
            OPlot, info.indep, info.dep2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
          ENDIF ELSE BEGIN
            OPlot, info.indep, info.dep2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
            XYOUTS, title_pos[0], Max(info.dep)*0.9,  info.y1title, Color=dataColor
            XYOUTS, title_pos[0], Max(info.dep2)*0.9, info.y2title, Color=dataColor2
          ENDELSE
        ENDIF
        IF n_elements(info.dep3) EQ n_elements(info.dep) THEN BEGIN
            OPlot, info.indep, info.dep3, Color=dataColor3, psym = (info.psym NE 0) ? -4:0
            XYOUTS, title_pos[0], Max(info.dep3)*0.9, info.y3title, Color=dataColor3
        ENDIF
        IF n_elements(info.dep4) EQ n_elements(info.dep) THEN BEGIN
            OPlot, info.indep, info.dep4, Color=dataColor4, psym = (info.psym NE 0) ? -2:0
            XYOUTS, title_pos[0], Max(info.dep4)*0.9, info.y4title, Color=dataColor4
        ENDIF
        IF n_elements(info.dep5) EQ n_elements(info.dep) THEN BEGIN
            OPlot, info.indep, info.dep5, Color=dataColor5, psym = (info.psym NE 0) ? -6:0
            XYOUTS, title_pos[0], Max(info.dep5)*0.9, info.y5title, Color=dataColor5
        ENDIF

        WSet, info.drawIndex
        Plot, info.indep, info.dep, XRange=info.xrange, XStyle=info.xstyle, YRange=info.yrange, $
           YStyle=info.ystyle, XLog=info.xlog, YLog=info.ylog, $
           Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords
        OPlot, info.indep, info.dep, Color=dataColor, psym=info.psym
;#
        title_pos = CONVERT_COORD([0.85, 1], /NORMAL, /TO_DATA)
        IF n_elements(info.dep2) EQ n_elements(info.dep) THEN BEGIN
          IF (info.dbly EQ 1) THEN BEGIN
            Plot, info.indep, info.dep, XRange=info.xrange, XStyle=info.xstyle, $
              YRange=info.yrange, Ystyle=4, xmargin=[10,10], $
              XLog=info.xlog, YLog=info.ylog, $
              Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords
            axis, yaxis=0, yrange=[Min(info.dep),Max(info.dep)], ytitle=info.y1title, Color=dataColor, /save
            OPlot, info.indep, info.dep, Color=dataColor, psym = (info.psym NE 0) ? -7:0
            axis, yaxis=1, yrange=[Min(info.dep2),Max(info.dep2)], ytitle=info.y2title, Color=dataColor2, /save
            OPlot, info.indep, info.dep2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
          ENDIF ELSE BEGIN
            OPlot, info.indep, info.dep2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
            XYOUTS, title_pos[0], Max(info.dep)*0.9,  info.y1title, Color=dataColor
            XYOUTS, title_pos[0], Max(info.dep2)*0.9, info.y2title, Color=dataColor2
          ENDELSE
        ENDIF
        IF n_elements(info.dep3) EQ n_elements(info.dep) THEN BEGIN
            OPlot, info.indep, info.dep3, Color=dataColor3, psym = (info.psym NE 0) ? -4:0
            XYOUTS, title_pos[0], Max(info.dep3)*0.9, info.y3title, Color=dataColor3
        ENDIF
        IF n_elements(info.dep4) EQ n_elements(info.dep) THEN BEGIN
            OPlot, info.indep, info.dep4, Color=dataColor4, psym = (info.psym NE 0) ? -2:0
            XYOUTS, title_pos[0], Max(info.dep4)*0.9, info.y4title, Color=dataColor4
        ENDIF
        IF n_elements(info.dep5) EQ n_elements(info.dep) THEN BEGIN
            OPlot, info.indep, info.dep5, Color=dataColor5, psym = (info.psym NE 0) ? -6:0
            XYOUTS, title_pos[0], Max(info.dep5)*0.9, info.y5title, Color=dataColor5
        ENDIF

        Widget_Control, event.top, Set_UValue=info, /No_Copy
      endif
    end

    ; ##Double Y Plot##
    Widget_Info(wWidget, FIND_BY_UNAME='DblYBtn'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then begin

        ; Switch Double Y Flag
        IF info.dbly EQ 0 THEN info.dbly=1 $
        ELSE IF info.dbly NE 0 THEN info.dbly=0
        ;print, 'DoubleY Flag', info.dbly;#####

        ; Destroy old pixmap and create a new one.
        WDelete, info.pixIndex
        Window, XSize=info.zxsize, YSize=info.zysize, /Free, /Pixmap
        info.pixIndex = !D. Window

        ; Set up colors for line plot.
        backColor = GetColor('Dark Green', !D.Table_Size-2)
        dataColor = GetColor('yellow', !D.Table_Size-3)
        axisColor = GetColor('green', !D.Table_Size-4)
;#
        dataColor2 = GetColor('pink', !D.Table_Size-5)

        ; Back to the initial Y range
        IF info.dbly NE 1 THEN BEGIN
          info.yrange = info.yrange_def
          info.ystyle = 1
        ENDIF

        ; Draw the plot in both windows.
        Plot, info.indep, info.dep, XRange=info.xrange, XStyle=info.xstyle, YRange=info.yrange, $
           YStyle=info.ystyle, XLog=info.xlog, YLog=info.ylog, $
           Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords
        OPlot, info.indep, info.dep, Color=dataColor, psym = (info.psym NE 0) ? -7:0

;#
        title_pos = CONVERT_COORD([0.85, 1], /NORMAL, /TO_DATA)
        IF n_elements(info.dep2) EQ n_elements(info.dep) THEN BEGIN
          IF (info.dbly EQ 1) THEN BEGIN
            Plot, info.indep, info.dep, XRange=info.xrange, XStyle=info.xstyle, $
              YRange=info.yrange, Ystyle=4, xmargin=[10,10], $
              XLog=info.xlog, YLog=info.ylog, $
              Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords
            axis, yaxis=0, yrange=[Min(info.dep),Max(info.dep)], ytitle=info.y1title, Color=dataColor, /save
            OPlot, info.indep, info.dep, Color=dataColor, psym = (info.psym NE 0) ? -7:0
            axis, yaxis=1, yrange=[Min(info.dep2),Max(info.dep2)], ytitle=info.y2title, Color=dataColor2, /save
            OPlot, info.indep, info.dep2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
          ENDIF ELSE BEGIN
            OPlot, info.indep, info.dep2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
            XYOUTS, title_pos[0], Max(info.dep)*0.9,  info.y1title, Color=dataColor
            XYOUTS, title_pos[0], Max(info.dep2)*0.9, info.y2title, Color=dataColor2
          ENDELSE
        ENDIF

        WSet, info.drawIndex
        Plot, info.indep, info.dep, XRange=info.xrange, XStyle=info.xstyle, YRange=info.yrange, $
           YStyle=info.ystyle, XLog=info.xlog, YLog=info.ylog, $
           Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords
        OPlot, info.indep, info.dep, Color=dataColor, psym = (info.psym NE 0) ? -7:0
;#
        title_pos = CONVERT_COORD([0.85, 1], /NORMAL, /TO_DATA)
        IF n_elements(info.dep2) EQ n_elements(info.dep) THEN BEGIN
          IF (info.dbly EQ 1) THEN BEGIN
            Plot, info.indep, info.dep, XRange=info.xrange, XStyle=info.xstyle, $
              YRange=info.yrange, Ystyle=4, xmargin=[10,10], $
              XLog=info.xlog, YLog=info.ylog, $
              Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords
            axis, yaxis=0, yrange=[Min(info.dep),Max(info.dep)], ytitle=info.y1title, Color=dataColor, /save
            OPlot, info.indep, info.dep, Color=dataColor, psym = (info.psym NE 0) ? -7:0
            axis, yaxis=1, yrange=[Min(info.dep2),Max(info.dep2)], ytitle=info.y2title, Color=dataColor2, /save
            OPlot, info.indep, info.dep2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
          ENDIF ELSE BEGIN
            OPlot, info.indep, info.dep2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
            XYOUTS, title_pos[0], Max(info.dep)*0.9,  info.y1title, Color=dataColor
            XYOUTS, title_pos[0], Max(info.dep2)*0.9, info.y2title, Color=dataColor2
          ENDELSE
        ENDIF

        ; Update the plot system variables.
        info.x = !X
        info.y = !Y
        info.p = !P

        Widget_Control, event.top, Set_UValue=info, /No_Copy
      endif
    end


    else: BEGIN;#####End of altered code#####

   ; Update window sizes.

IF (!d.name eq 'X') THEN $
  event.x = (event.x - 355 > 200) $ ; adjust for a part of STAT display area (Linux, etc.)
ELSE $ ; !d.name eq 'WIN'
  event.x = (event.x - 318 > 200) ; adjust for a part of STAT display area (Win)

event.y = (event.y -  50 > 200) ; adjust for a part of Button area

info.zxsize = event.x
info.zysize = event.y

   ; Destroy old pixmap and create a new one.

WDelete, info.pixIndex
Window, XSize=event.x, YSize=event.y, /Free, /Pixmap
info.pixIndex = !D. Window

   ; Resize draw widget.

Widget_Control, info.drawID, Draw_XSize=(event.x > 200), Draw_YSize=(event.y > 150)

   ;  Set up colors for line plot.

backColor = GetColor('Dark Green', !D.Table_Size-2)
dataColor = GetColor('yellow', !D.Table_Size-3)
axisColor = GetColor('green', !D.Table_Size-4)
;#
dataColor2 = GetColor('pink', !D.Table_Size-5)
dataColor3 = GetColor('aqua', !D.Table_Size-6)
dataColor4 = GetColor('magenta', !D.Table_Size-7)
dataColor5 = GetColor('gray', !D.Table_Size-8)

   ; Draw the plot in both windows.

Plot, info.indep, info.dep, XRange=info.xrange, XStyle=info.xstyle, YRange=info.yrange, $
   YStyle=info.ystyle, XLog=info.xlog, YLog=info.ylog, $
   Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords
OPlot, info.indep, info.dep, Color=dataColor, psym = (info.psym NE 0) ? -7:0

;#
title_pos = CONVERT_COORD([0.85, 1], /NORMAL, /TO_DATA)
IF n_elements(info.dep2) EQ n_elements(info.dep) THEN BEGIN
  IF (info.dbly EQ 1) THEN BEGIN
    Plot, info.indep, info.dep, XRange=info.xrange, XStyle=info.xstyle, $
      YRange=info.yrange, Ystyle=4, xmargin=[10,10], $
      XLog=info.xlog, YLog=info.ylog, $
      Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords
    axis, yaxis=0, yrange=[Min(info.dep),Max(info.dep)], ytitle=info.y1title, Color=dataColor, /save
    OPlot, info.indep, info.dep, Color=dataColor, psym = (info.psym NE 0) ? -7:0
    axis, yaxis=1, yrange=[Min(info.dep2),Max(info.dep2)], ytitle=info.y2title, Color=dataColor2, /save
    OPlot, info.indep, info.dep2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
  ENDIF ELSE BEGIN
    OPlot, info.indep, info.dep2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
    XYOUTS, title_pos[0], Max(info.dep)*0.9,  info.y1title, Color=dataColor
    XYOUTS, title_pos[0], Max(info.dep2)*0.9, info.y2title, Color=dataColor2
  ENDELSE
ENDIF
IF n_elements(info.dep3) EQ n_elements(info.dep) THEN BEGIN
    OPlot, info.indep, info.dep3, Color=dataColor3, psym = (info.psym NE 0) ? -4:0
    XYOUTS, title_pos[0], Max(info.dep3)*0.9, info.y3title, Color=dataColor3
ENDIF
IF n_elements(info.dep4) EQ n_elements(info.dep) THEN BEGIN
    OPlot, info.indep, info.dep4, Color=dataColor4, psym = (info.psym NE 0) ? -2:0
    XYOUTS, title_pos[0], Max(info.dep4)*0.9, info.y4title, Color=dataColor4
ENDIF
IF n_elements(info.dep5) EQ n_elements(info.dep) THEN BEGIN
    OPlot, info.indep, info.dep5, Color=dataColor5, psym = (info.psym NE 0) ? -6:0
    XYOUTS, title_pos[0], Max(info.dep5)*0.9, info.y5title, Color=dataColor5
ENDIF

WSet, info.drawIndex
Plot, info.indep, info.dep, XRange=info.xrange, XStyle=info.xstyle, YRange=info.yrange, $
   YStyle=info.ystyle, XLog=info.xlog, YLog=info.ylog, $
   Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords
OPlot, info.indep, info.dep, Color=dataColor, psym = (info.psym NE 0) ? -7:0

;#
title_pos = CONVERT_COORD([0.85, 1], /NORMAL, /TO_DATA)
IF n_elements(info.dep2) EQ n_elements(info.dep) THEN BEGIN
  IF (info.dbly EQ 1) THEN BEGIN
    Plot, info.indep, info.dep, XRange=info.xrange, XStyle=info.xstyle, $
      YRange=info.yrange, Ystyle=4, xmargin=[10,10], $
      XLog=info.xlog, YLog=info.ylog, $
      Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords
    axis, yaxis=0, yrange=[Min(info.dep),Max(info.dep)], ytitle=info.y1title, Color=dataColor, /save
    OPlot, info.indep, info.dep, Color=dataColor, psym = (info.psym NE 0) ? -7:0
    axis, yaxis=1, yrange=[Min(info.dep2),Max(info.dep2)], ytitle=info.y2title, Color=dataColor2, /save
    OPlot, info.indep, info.dep2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
  ENDIF ELSE BEGIN
    OPlot, info.indep, info.dep2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
    XYOUTS, title_pos[0], Max(info.dep)*0.9,  info.y1title, Color=dataColor
    XYOUTS, title_pos[0], Max(info.dep2)*0.9, info.y2title, Color=dataColor2
  ENDELSE
ENDIF
IF n_elements(info.dep3) EQ n_elements(info.dep) THEN BEGIN
    OPlot, info.indep, info.dep3, Color=dataColor3, psym = (info.psym NE 0) ? -4:0
    XYOUTS, title_pos[0], Max(info.dep3)*0.9, info.y3title, Color=dataColor3
ENDIF
IF n_elements(info.dep4) EQ n_elements(info.dep) THEN BEGIN
    OPlot, info.indep, info.dep4, Color=dataColor4, psym = (info.psym NE 0) ? -2:0
    XYOUTS, title_pos[0], Max(info.dep4)*0.9, info.y4title, Color=dataColor4
ENDIF
IF n_elements(info.dep5) EQ n_elements(info.dep) THEN BEGIN
    OPlot, info.indep, info.dep5, Color=dataColor5, psym = (info.psym NE 0) ? -6:0
    XYOUTS, title_pos[0], Max(info.dep5)*0.9, info.y5title, Color=dataColor5
ENDIF

   ; Update system parameters.

info.x = !X
info.y = !Y
info.p = !P

Widget_Control, event.top, Set_UValue=info, /No_Copy

    end
  endcase

END ; of StatPLOT_RESIZE **********************************************************


PRO StatPLOT_PROCESS_EVENTS, event

   ; This event handler ONLY responds to button down events from the
   ; draw widget. If it gets a DOWN event, it does three things: (1) sets
   ; the static and dynamic corners of the arrow box, (2) changes the
   ; event handler for the draw widget to StatPLOT_DRAWBOX and turns on MOTION
   ; events, and (3) update the user's color table vectors.

possibleEventTypes = [ 'DOWN', 'UP', 'MOTION', 'SCROLL' ]
thisEvent = possibleEventTypes(event.type)
IF thisEvent NE 'DOWN' THEN RETURN

    ; Must be DOWN event to get here, so get info structure.

 Widget_Control, event.top, Get_UValue=info, /No_Copy

    ; Set the static corners of the arrow box to current
    ; cursor location.

 info.xs = event.x
 info.ys = event.y

    ; Change the event handler for the draw widget and turn MOTION
    ; events ON.

 Widget_Control, event.id, Event_Pro='StatPLOT_DRAWBOX', $
    Draw_Motion_Events=1

   ; Put the info structure back into its storage location.

Widget_Control, event.top, Set_UValue=info, /No_Copy
END ; of StatPLOT_PROCESS_EVENTS *****************************************************



PRO StatPLOT_DRAWBOX, event

   ; This event handler continuously draws and erases the arrow box until it
   ; receives an UP event from the draw widget. Then it turns draw widget motion
   ; events OFF and changes the event handler for the draw widget back to
   ; StatPLOT_PROCESS_EVENTS.

   ; Get the info structure out of the top-level base.

Widget_Control, event.top, Get_UValue=info, /No_Copy

   ;  Set up colors for line plot.

backColor = GetColor('Dark Green', !D.Table_Size-2)
dataColor = GetColor('yellow', !D.Table_Size-3)
axisColor = GetColor('green', !D.Table_Size-4)
boxColor = GetColor('goldenrod', !D.Table_Size-5)
;#
dataColor2 = GetColor('pink', !D.Table_Size-6)
dataColor3 = GetColor('aqua', !D.Table_Size-7)
dataColor4 = GetColor('magenta', !D.Table_Size-8)
dataColor5 = GetColor('gray', !D.Table_Size-9)

   ; What type of an event is this?

possibleEventTypes = [ 'DOWN', 'UP', 'MOTION', 'SCROLL' ]
thisEvent = possibleEventTypes(event.type)

IF thisEvent EQ 'UP' THEN BEGIN

      ; If this is an UP event, you need to erase the zoombox, restore
      ; the user's color table, turn motion events OFF, set the
      ; draw widget's event handler back to StatPLOT_PROCESS_EVENTS, and
      ; draw the "zoomed" plot in both the draw widget and the pixmap.

      ; Erase the arrow box one final time by copying the plot from the pixmap.

   WSet, info.drawIndex
   Device, Copy = [0, 0, info.zxsize, info.zysize, 0, 0, info.pixIndex]

      ; Turn motion events off and redirect the events to StatPLOT_PROCESS_EVENTS.

    Widget_Control, event.id, Draw_Motion_Events=0, $
       Event_Pro='StatPLOT_PROCESS_EVENTS'

      ; Draw the "zoomed" plot. Start by getting the new limits to the plot
      ; (i.e., the LAST zoom box outline).

   x = [info.xs, event.x]
   y = [info.ys, event.y]

;#
   ; Make sure the x values are ordered as [min, max].
   IF info.xs GT event.x THEN x = [event.x, info.xs]

   ; Don't want exact style if we are drawing entire plot.
   ;IF info.xs EQ event.x THEN xstyle = 0 ELSE xstyle = 1
   xstyle = 1
   info.xstyle = xstyle

;#
   ; Make sure the y values are ordered as [min, max].
   IF info.ys GT event.y THEN y = [event.y, info.ys]

   ; Don't want exact style if we are drawing entire plot.
   ;IF info.ys EQ event.y THEN ystyle = 0 ELSE ystyle = 1
   ystyle = 1
   info.ystyle = ystyle


   ; Restore plot system variables.
   !X = info.x
   !Y = info.y
   !P = info.p


   ; Convert the x device coordinates to data coordinates.
   coords = Convert_Coord(x, y, /Device, /To_Data)

;#
   ; Make sure the x coordinates are within the data boundaries of the plot.
   IF info.xlog THEN BEGIN
      x1 = 10^!X.CRange(0) > coords(0,0) < 10^!X.CRange(1)
      x2 = 10^!X.CRange(0) > coords(0,1) < 10^!X.CRange(1)
   ENDIF ELSE BEGIN
      x1 = !X.CRange(0) > coords(0,0) < !X.CRange(1)
      x2 = !X.CRange(0) > coords(0,1) < !X.CRange(1)
   ENDELSE

   ; If x range is too narrow
   ;IF (x2-x1 LT (max(info.indep)-min(info.indep))/(n_elements(info.indep)-1) ) THEN BEGIN
   ;  x1 = info.xrange[0]
   ;  x2 = info.xrange[1]
   ;ENDIF

   ;info.xrange = [x1,x2]

;#
   ; Make sure the y coordinates are within the data boundaries of the plot.
   IF info.ylog THEN BEGIN
      y1 = 10^!Y.CRange(0) > coords(1,0) < 10^!Y.CRange(1)
      y2 = 10^!Y.CRange(0) > coords(1,1) < 10^!Y.CRange(1)
   ENDIF ELSE BEGIN
      y1 = !Y.CRange(0) > coords(1,0) < !Y.CRange(1)
      y2 = !Y.CRange(0) > coords(1,1) < !Y.CRange(1)
   ENDELSE

   ; If y range is too narrow
   ;IF (y2-y1 LT (max(info.dep)-min(info.dep))/(n_elements(info.dep)-1) ) THEN BEGIN
   ;  y1 = info.yrange[0]
   ;  y2 = info.yrange[1]
   ;ENDIF

   ;info.yrange = [y1,y2]

   ; Back to the initial Y range
   ;IF ((x1 EQ x2) or (y1 EQ y2)) THEN info.yrange = info.yrange_def


      ; Draw the "zoomed" plot in both the draw widget and the pixmap.

   Plot, info.indep, info.dep, XRange=info.xrange, XStyle=xstyle, YRange=info.yrange, YStyle=ystyle, $
      Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords, $
      XLog=info.xlog, YLog=info.ylog
   OPlot, info.indep, info.dep, Color=dataColor, psym = (info.psym NE 0) ? -7:0
;#
   title_pos = CONVERT_COORD([0.85, 1], /NORMAL, /TO_DATA)
   IF n_elements(info.dep2) EQ n_elements(info.dep) THEN BEGIN
     IF (info.dbly EQ 1) THEN BEGIN
       Plot, info.indep, info.dep, XRange=info.xrange, XStyle=info.xstyle, $
         YRange=info.yrange, Ystyle=4, xmargin=[10,10], $
         XLog=info.xlog, YLog=info.ylog, $
         Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords
       axis, yaxis=0, yrange=[Min(info.dep),Max(info.dep)], ytitle=info.y1title, Color=dataColor, /save
       OPlot, info.indep, info.dep, Color=dataColor, psym = (info.psym NE 0) ? -7:0
       axis, yaxis=1, yrange=[Min(info.dep2),Max(info.dep2)], ytitle=info.y2title, Color=dataColor2, /save
       OPlot, info.indep, info.dep2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
     ENDIF ELSE BEGIN
       OPlot, info.indep, info.dep2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
       XYOUTS, title_pos[0], Max(info.dep)*0.9,  info.y1title, Color=dataColor
       XYOUTS, title_pos[0], Max(info.dep2)*0.9, info.y2title, Color=dataColor2
     ENDELSE
   ENDIF
   IF n_elements(info.dep3) EQ n_elements(info.dep) THEN BEGIN
       OPlot, info.indep, info.dep3, Color=dataColor3, psym = (info.psym NE 0) ? -4:0
       XYOUTS, title_pos[0], Max(info.dep3)*0.9, info.y3title, Color=dataColor3
   ENDIF
   IF n_elements(info.dep4) EQ n_elements(info.dep) THEN BEGIN
       OPlot, info.indep, info.dep4, Color=dataColor4, psym = (info.psym NE 0) ? -2:0
       XYOUTS, title_pos[0], Max(info.dep4)*0.9, info.y4title, Color=dataColor4
   ENDIF
   IF n_elements(info.dep5) EQ n_elements(info.dep) THEN BEGIN
       OPlot, info.indep, info.dep5, Color=dataColor5, psym = (info.psym NE 0) ? -6:0
       XYOUTS, title_pos[0], Max(info.dep5)*0.9, info.y5title, Color=dataColor5
   ENDIF

   WSet, info.pixIndex
   Plot, info.indep, info.dep, XRange=info.xrange, XStyle=xstyle, YRange=info.yrange, YStyle=ystyle, $
      Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords, $
      XLog=info.xlog, YLog=info.ylog
   OPlot, info.indep, info.dep, Color=dataColor, psym = (info.psym NE 0) ? -7:0
;#
   title_pos = CONVERT_COORD([0.85, 1], /NORMAL, /TO_DATA)
   IF n_elements(info.dep2) EQ n_elements(info.dep) THEN BEGIN
     IF (info.dbly EQ 1) THEN BEGIN
       Plot, info.indep, info.dep, XRange=info.xrange, XStyle=info.xstyle, $
         YRange=info.yrange, Ystyle=4, xmargin=[10,10], $
         XLog=info.xlog, YLog=info.ylog, $
         Background=backColor, Color=axisColor, /NoData, _Extra=*info.extraKeywords
       axis, yaxis=0, yrange=[Min(info.dep),Max(info.dep)], ytitle=info.y1title, Color=dataColor, /save
       OPlot, info.indep, info.dep, Color=dataColor, psym = (info.psym NE 0) ? -7:0
       axis, yaxis=1, yrange=[Min(info.dep2),Max(info.dep2)], ytitle=info.y2title, Color=dataColor2, /save
       OPlot, info.indep, info.dep2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
     ENDIF ELSE BEGIN
       OPlot, info.indep, info.dep2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
       XYOUTS, title_pos[0], Max(info.dep)*0.9,  info.y1title, Color=dataColor
       XYOUTS, title_pos[0], Max(info.dep2)*0.9, info.y2title, Color=dataColor2
     ENDELSE
   ENDIF
   IF n_elements(info.dep3) EQ n_elements(info.dep) THEN BEGIN
       OPlot, info.indep, info.dep3, Color=dataColor3, psym = (info.psym NE 0) ? -4:0
       XYOUTS, title_pos[0], Max(info.dep3)*0.9, info.y3title, Color=dataColor3
   ENDIF
   IF n_elements(info.dep4) EQ n_elements(info.dep) THEN BEGIN
       OPlot, info.indep, info.dep4, Color=dataColor4, psym = (info.psym NE 0) ? -2:0
       XYOUTS, title_pos[0], Max(info.dep4)*0.9, info.y4title, Color=dataColor4
   ENDIF
   IF n_elements(info.dep5) EQ n_elements(info.dep) THEN BEGIN
       OPlot, info.indep, info.dep5, Color=dataColor5, psym = (info.psym NE 0) ? -6:0
       XYOUTS, title_pos[0], Max(info.dep5)*0.9, info.y5title, Color=dataColor5
   ENDIF

      ; Update the plot system variables.

   info.x = !X
   info.y = !Y
   info.p = !P

;******
  ; If Double-Y is selected, Y-range has default value
  IF (info.dbly EQ 1) THEN BEGIN
    y1 = info.yrange[0]
    y2 = info.yrange[1]
  ENDIF

; Series of #1
   ; X array must contain 2 OR more elements.
   IF (n_elements( where((info.indep GT x1) AND (info.indep LT x2) $
                        AND (info.dep GT y1) AND (info.dep LT y2)) ) GE 2) $
      THEN BEGIN
      stat_x  = info.indep[where((info.indep GT x1) AND (info.indep LT x2) $
                                AND (info.dep GT y1) AND (info.dep LT y2))]
      stat_y  = info.dep[where((info.indep GT x1) AND (info.indep LT x2) AND (info.dep GT y1) AND (info.dep LT y2))]

   ENDIF ELSE BEGIN
;     x1 = info.xrange[0]
;     x2 = info.xrange[1]
;     y1 = info.yrange[0]
;     y2 = info.yrange[1]
;
;     IF (max(info.indep) NE min(info.indep)) AND $
;          (n_elements( where((info.indep GT x1) AND (info.indep LT x2) $
;                            AND (info.dep GT y1) AND (info.dep LT y2)) ) GT 2) THEN BEGIN
;       stat_x = info.indep[where((info.indep GT x1) AND (info.indep LT x2) $
;                                AND (info.dep GT y1) AND (info.dep LT y2))]
;     ENDIF ELSE BEGIN
;       Widget_Control, event.top, Set_UValue=info, /No_Copy
;       RETURN
;     ENDELSE
   stat_x = [0,0]
   stat_y = [0,0]
   ENDELSE

; Series of #2
  IF n_elements(info.dep2) EQ n_elements(info.indep) THEN BEGIN
   IF (n_elements( where((info.indep GT x1) AND (info.indep LT x2) $
                        AND (info.dep2 GT y1) AND (info.dep2 LT y2)) ) GE 2) $
      THEN BEGIN
      stat_x2  = info.indep[where((info.indep GT x1) AND (info.indep LT x2) $
                               AND (info.dep2 GT y1) AND (info.dep2 LT y2))]
      stat_y2 = info.dep2[where((info.indep GT x1) AND (info.indep LT x2) AND (info.dep2 GT y1) AND (info.dep2 LT y2))]

   ENDIF ELSE BEGIN
     stat_x2 = [0,0]
     stat_y2 = [0,0]
   ENDELSE
  ENDIF
; Series of #3
  IF n_elements(info.dep3) EQ n_elements(info.indep) THEN BEGIN
   IF (n_elements( where((info.indep GT x1) AND (info.indep LT x2) $
                        AND (info.dep3 GT y1) AND (info.dep3 LT y2)) ) GE 2) $
      THEN BEGIN
      stat_x3  = info.indep[where((info.indep GT x1) AND (info.indep LT x2) $
                               AND (info.dep3 GT y1) AND (info.dep3 LT y2))]
      stat_y3 = info.dep3[where((info.indep GT x1) AND (info.indep LT x2) AND (info.dep3 GT y1) AND (info.dep3 LT y2))]

   ENDIF ELSE BEGIN
     stat_x3 = [0,0]
     stat_y3 = [0,0]
   ENDELSE
  ENDIF
  ; Series of #4
  IF n_elements(info.dep4) EQ n_elements(info.indep) THEN BEGIN
   IF (n_elements( where((info.indep GT x1) AND (info.indep LT x2) $
                        AND (info.dep4 GT y1) AND (info.dep4 LT y2)) ) GE 2) $
      THEN BEGIN
      stat_x4  = info.indep[where((info.indep GT x1) AND (info.indep LT x2) $
                               AND (info.dep4 GT y1) AND (info.dep4 LT y2))]
      stat_y4 = info.dep4[where((info.indep GT x1) AND (info.indep LT x2) AND (info.dep4 GT y1) AND (info.dep4 LT y2))]

   ENDIF ELSE BEGIN
     stat_x4 = [0,0]
     stat_y4 = [0,0]
   ENDELSE
  ENDIF
  ; Series of #5
  IF n_elements(info.dep5) EQ n_elements(info.indep) THEN BEGIN
   IF (n_elements( where((info.indep GT x1) AND (info.indep LT x2) $
                        AND (info.dep5 GT y1) AND (info.dep5 LT y2)) ) GE 2) $
      THEN BEGIN
      stat_x5  = info.indep[where((info.indep GT x1) AND (info.indep LT x2) $
                               AND (info.dep5 GT y1) AND (info.dep5 LT y2))]
      stat_y5 = info.dep5[where((info.indep GT x1) AND (info.indep LT x2) AND (info.dep5 GT y1) AND (info.dep5 LT y2))]

   ENDIF ELSE BEGIN
     stat_x5 = [0,0]
     stat_y5 = [0,0]
   ENDELSE
  ENDIF
;******

   ; Show statistical values

   XMIN_TEXT = widget_info(Event.top, find_by_uname='XMIN_TEXT')
   ;value = min(stat_x)
   value = x1
   value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
   WIDGET_CONTROL, XMIN_TEXT, set_value = string(value, FORMAT=value_format)

   XMAX_TEXT = widget_info(Event.top, find_by_uname='XMAX_TEXT')
   ;value = max(stat_x)
   value = x2
   value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
   WIDGET_CONTROL, XMAX_TEXT, set_value = string(value, FORMAT=value_format)

   YMIN_TEXT = widget_info(Event.top, find_by_uname='YMIN_TEXT')
   ;value = min(stat_y)
   value = (info.dbly NE 1) ? y1 : 'NaN'
   value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
   WIDGET_CONTROL, YMIN_TEXT, set_value = string(value, FORMAT=value_format)

   YMAX_TEXT = widget_info(Event.top, find_by_uname='YMAX_TEXT')
   ;value = max(stat_y)
   value = (info.dbly NE 1) ? y2 : 'NaN'
   value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
   WIDGET_CONTROL, YMAX_TEXT, set_value = string(value, FORMAT=value_format)


   Y1MEAN_TEXT = widget_info(Event.top, find_by_uname='Y1MEAN_TEXT')
   value = mean(stat_y)
   value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
   WIDGET_CONTROL, Y1MEAN_TEXT, set_value = string(value, FORMAT=value_format)

   Y1DEV_TEXT = widget_info(Event.top, find_by_uname='Y1DEV_TEXT')
   value = stddev(stat_y)
   value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
   WIDGET_CONTROL, Y1DEV_TEXT, set_value = string(value, FORMAT=value_format)

   Y1SPL_TEXT = widget_info(Event.top, find_by_uname='Y1SPL_TEXT')
   value = n_elements(stat_x)
   value = (value GT 2) ? value : 'NaN'
   IF size(value, /type) NE 7 THEN $ ; Type 7 means "string"
     value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
   WIDGET_CONTROL, Y1SPL_TEXT, set_value = string(value, FORMAT=value_format)

   IF n_elements(info.dep2) EQ n_elements(info.indep) THEN BEGIN
     Y2MEAN_TEXT = widget_info(Event.top, find_by_uname='Y2MEAN_TEXT')
     value = mean(stat_y2)
     value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
     WIDGET_CONTROL, Y2MEAN_TEXT, set_value = string(value, FORMAT=value_format)
     Y2DEV_TEXT = widget_info(Event.top, find_by_uname='Y2DEV_TEXT')
     value = stddev(stat_y2)
     value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
     WIDGET_CONTROL, Y2DEV_TEXT, set_value = string(value, FORMAT=value_format)
     Y2SPL_TEXT = widget_info(Event.top, find_by_uname='Y2SPL_TEXT')
     value = n_elements(stat_x2)
     value = (value GT 2) ? value : 'NaN'
     IF size(value, /type) NE 7 THEN $ ; Type 7 means "string"
       value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
     WIDGET_CONTROL, Y2SPL_TEXT, set_value = string(value, FORMAT=value_format)
   ENDIF
   IF n_elements(info.dep3) EQ n_elements(info.indep) THEN BEGIN
     Y3MEAN_TEXT = widget_info(Event.top, find_by_uname='Y3MEAN_TEXT')
     value = mean(stat_y3)
     value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
     WIDGET_CONTROL, Y3MEAN_TEXT, set_value = string(value, FORMAT=value_format)
     Y3DEV_TEXT = widget_info(Event.top, find_by_uname='Y3DEV_TEXT')
     value = stddev(stat_y3)
     value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
     WIDGET_CONTROL, Y3DEV_TEXT, set_value = string(value, FORMAT=value_format)
     Y3SPL_TEXT = widget_info(Event.top, find_by_uname='Y3SPL_TEXT')
     value = n_elements(stat_x3)
     value = (value GT 2) ? value : 'NaN'
     IF size(value, /type) NE 7 THEN $ ; Type 7 means "string"
       value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
     WIDGET_CONTROL, Y3SPL_TEXT, set_value = string(value, FORMAT=value_format)
   ENDIF
   IF n_elements(info.dep4) EQ n_elements(info.indep) THEN BEGIN
     Y4MEAN_TEXT = widget_info(Event.top, find_by_uname='Y4MEAN_TEXT')
     value = mean(stat_y4)
     value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
     WIDGET_CONTROL, Y4MEAN_TEXT, set_value = string(value, FORMAT=value_format)
     Y4DEV_TEXT = widget_info(Event.top, find_by_uname='Y4DEV_TEXT')
     value = stddev(stat_y4)
     value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
     WIDGET_CONTROL, Y4DEV_TEXT, set_value = string(value, FORMAT=value_format)
     Y4SPL_TEXT = widget_info(Event.top, find_by_uname='Y4SPL_TEXT')
     value = n_elements(stat_x4)
     value = (value GT 2) ? value : 'NaN'
     IF size(value, /type) NE 7 THEN $ ; Type 7 means "string"
       value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
     WIDGET_CONTROL, Y4SPL_TEXT, set_value = string(value, FORMAT=value_format)
   ENDIF
   IF n_elements(info.dep5) EQ n_elements(info.indep) THEN BEGIN
     Y5MEAN_TEXT = widget_info(Event.top, find_by_uname='Y5MEAN_TEXT')
     value = mean(stat_y5)
     value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
     WIDGET_CONTROL, Y5MEAN_TEXT, set_value = string(value, FORMAT=value_format)
     Y5DEV_TEXT = widget_info(Event.top, find_by_uname='Y5DEV_TEXT')
     value = stddev(stat_y5)
     value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
     WIDGET_CONTROL, Y5DEV_TEXT, set_value = string(value, FORMAT=value_format)
     Y5SPL_TEXT = widget_info(Event.top, find_by_uname='Y5SPL_TEXT')
     value = n_elements(stat_x5)
     value = (value GT 2) ? value : 'NaN'
     IF size(value, /type) NE 7 THEN $ ; Type 7 means "string"
       value_format = (value GT 1000) OR (value LT 0.1) ? '(E11.3)' : '(F11.3)'
     WIDGET_CONTROL, Y5SPL_TEXT, set_value = string(value, FORMAT=value_format)
   ENDIF


   ; Put the info structure back into its storage location and then, out of here!
   Widget_Control, event.top, Set_UValue=info, /No_Copy
   RETURN
ENDIF ; thisEvent = UP


   ; Most of the action in this event handler occurs here while we are waiting
   ; for an UP event to occur. As long as we don't get it, keep erasing the
   ; old zoom box and drawing a new one.

   ; Erase the old zoom box.

WSet, info.drawIndex
Device, Copy = [0, 0, info.zxsize, info.zysize, 0, 0, info.pixIndex]

   ; Update the dynamic corner of the zoom box to the current cursor location.

 info.xd = event.x
 info.yd = event.y

   ; Restore plot system variables.

!X = info.x
!Y = info.y
!P = info.p

   ; Convert the x device coordinates to data coordinates.

x = [info.xs, event.x]
y = [info.ys, event.y]
coords = Convert_Coord(x, y, /Device, /To_Data)

; Make sure the x coordinates are within the data boundaries of the plot.
IF info.xlog THEN BEGIN
   x1 = 10^!X.CRange(0) > coords(0,0) < 10^!X.CRange(1)
   x2 = 10^!X.CRange(0) > coords(0,1) < 10^!X.CRange(1)
ENDIF ELSE BEGIN
   x1 = !X.CRange(0) > coords(0,0) < !X.CRange(1)
   x2 = !X.CRange(0) > coords(0,1) < !X.CRange(1)
ENDELSE

; Make sure the y coordinates are within the data boundaries of the plot.
IF info.ylog THEN BEGIN
   y1 = 10^!Y.CRange(0) > coords(1,0) < 10^!Y.CRange(1)
   y2 = 10^!Y.CRange(0) > coords(1,1) < 10^!Y.CRange(1)
ENDIF ELSE BEGIN
   y1 = !Y.CRange(0) > coords(1,0) < !Y.CRange(1)
   y2 = !Y.CRange(0) > coords(1,1) < !Y.CRange(1)
ENDELSE

   ; Draw the arrow box.
IF info.dbly EQ 1 THEN $
   Arrow, x1, y1, x2, y1, /Data, Color=boxColor, /Solid, HSize=12 $
ELSE $
   Arrow, x1, y1, x2, y2, /Data, Color=boxColor, /Solid, HSize=12

IF info.ylog THEN BEGIN
  IF info.dbly EQ 1 THEN BEGIN
   PlotS, [x1, x1], [10^!Y.CRange[0], 10^!Y.CRange[1]], Color=boxColor
   PlotS, [x2, x2], [10^!Y.CRange[0], 10^!Y.CRange[1]], Color=boxColor
   PlotS, [x1, x2], [10^!Y.CRange[0], 10^!Y.CRange[0]], Color=boxColor
   PlotS, [x1, x2], [10^!Y.CRange[1], 10^!Y.CRange[1]], Color=boxColor
  ENDIF ELSE BEGIN
   PlotS, [x1, x1], [y1, y2], Color=boxColor
   PlotS, [x2, x2], [y1, y2], Color=boxColor
   PlotS, [x1, x2], [y1, y1], Color=boxColor
   PlotS, [x1, x2], [y2, y2], Color=boxColor
  ENDELSE
ENDIF ELSE BEGIN
  IF info.dbly EQ 1 THEN BEGIN
   PlotS, [x1, x1], [!Y.CRange[0], !Y.CRange[1]], Color=boxColor
   PlotS, [x2, x2], [!Y.CRange[0], !Y.CRange[1]], Color=boxColor
   PlotS, [x1, x2], [!Y.CRange[0], !Y.CRange[0]], Color=boxColor
   PlotS, [x1, x2], [!Y.CRange[1], !Y.CRange[1]], Color=boxColor
  ENDIF ELSE BEGIN
   PlotS, [x1, x1], [y1, y2], Color=boxColor
   PlotS, [x2, x2], [y1, y2], Color=boxColor
   PlotS, [x1, x2], [y1, y1], Color=boxColor
   PlotS, [x1, x2], [y2, y2], Color=boxColor
  ENDELSE
ENDELSE

;##### Old version, Range selected by X only. #####
;   ; Draw the arrow box.
;   Arrow, x1, y1, x2, y1, /Data, Color=boxColor, /Solid, HSize=12, thick=3
;   Arrow, x2, y1, x1, y1, /Data, Color=boxColor, /Solid, HSize=12, thick=3

;; Make sure the x values are ordered in ascending sequence.
;IF x1 GT x2 THEN BEGIN
;  temp = x2
;  x2 = x1
;  x1 = temp
;ENDIF

;IF info.ylog THEN BEGIN
;   PlotS, [!X.CRange[0], !X.CRange[0]], [10^!Y.CRange[0], 10^!Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [x1, x1], [10^!Y.CRange[0], 10^!Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [!X.CRange[0], x1], [10^!Y.CRange[0], 10^!Y.CRange[0]], Color=boxColor, thick=3
;   PlotS, [!X.CRange[0], x1], [10^!Y.CRange[1], 10^!Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [!X.CRange[0], x1], [10^!Y.CRange[0], 10^!Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [!X.CRange[0], x1], [10^!Y.CRange[1], 10^!Y.CRange[0]], Color=boxColor, thick=3
;   ;
;   PlotS, [x1, x1], [10^!Y.CRange[0], 10^!Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [x2, x2], [10^!Y.CRange[0], 10^!Y.CRange[1]], Color=boxColor, thick=3
;   ;
;   PlotS, [!X.CRange[1], !X.CRange[1]], [10^!Y.CRange[0], 10^!Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [x2, x2], [10^!Y.CRange[0], 10^!Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [x2, !X.CRange[1]], [10^!Y.CRange[0], 10^!Y.CRange[0]], Color=boxColor, thick=3
;   PlotS, [x2, !X.CRange[1]], [10^!Y.CRange[1], 10^!Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [x2, !X.CRange[1]], [10^!Y.CRange[0], 10^!Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [x2, !X.CRange[1]], [10^!Y.CRange[1], 10^!Y.CRange[0]], Color=boxColor, thick=3
;ENDIF ELSE BEGIN
;   PlotS, [!X.CRange[0], !X.CRange[0]], [!Y.CRange[0], !Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [x1, x1], [!Y.CRange[0], !Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [!X.CRange[0], x1], [!Y.CRange[0], !Y.CRange[0]], Color=boxColor, thick=3
;   PlotS, [!X.CRange[0], x1], [!Y.CRange[1], !Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [!X.CRange[0], x1], [!Y.CRange[0], !Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [!X.CRange[0], x1], [!Y.CRange[1], !Y.CRange[0]], Color=boxColor, thick=3
;   ;
;   PlotS, [x1, x1], [!Y.CRange[0], !Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [x2, x2], [!Y.CRange[0], !Y.CRange[1]], Color=boxColor, thick=3
;   ;
;   PlotS, [!X.CRange[1], !X.CRange[1]], [!Y.CRange[0], !Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [x2, x2], [!Y.CRange[0], !Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [x2, !X.CRange[1]], [!Y.CRange[0], !Y.CRange[0]], Color=boxColor, thick=3
;   PlotS, [x2, !X.CRange[1]], [!Y.CRange[1], !Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [x2, !X.CRange[1]], [!Y.CRange[0], !Y.CRange[1]], Color=boxColor, thick=3
;   PlotS, [x2, !X.CRange[1]], [!Y.CRange[1], !Y.CRange[0]], Color=boxColor, thick=3
;ENDELSE
;##### Old version #####

   ; Put the info structure back into its storage location.

Widget_Control, event.top, Set_UValue=info, /No_Copy
END ; of StatPLOT_DRAWBOX ******************************************************************


PRO STAT_PLOT, info, Group_Leader=group, _Extra=extra

   ; This procedure allows the user to "zoom" into the data plot
   ; by drawing an arrow box around the part of the data to zoom into.
   ; The arrow box will be drawn and erased by using a pixmap and
   ; the "Device Copy" technique.

   ; On an error condition, return to the main level of IDL.

On_Error, 1

   ;  Set up colors for line plot.

backColor = GetColor('Dark Green', !D.Table_Size-2)
dataColor = GetColor('yellow', !D.Table_Size-3)
axisColor = GetColor('green', !D.Table_Size-4)
;#
dataColor2 = GetColor('pink', !D.Table_Size-5)
dataColor3 = GetColor('aqua', !D.Table_Size-6)
dataColor4 = GetColor('magenta', !D.Table_Size-7)
dataColor5 = GetColor('gray', !D.Table_Size-8)


; Check for keywords. Set defaults if necessary.
dep = info.dep
indep = info.indep

zxsize = info.zxsize
zysize = info.zysize
xlog = info.xlog
ylog = info.ylog
;#
y2 = info.dep2
y3 = info.dep3
y4 = info.dep4
y5 = info.dep5
y1title = 'Y1:'+info.y1title
y2title = 'Y2:'+info.y2title
y3title = 'Y3:'+info.y3title
y4title = 'Y4:'+info.y4title
y5title = 'Y5:'+info.y5title

psym=info.psym

;#####Start of WIDGET make code#####
tlb = Widget_Base(Title='FISv STAT Window', TLB_Size_Events=1, column=1 $
                  ,XOFFSET=150 ,YOFFSET=150)

   ; The only thing in the top-level base is a draw widget.
   ; The draw widget will have its own event handler.

; Main Base - Draw window & Stat panel
main_base = Widget_Base(tlb, /row)
; Draw Window
drawID = Widget_Draw(main_base, XSize=zxsize, YSize=zysize, $
   Button_Events=1, Event_Pro='StatPLOT_PROCESS_EVENTS')

  ; ### Stat BASE #0 (Root) ###
  stat_base0 = Widget_Base(main_base, /frame, /column)

  ; ## Base for box of Ranges ##
  range_base = Widget_Base(stat_base0, /row)
    ; ## for Min ##
    rmin_base = Widget_Base(range_base, /column)
      ; X&Y Minimum
      xmin_title = Widget_Label(rmin_base, value='Range Min. :', /align_right)
      xmin_base = Widget_Base(rmin_base, /row)
        xmin_label = Widget_Label(xmin_base, value='X  :')
        xmin_text = Widget_Text(xmin_base, UNAME='XMIN_TEXT', xsize=11)
      ymin_base = Widget_Base(rmin_base, /row)
        ymin_label = Widget_Label(ymin_base, value='Y  :')
        ymin_text = Widget_Text(ymin_base, UNAME='YMIN_TEXT', xsize=11)
    ; ## for Max ##
    rmax_base = Widget_Base(range_base, /column)
      ; X&Y Maximum
      xmax_title = Widget_Label(rmax_base, value='Max. :', /align_right)
      xmax_base = Widget_Base(rmax_base, /row)
        xmax_text = Widget_Text(xmax_base, UNAME='XMAX_TEXT', xsize=11)
      ymax_base = Widget_Base(rmax_base, /row)
        ymax_text = Widget_Text(ymax_base, UNAME='YMAX_TEXT', xsize=11)


  ; ###  Stat BASE #1 ###
  stat_base = Widget_Base(stat_base0, /row)

    ; ## Base for MEAN ##
    mean_base = Widget_Base(stat_base, /column)

      ; Y Means
      mean_title = Widget_Label(mean_base, value='MEAN :', /align_right)

      y1mean_base = Widget_Base(mean_base, /row)
      y1_label = Widget_Label(y1mean_base, value='Y1 :')
      y1mean_text = Widget_Text(y1mean_base, UNAME='Y1MEAN_TEXT', xsize=11)

      IF n_elements(y2) EQ n_elements(indep) THEN BEGIN
        y2mean_base = Widget_Base(mean_base, /row)
        y2_label = Widget_Label(y2mean_base, value='Y2 :')
        y2mean_text = Widget_Text(y2mean_base, UNAME='Y2MEAN_TEXT', xsize=11)
      ENDIF
      IF n_elements(y3) EQ n_elements(indep) THEN BEGIN
        y3mean_base = Widget_Base(mean_base, /row)
        y3_label = Widget_Label(y3mean_base, value='Y3 :')
        y3mean_text = Widget_Text(y3mean_base, UNAME='Y3MEAN_TEXT', xsize=11)
      ENDIF
      IF n_elements(y4) EQ n_elements(indep) THEN BEGIN
        y4mean_base = Widget_Base(mean_base, /row)
        y4_label = Widget_Label(y4mean_base, value='Y4 :')
        y4mean_text = Widget_Text(y4mean_base, UNAME='Y4MEAN_TEXT', xsize=11)
      ENDIF
      IF n_elements(y5) EQ n_elements(indep) THEN BEGIN
        y5mean_base = Widget_Base(mean_base, /row)
        y5_label = Widget_Label(y5mean_base, value='Y5 :')
        y5mean_text = Widget_Text(y5mean_base, UNAME='Y5MEAN_TEXT', xsize=11)
      ENDIF

    ; ## Base for Std.DEV. ##
    dev_base = Widget_Base(stat_base, /column)

      ; Y Deviations
      dev_title = Widget_Label(dev_base, value='DEV :', /align_right)

      y1dev_base = Widget_Base(dev_base, /row)
      y1dev_text = Widget_Text(y1dev_base, UNAME='Y1DEV_TEXT', xsize=11)

      IF n_elements(y2) EQ n_elements(indep) THEN BEGIN
        y2dev_base = Widget_Base(dev_base, /row)
        y2dev_text = Widget_Text(y2dev_base, UNAME='Y2DEV_TEXT', xsize=11)
      ENDIF
      IF n_elements(y3) EQ n_elements(indep) THEN BEGIN
        y3dev_base = Widget_Base(dev_base, /row)
        y3dev_text = Widget_Text(y3dev_base, UNAME='Y3DEV_TEXT', xsize=11)
      ENDIF
      IF n_elements(y4) EQ n_elements(indep) THEN BEGIN
        y4dev_base = Widget_Base(dev_base, /row)
        y4dev_text = Widget_Text(y4dev_base, UNAME='Y4DEV_TEXT', xsize=11)
      ENDIF
      IF n_elements(y5) EQ n_elements(indep) THEN BEGIN
        y5dev_base = Widget_Base(dev_base, /row)
        y5dev_text = Widget_Text(y5dev_base, UNAME='Y5DEV_TEXT', xsize=11)
      ENDIF

    ; ## Base for Number of Sample ##
    sample_base = Widget_Base(stat_base, /column)

      ; num. of samples
      sample_title = Widget_Label(sample_base, value='Sample :', /align_right)

      y1spl_base = Widget_Base(sample_base, /row)
      y1spl_text = Widget_Text(y1spl_base, UNAME='Y1SPL_TEXT', xsize=11)

      IF n_elements(y2) EQ n_elements(indep) THEN BEGIN
        y2spl_base = Widget_Base(sample_base, /row)
        y2spl_text = Widget_Text(y2spl_base, UNAME='Y2SPL_TEXT', xsize=11)
      ENDIF
      IF n_elements(y3) EQ n_elements(indep) THEN BEGIN
        y3spl_base = Widget_Base(sample_base, /row)
        y3spl_text = Widget_Text(y3spl_base, UNAME='Y3SPL_TEXT', xsize=11)
      ENDIF
      IF n_elements(y4) EQ n_elements(indep) THEN BEGIN
        y4spl_base = Widget_Base(sample_base, /row)
        y4spl_text = Widget_Text(y4spl_base, UNAME='Y4SPL_TEXT', xsize=11)
      ENDIF
      IF n_elements(y5) EQ n_elements(indep) THEN BEGIN
        y5spl_base = Widget_Base(sample_base, /row)
        y5spl_text = Widget_Text(y5spl_base, UNAME='Y5SPL_TEXT', xsize=11)
      ENDIF

; Create buttons
ctrlbase = Widget_Base(tlb, row=1)


chkecbox_base = Widget_Base(ctrlbase, row=1, /NONEXCLUSIVE, /frame)
SymbBtn = Widget_Button(chkecbox_base, VALUE='Symbol', UNAME='SymbBtn', Tooltip='Symbol On/Off')
IF info.psym NE 0 THEN $
  Widget_Control, SymbBtn, /set_button

dbly=0
IF (n_elements(y2) EQ n_elements(indep)) and (n_elements([y3,y4,y5]) EQ 3) THEN BEGIN
  DblYBtn = Widget_Button(chkecbox_base, VALUE='Double Y', UNAME='DblYBtn', Tooltip='Double Y-axis On/Off')
ENDIF

buttbase = Widget_Base(ctrlbase, row=1)
ClsBtn = Widget_Button(buttbase, value='Close', uname='ClsBtn', Tooltip='Colse this window')

;#####End of WIDGET make code#####

   ; Realize the program.

Widget_Control, tlb, /Realize

   ; Get the window index number of the draw widget.
   ; Make the draw widget the current graphics window
   ; and draw the plot of the data in it. Make the
   ; X data range exactly fit the data.

Widget_Control, drawID, Get_Value=drawIndex
WSet, drawIndex
xrange = info.xrange
yrange = info.yrange
yrange_def = info.yrange_def
IF info.dbly EQ 1 THEN yrange = yrange_def
y2range = [Min(y2),Max(y2)]

Plot, indep, dep, XRange=xrange, YRange=yrange, XLog=xlog, YLog=ylog, $
   xstyle=1, ystyle=1, $
   Background=backColor, Color=axisColor, /NoData, _Extra=extra
OPlot, indep, dep, Color=dataColor, psym=psym

;#
title_pos = CONVERT_COORD([0.85, 1], /NORMAL, /TO_DATA)
IF n_elements(y2) EQ n_elements(indep) THEN BEGIN
    OPlot, indep, y2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
    XYOUTS, title_pos[0], Max(dep)*0.9, y1title, Color=dataColor
    XYOUTS, title_pos[0], Max(y2)*0.9, y2title, Color=dataColor2
ENDIF
IF n_elements(y3) EQ n_elements(indep) THEN BEGIN
    OPlot, indep, y3, Color=dataColor3, psym = (info.psym NE 0) ? -4:0
    XYOUTS, title_pos[0], Max(y3)*0.9, y3title, Color=dataColor3
ENDIF
IF n_elements(y4) EQ n_elements(indep) THEN BEGIN
    OPlot, indep, y4, Color=dataColor4, psym = (info.psym NE 0) ? -2:0
    XYOUTS, title_pos[0], Max(y4)*0.9, y4title, Color=dataColor4
ENDIF
IF n_elements(y5) EQ n_elements(indep) THEN BEGIN
    OPlot, indep, y5, Color=dataColor5, psym = (info.psym NE 0) ? -6:0
    XYOUTS, title_pos[0], Max(y5)*0.9, y5title, Color=dataColor5
ENDIF

  ; Create a pixmap window the same size as the draw widget window.
  ; Store its window index number in a local variable. Draw the same
  ; plot you just put in the draw widget in the pixmap window.

Window, /Free, XSize=zxsize, YSize=zysize, /Pixmap
pixIndex = !D.Window
Plot, indep, dep, XRange=xrange, YRange=yrange, XLog=xlog, YLog=ylog, $
   xstyle=1, ystyle=1, $
   Background=backColor, Color=axisColor, /NoData, _Extra=extra
OPlot, indep, dep, Color=dataColor, psym=psym
;#
title_pos = CONVERT_COORD([0.85, 1], /NORMAL, /TO_DATA)
IF n_elements(y2) EQ n_elements(indep) THEN BEGIN
    OPlot, indep, y2, Color=dataColor2, psym = (info.psym NE 0) ? -1:0
    XYOUTS, title_pos[0], Max(dep)*0.9, y1title, Color=dataColor
    XYOUTS, title_pos[0], Max(y2)*0.9, y2title, Color=dataColor2
ENDIF
IF n_elements(y3) EQ n_elements(indep) THEN BEGIN
    OPlot, indep, y3, Color=dataColor3, psym = (info.psym NE 0) ? -4:0
    XYOUTS, title_pos[0], Max(y3)*0.9, y3title, Color=dataColor3
ENDIF
IF n_elements(y4) EQ n_elements(indep) THEN BEGIN
    OPlot, indep, y4, Color=dataColor4, psym = (info.psym NE 0) ? -2:0
    XYOUTS, title_pos[0], Max(y4)*0.9, y4title, Color=dataColor4
ENDIF
IF n_elements(y5) EQ n_elements(indep) THEN BEGIN
    OPlot, indep, y5, Color=dataColor5, psym = (info.psym NE 0) ? -6:0
    XYOUTS, title_pos[0], Max(y5)*0.9, y5title, Color=dataColor5
ENDIF


IF N_Elements(extra) EQ 0 THEN extraKeywords = Ptr_New(/Allocate_Heap) ELSE extraKeywords = Ptr_New(extra)

   ; Create an info structure to hold information required by the program.

info = { $
   dep:dep, $                       ; The dependent data to be plotted.
   dep2:y2, $                       ;# Added member (2003/04/15)
   dep3:y3, $                       ;#
   dep4:y4, $                       ;#
   dep5:y5, $                       ;#
   y1title:y1title, $               ;#
   y2title:y2title, $               ;#
   y3title:y3title, $               ;#
   y4title:y4title, $               ;#
   y5title:y5title, $               ;#
   indep:indep, $                   ; The independent data to be plotted.
   zxsize:zxsize, $                 ; The X size of the draw widget.
   zysize:zysize, $                 ; The Y size of the draw widget.
   drawID:drawID, $                 ; The widget identifier of the draw widget.
   drawIndex:drawIndex, $           ; The draw window index number.
   pixIndex:pixIndex, $             ; The pixmap window index number.
   xrange:xrange, $                 ; The current X range of the plot.
   yrange:yrange, $                 ; The current Y range of the plot.
   yrange_def:yrange_def, $         ;# default Y range (2003/04/24)
   xlog:xlog, $                     ; A flag for X logarithmic axis style.
   ylog:ylog, $                     ; A flag for Y logarithmic axis style.
   xstyle:1, $                      ; The setting for the XStyle keyword.
   ystyle:1, $                      ;# Added member (2003/04/24)
   xs:0d, $                         ; X static corner of the zoom box.
   ys:0d, $                         ; Y static corner of the zoom box.
   xd:0, $                          ; X dynamic corner of the zoom box.
   yd:0, $                          ; Y dynamic corner of the zoom box.
   x:!X, $                          ; The !X system variable after plot.
   y:!Y, $                          ; The !Y system variable after plot.
   p:!P, $                          ; The !P system variable after plot.
   psym:psym, $                     ;# Added member (2003/03/27)
   dbly:dbly, $                     ;# (2003/04/22)
   extraKeywords:extraKeywords }    ; Extra plot keywords pointer.

   ; Store the info structure in the user value of the top-level base.

Widget_Control, tlb, Set_UValue=info, /No_Copy

   ; Register this program and set up the event loop.

XManager, 'StatPLOT', tlb, Cleanup='StatPLOT_Cleanup', Group_Leader=group, /No_Block, $
   Event_Handler='StatPLOT_Resize'
END ; of StatPLOT ****************************************************************************
