; $Id: textdump.pro,v 1.14 2007/02/05 07:16:44 makiuti Exp $
;+
; NAME:
;       TEXTDUMP
;
; PURPOSE:
;       The purpose of this procedure is to show the FIS data which
;       are selected in the FISv and output the data as a text file.
;       This procedure is called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.jaxa.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       TextDump, x, y, xuvalue, yuvalue, Event
;
; MODIFICATION HISTORY:
;        Written by: Makiuti,S.
;        07/02/05 Last modified
;-

; If the 'Save' button is pressed in a table window
pro textdump_event, Event
COMMON tabledata, tbl_value, tbl_labels

On_Error, 2

  wTarget = Event.id
  wWidget = Event.top

  ;help,x2,y2
  ; Get Table ID & contents value
  TableID = Widget_Info(wWidget, Find_By_Uname='DUMP_TABLE')
  Widget_Control, TableID, Get_Value=table_data

  ; ## Refered 'table_widget_example1.pro' ##
  ; Retrieve the table's selection.
  selection = WIDGET_INFO(TableID, /TABLE_SELECT)
  ; Check to see whether a selection exists, setting the
  ; variable 'hasSelection' accordingly.
  IF (selection[0] ne -1) THEN hasSelection = 1 ELSE hasSelection = 0

  IF ((wTarget eq TableID) AND hasSelection) THEN BEGIN
    IF (hasSelection) THEN WIDGET_CONTROL, TableID, GET_VALUE=value, /USE_TABLE_SELECT
      IF (n_elements(value) GT 1) THEN BEGIN
        window, 28, title='Dump Table Plot'
        PLOT, value, psym=-2, xstyle=2, ystyle=2, /ynozero
        return
      ENDIF
  ENDIF


  case wTarget of

    Widget_Info(wWidget, FIND_BY_UNAME='Dump_Btn'): begin
      if( Tag_Names(Event, /STRUCTURE_NAME) eq 'WIDGET_BUTTON' )then begin

        text_file = DIALOG_PICKFILE(FILTER = '*.txt', /WRITE,$
                      DIALOG_PARENT = wWidget)

        ; If any file was not selected
        IF (text_file EQ "") THEN Retall

        ; check the data type, then set the format
        dim_tbl_value = size(tbl_value,/dimensions)
        format = (size(tbl_value, /type) eq 5) ? '('+string(dim_tbl_value[0])+'D19.6)' : ''

        openw, lun, text_file, width=2250, /get_lun
        printf, lun, ' '+tbl_labels ; put a 'space' at the beginning of the line.
        printf, lun, tbl_value, format=format
        free_lun, lun

      endif
    end
    Widget_Info(wWidget, FIND_BY_UNAME='Cls_Btn'): begin
      WIDGET_CONTROL, Event.top, /DESTROY
      window_index = !d.window
      IF (!d.window eq 28) THEN wdelete, window_index
    end
    else:
  endcase

end
;
;
; Make a table window and show X&Y data in the table
pro textdump, x, y, xuvalue, yuvalue, Event
COMMON tabledata, tbl_value, tbl_labels

  tlb = widget_info(Event.top, find_by_uname='FISV_BASE_0')

  ; Show table on another window
  Table_Base=Widget_Base(UNAME='Table_Base', TITLE='FIS Data Table' ,COLUMN=1,$
    GROUP_LEADER=tlb)

  x2=transpose(x)
  y2=transpose(y)
  IF size(yuvalue,/n_dimensions) NE 0 THEN yuvalue=transpose(yuvalue)

  tbl_value = [x2,y2]
  dim_tbl_value = size(tbl_value,/dimensions)
  tbl_labels = [xuvalue, [yuvalue]]

  ; check the data type, then set the format and column_width
  format = (size(tbl_value, /type) eq 5) ? '(D18.6)' : ''
  cwidth = (size(tbl_value, /type) eq 5) ? 120 : 90

  table=Widget_Table(Table_Base, UNAME='DUMP_TABLE', $
    value=tbl_value, column_labels=tbl_labels, /ALL_EVENTS, $
    x_scroll_size=dim_tbl_value[0], y_scroll_size=Float(Getenv('Table_Col_SIZE')), $
    format=format, /RESIZEABLE_COLUMNS, COLUMN_WIDTHS=cwidth)

  btn_base = Widget_Base(Table_Base, /ROW, /ALIGN_CENTER, YPAD=5)
  dump_btn=Widget_Button(btn_base, UNAME='Dump_Btn', value='Save' ,/ALIGN_CENTER,$
    SCR_XSIZE=120 ,SCR_YSIZE=25, TOOLTIP='Save as a text file')
  cls_btn=Widget_Button(btn_base, UNAME='Cls_Btn', value='Close' ,/ALIGN_CENTER,$
    SCR_XSIZE=120 ,SCR_YSIZE=25, TOOLTIP='Close this window')

  widget_control, Table_Base, /realize

  XMANAGER, 'textdump', Table_Base, GROUP_LEADER=tlb, /NO_BLOCK
end