; $Id: tsdeditor.pro,v 1.9 2007/07/05 10:40:04 makiuti Exp $
;+
; NAME:
;       TSDEDITOR
;
; PURPOSE:
;       The purpose of this procedure is to edit the data in a TSD file.
;       ;; Only 'FIS_OBS' except 'time' can be edited for the moment.
;       This procedure is called by FISv.
;
; AUTHOR:
;       ASTRO-F Data Reduction team
;       Web Page: http://www.ir.isas.jaxa.jp/ASTRO-F/iris_data/
;
; CATEGORY:
;       FIS Data Visualizer
;
; CALLING SEQUENCE:
;       tsdeditor, x, y, xuvalue, yuvalue, Event
;
; MODIFICATION HISTORY:
;       Written by Makiuti,S.
;       2006/02/02 Last modified
;-
;
pro tsdeditor_event, Event
COMMON ShareFISv
COMMON tsdedit_share, tbl_value, tbl_labels

On_Error, 2

  wTarget = Event.id
  wWidget = Event.top

  ; Get Table ID & contents value
  TableID = Widget_Info(wWidget, Find_By_Uname='WID_TABLE_0')
  Widget_Control, TableID, Get_Value=table_data


  ; ## Refered 'table_widget_example1.pro' ##
  ; Retrieve the table's selection.
  selection = WIDGET_INFO(TableID, /TABLE_SELECT)
  ; Check to see whether a selection exists, setting the
  ; variable 'hasSelection' accordingly.
  IF (selection[0] ne -1) THEN hasSelection = 1 ELSE hasSelection = 0

  IF ((wTarget eq TableID) AND hasSelection) THEN BEGIN
    IF (hasSelection) THEN WIDGET_CONTROL, TableID, GET_VALUE=value, /USE_TABLE_SELECT
      IF (n_elements(value) GT 1) THEN BEGIN
        window, 28, title='TSD Editor'
        PLOT, value, psym=-2, xstyle=2, ystyle=2, /ynozero
        return
      ENDIF
  ENDIF


  case wTarget of

    ; # Cancel
    Widget_Info(wWidget, FIND_BY_UNAME='CANCEL'): begin
      WIDGET_CONTROL, Event.top, /DESTROY
      window_index = !d.window
      IF (!d.window eq 28) THEN wdelete, window_index
    end

    ; # Save and Exit
    Widget_Info(wWidget, FIND_BY_UNAME='SAVE'): begin

      FOR i=0, n_elements(tbl_labels)-1 DO BEGIN

        Svalue = transpose(table_data[i,*])

        ; Status
        IF (strmatch(tbl_labels[i], 'CREON') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'CREON', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'ShtOp') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'ShtOp', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'FWposOn') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'FWposOn', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'FWpos_B1') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'FWpos_B1', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'FWpos_B0') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'FWpos_B0', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'MposOn') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'MposOn', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'Mpos_B1') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'Mpos_B1', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'Mpos_B0') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'Mpos_B0', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'RstWideLOn') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'RstWideLOn', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'RstWideSOn') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'RstWideSOn', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'RstN170On') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'RstN170On', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'RstN60On') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'RstN60On', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'SWboostOn') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'SWboostOn', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'LWboostOn') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'LWboostOn', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'SWbiasOn') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'SWbiasOn', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'LWbiasOn') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'LWbiasOn', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'CalALOn') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'CalALOn', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'CalASOn') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'CalASOn', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'CalBOn') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'CalBOn', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'SinALOn') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'SinALOn', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'SinASOn') EQ 1) THEN BEGIN
          result=tsd->Set_Status_TSD('FIS_OBS', 'SinASOn', Svalue)

       ; Analog Telemetry
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'PacketID') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'PacketID', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'RstCntWideL') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'RstCntWideL', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'RstCntWideS') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'RstCntWideS', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'RstCntN170') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'RstCntN170', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'RstCntN60') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'RstCntN60', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'CalPlsCntAL') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'CalPlsCntAL', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'CalPlsCntAS') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'CalPlsCntAS', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'CalPlsCntB') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'CalPlsCntB', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'ADErrCntSW') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'ADErrCntSW', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'ADErrCntLW') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'ADErrCntLW', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'MPosCnt') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'MPosCnt', long(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'DacBiasWideS') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'DacBiasWideS', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'DacBiasN60') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'DacBiasN60', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'DacBiasWideL') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'DacBiasWideL', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'DacBiasN170') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'DacBiasN170', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'DacCalAS') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'DacCalAS', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'DacCalAL') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'DacCalAL', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'DacCalB') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'DacCalB', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'DacSinAS') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'DacSinAS', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'DacSinAL') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'DacSinAL', byte(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'TpBody') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'TpBody', fix(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'TpSW') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'TpSW', fix(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'TpLW') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'TpLW', fix(Svalue) )
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'TpCalFT') EQ 1) THEN BEGIN
          result=tsd->Set_Value_TSD('FIS_OBS', 'TpCalFT', fix(Svalue) )

        ; Detector data
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'VDET*') EQ 1) THEN BEGIN
          det=tsd->Get_Value_TSD('FIS_OBS', 'DET')
          ch = uint(strmid(tbl_labels[i], 4,3))
          det[ch-1,*] = Svalue
          result=tsd->Set_Value_TSD('FIS_OBS', 'DET', det)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'FLUX*') EQ 1) THEN BEGIN
          flux=tsd->Get_Value_TSD('FIS_OBS', 'FLUX')
          ch = uint(strmid(tbl_labels[i], 4,3))
          flux[ch-1,*] = Svalue
          result=tsd->Set_Value_TSD('FIS_OBS', 'FLUX', flux)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'FERR*') EQ 1) THEN BEGIN
          ferr=tsd->Get_Value_TSD('FIS_OBS', 'FERR')
          ch = uint(strmid(tbl_labels[i], 4,3))
          ferr[ch-1,*] = Svalue
          result=tsd->Set_Value_TSD('FIS_OBS', 'FERR', ferr)

        ; Flag
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'BAD_FRAME') EQ 1) THEN BEGIN
          result=tsd->Set_Flag_TSD('FIS_OBS', 'BAD_FRAME', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'UNDEF_ANOM_FRAME') EQ 1) THEN BEGIN
          result=tsd->Set_Flag_TSD('FIS_OBS', 'UNDEF_ANOM_FRAME', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'BLANK') EQ 1) THEN BEGIN
          result=tsd->Set_Flag_TSD('FIS_OBS', 'BLANK', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'IN_SAA') EQ 1) THEN BEGIN
          result=tsd->Set_Flag_TSD('FIS_OBS', 'IN_SAA', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'NEAR_MOON') EQ 1) THEN BEGIN
          result=tsd->Set_Flag_TSD('FIS_OBS', 'NEAR_MOON', Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'UNTRUSTED_FRAME') EQ 1) THEN BEGIN
          result=tsd->Set_Flag_TSD('FIS_OBS', 'UNTRUSTED_FRAME', Svalue)

        ; Pix_Flag
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'BAD_*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'BAD', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'UANM*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'UNDEF_ANOM', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'AERR*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'ARITH_ERR', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'DEAD*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'DEAD', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'SATU*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'SATURATE', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'REST*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'RESET', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'RANM*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'RSTANOM', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'NDIF*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'NO_DIFF', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'NRPC*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'NO_RP_CORR', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'NDKC*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'NO_DK_CORR', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'NDCC*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'NO_DCCAL_CORR', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'NTRC*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'NO_TR_CORR', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'NFLF*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'NO_FLAT_FIELD', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'NGPG*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'NO_GPGL', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'NMTG*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'NO_MTGL', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'NABC*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'NO_ABSCAL', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'GPG1*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'GPGL_TYPE1', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'GPG2*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'GPGL_TYPE2', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'GPG3*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'GPGL_TYPE3', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'GPG4*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'GPGL_TYPE4', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'GPGT*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'GPGL_TAIL', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'MTG1*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'MTGL_TYPE1', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'MTG2*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'MTGL_TYPE2', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'MTG3*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'MTGL_TYPE3', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'MTG4*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'MTGL_TYPE4', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'MTGT*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'MTGL_TAIL', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'NPER*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'NO_PERI_CORR', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'SPAK*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'SX_PEAK', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'SRCE*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'SX_SOURCE', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'SOBJ*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Pix_Flag_TSD('FIS_OBS', 'SX_OBJ', PIXNO=ch-1, Svalue)

        ; Quality
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QCVP*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_CV_PARAM', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QRCP*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_RC_PARAM', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QRCC*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_RC_CF', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QDFE*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_DF_EQ', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QRPD*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_RP_DATA', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QRPP*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_RP_PARAM', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QRPT*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_RP_TABLE', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QFFP*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_FF_PARAM', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QFFC*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_FF_CF', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QGPC*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_GPGL_CORR', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QMTC*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_MTGL_CORR', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QTRH*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_TR_HIST', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QTRP*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_TR_PARAM', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QDKD*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_DK_DATA', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QDKP*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_DK_PARAM', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QDKT*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_DK_TABLE', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QFXC*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_FX_CORR', PIXNO=ch-1, Svalue)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'QFXP*') EQ 1) THEN BEGIN
          ch = uint(strmid(tbl_labels[i], 4,3))
          result=tsd->Set_Quality_TSD('FIS_OBS', 'QUAL_FX_PARAM', PIXNO=ch-1, Svalue)

        ; Counter
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'CSAA*') EQ 1) THEN BEGIN
          data=tsd->Get_Value_TSD('FIS_OBS', 'CNT_SAA')
          ch = uint(strmid(tbl_labels[i], 4,3))
          data[ch-1,*] = Svalue
          result=tsd->Set_Value_TSD('FIS_OBS', 'CNT_SAA', data)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'CGGP*') EQ 1) THEN BEGIN
          data=tsd->Get_Value_TSD('FIS_OBS', 'CNT_GLITCH_GP')
          ch = uint(strmid(tbl_labels[i], 4,3))
          data[ch-1,*] = Svalue
          result=tsd->Set_Value_TSD('FIS_OBS', 'CNT_GLITCH_GP', data)
        ENDIF ELSE IF (strmatch(tbl_labels[i], 'CGMT*') EQ 1) THEN BEGIN
          data=tsd->Get_Value_TSD('FIS_OBS', 'CNT_GLITCH_MT')
          ch = uint(strmid(tbl_labels[i], 4,3))
          data[ch-1,*] = Svalue
          result=tsd->Set_Value_TSD('FIS_OBS', 'CNT_GLITCH_MT', data)


        ENDIF ELSE BEGIN
          print, tbl_labels[i]+' has not been changed.'
        ENDELSE

      ENDFOR

      WIDGET_CONTROL, Event.top, /DESTROY
      window_index = !d.window
      IF (!d.window eq 28) THEN wdelete, window_index
    end

    ; # ELSE
    else:
  endcase

end

pro tsdeditor, x, y, xuvalue, yuvalue, Event
COMMON tsdedit_share

  ;
  tlb = widget_info(Event.top, find_by_uname='FISV_BASE_0')

  TSD_EDITOR_BASE = Widget_Base( GROUP_LEADER=tlb,  $
      UNAME='TSD_EDITOR_BASE' ,XOFFSET=5 ,YOFFSET=5  $
      ,/BASE_ALIGN_CENTER ,TITLE='TSD Editor' ,SPACE=3 ,XPAD=3 ,YPAD=3  $
      ,COLUMN=1)

  x2=transpose(x)
  y2=transpose(y)
  IF size(yuvalue,/n_dimensions) NE 0 THEN yuvalue=transpose(yuvalue)

  tbl_value = [x2,y2]
  dim_tbl_value = size(tbl_value,/dimensions)
  tbl_labels = [xuvalue, [yuvalue]]

  WID_Label = Widget_Label(TSD_EDITOR_BASE, value='TSD EDITOR ver.2.0', /frame, $
      /align_center, xsize=200, ysize=20)


  ; check the data type, then set the format and column_width
  format = (size(tbl_value, /type) eq 5) ? '(E16.9)' : ''
  cwidth = (size(tbl_value, /type) eq 5) ? 110 : 80

  WID_TABLE_0 = Widget_Table(TSD_EDITOR_BASE, UNAME='WID_TABLE_0'  $
      ,value=tbl_value, column_labels=tbl_labels ,/EDITABLE ,/ALL_EVENTS $
      ,x_scroll_size=dim_tbl_value[0], y_scroll_size=Float(Getenv('Table_Col_SIZE')) $
      ,format=format, /RESIZEABLE_COLUMNS, COLUMN_WIDTHS=cwidth)


  WID_BASE_1 = Widget_Base(TSD_EDITOR_BASE, UNAME='WID_BASE_1'  $
      ,XOFFSET=26 ,YOFFSET=495 ,TITLE='IDL' ,SPACE=3 ,XPAD=3 ,YPAD=3  $
      ,ROW=1 ,/GRID_LAYOUT)

  WID_BUTTON_1 = Widget_Button(WID_BASE_1, UNAME='SAVE'  $
      ,/ALIGN_CENTER ,VALUE='Save and Exit', TOOLTIP='Save and Exit')

  WID_BUTTON_0 = Widget_Button(WID_BASE_1, UNAME='CANCEL'  $
      ,/ALIGN_CENTER ,VALUE='CANCEL', TOOLTIP='CANCEL')


  Widget_Control, /REALIZE, TSD_EDITOR_BASE


  XManager, 'tsdeditor', TSD_EDITOR_BASE, /NO_BLOCK

end
