;#> frin_plot_total.dc3
; Identifier	frin_plot_total
;
; Purpose	plot all of the fringes
;
; Synopsis	frin_plot_total, wn, xx, wt, fmin, freq, aac, aas, chisq, pl, frid
;
; Arguments	Name	I/O    Type:	Description
;		------------------------------------------------------------
;		wn	 I     fltarr	wave numbers
;		xx	 I     fltarr	fringe data
;		wt	 I     fltarr	weights
;		fmin	 I     float	selected fringes
;		freq	 I     fltarr	fringe frequencies
;		aac	 I     fltarr	cosine amps
;		aas	 I     fltarr	sine amps
;		chisq	 I     fltarr	chi-sq values
;		pl	 I     struct	plot struct
;		frid	 I     struct	fringe identity
;
; Returns	--
;
; Description
;
;
;
;
; Comment
;
;
; Example	--
;
; Category	UTIL ( module: fringes )
;
; Filename	frin_plot_total.pro
;
; Author	Do Kester
;
; Version	1.2
;
; History	1.0  15-03-2001 DK, first implementation of frin_plot_total.pro
;		1.1  05-04-2001 DK, remove compile_opt
;		1.2  16-05-2001 DK, remove minor tick marks on y-axis
;#<
;=========================================================================

pro frin_plot_total, wn, xx, wt, fmin, freq, aac, aas, chisq, pl, frid

; compile_opt idl2, hidden

forward_function frin_title

winset, 0, xsize=600, ysize=400, title='Fringes found'

md = median( abs( xx ) ) * 6
; wt = temporary( wt ) * md - md
yran = [ -md, md ]
wr = ( wn[n_elements( wn )-1] - wn[0] ) / 3
xran = [ wn[0], wn[0] + wr ]

frin_plot_part, wn, xx, wt, pl, frid, xr=xran, yr=yran, pos=[0.1,0.1,0.9,0.3], $
		xtitle="wavenumber", yminor=1, yticklen=0.01

xran = xran + wr
frin_plot_part, wn, xx, wt, pl, frid, xr=xran, yr=yran, pos=[0.1,0.3,0.9,0.5], $
		/noerase, yminor=1, yticklen=0.01

xran = xran + wr
frin_plot_part, wn, xx, wt, pl, frid, xr=xran, yr=yran, pos=[0.1,0.5,0.9,0.7], $
		/noerase, yminor=1, yticklen=0.01

maxa = max( [aac, aas], min=mina )
maxc = max( chisq, min= minc )

pltitle = frin_title( frid )

plot, freq, aac, pos=[0.1,0.7,0.9,0.9], /noerase, title= pltitle, $
	xst=3, yst=3, yminor=1, yticklen=0.01
oplot, freq, aas, color=5
oplot, freq, ( chisq - minc ) * ( maxa - mina ) / ( maxc - minc ) + mina, $
	color=2
for k = 0, n_elements( fmin ) - 1 do begin
  oplot, [fmin[k], fmin[k]], [!y.crange[0],!y.crange[1]], color=3
endfor

return
end

