; $Id: clear_all_flag.pro,v 1.3 2006/12/26 04:49:46 makiuti Exp $
; Copyright (c) 2006, ASTRO-F Data Reduction Team. All rights reserved.
;
;+
; NAME:
;   CLEAR_ALL_FLAG
;
; PURPOSE:
;   Clear (set all 0) most flags, pix_flags, and quality values in a TSD.
;
;    * 'BLANK', 'IN_SAA', 'NEAR_MOON', 'UNTRUSTED_FRAME' of frame flags 
;      will not be cleared by default. ALL option can be used to celar 
;      all flags.
;    * Target is the 'FIS_OBS' branch only at the moment.
;
; CALLING SEQUENCE:
;
;   Clear_All_Flag, tsd[, /ALL]
;
; MODIFICATION HISTORY:
;   Written by Makiuti, S
;   2006/12/26 Last modified
;-
PRO Clear_All_Flag, tsd, ALL=all

  ;;;;
  ;  Clear Frame Flags
  ;;;;

    IF keyword_set(all) THEN BEGIN
      ; If /ALL option was set, all frame flags will be cleared.
      dumy = tsd->Get_Value_TSD('FIS_OBS', 'FLAG')
      dumy[*]=0
      ret = tsd->Set_Value_TSD('FIS_OBS', 'FLAG', dumy)
    ENDIF ELSE BEGIN
      ; By default, only 'BAD_FRAME' and 'UNDEF_ANOM_FRAME' will be cleared.
      tmp = tsd->Get_Flag_TSD('FIS_OBS', 'BAD_FRAME')
      tmp[*] = 0 ; clear
      ret = tsd->Set_Flag_TSD('FIS_OBS', 'BAD_FRAME', tmp)

      tmp = tsd->Get_Flag_TSD('FIS_OBS', 'UNDEF_ANOM_FRAME')
      tmp[*] = 0 ; clear
      ret = tsd->Set_Flag_TSD('FIS_OBS', 'UNDEF_ANOM_FRAME', tmp)
    ENDELSE

  ;;;;
  ;  Clear Pixel Flags
  ;;;;

    dumy = tsd->Get_Value_TSD('FIS_OBS', 'PIX_FLAG')
    dumy[*]=0
    ret = tsd->Set_Value_TSD('FIS_OBS', 'PIX_FLAG', dumy)

  ;;;;
  ;  Clear Quality Indicators
  ;;;;

    dumy = tsd->Get_Value_TSD('FIS_OBS', 'QUALITY')
    dumy[*]=0
    ret = tsd->Set_Value_TSD('FIS_OBS', 'QUALITY', dumy)

; -- obsolete codes below --
;
;    fisobs_frm_flag_members = $
;      [ 'BAD_FRAME', 'UNDEF_ANOM_FRAME', 'BLANK', 'IN_SAA', $
;        'NEAR_MOON', 'UNTRUSTED_FRAME' ]
;
;    fisobs_pix_flag_members = $
;      [ 'BAD', 'UNDEF_ANOM', 'ARITH_ERR', 'DEAD', $
;        'SATURATE', 'RESET', 'RSTANOM', 'NO_DIFF', $
;        'NO_RP_CORR', 'NO_DK_CORR', 'NO_DCCAL_CORR', 'NO_TR_CORR', $
;        'NO_FLAT_FIELD', 'NO_GPGL', 'NO_MTGL', 'NO_ABSCAL', $
;        'GPGL_TYPE1', 'GPGL_TYPE2', 'GPGL_TYPE3', 'GPGL_TYPE4', $
;        'GPGL_TAIL', 'MTGL_TYPE1', 'MTGL_TYPE2', 'MTGL_TYPE3', $
;        'MTGL_TYPE4', 'MTGL_TAIL', 'NO_PERI_CORR', 'SX_PEAK', $
;        'SX_SOURCE', 'SX_OBJ' ]
;
;    fisobs_quality_members = $
;      [ 'QUAL_CV_PARAM', 'QUAL_CV_PARAM', 'QUAL_RC_PARAM', 'QUAL_RC_PARAM', $
;        'QUAL_RC_CF', 'QUAL_RC_CF', 'QUAL_DF_EQ', 'QUAL_DF_EQ', $
;        'QUAL_RP_DATA', 'QUAL_RP_DATA', 'QUAL_RP_PARAM', 'QUAL_RP_PARAM', $
;        'QUAL_RP_TABLE', 'QUAL_RP_TABLE', 'QUAL_FF_PARAM', 'QUAL_FF_PARAM', $
;        'QUAL_FF_CF', 'QUAL_FF_CF', 'QUAL_GPGL_CORR', 'QUAL_GPGL_CORR', $
;        'QUAL_MTGL_CORR', 'QUAL_MTGL_CORR', 'QUAL_TR_HIST', 'QUAL_TR_HIST', $
;        'QUAL_TR_PARAM', 'QUAL_TR_PARAM', 'QUAL_DK_DATA', 'QUAL_DK_DATA', $
;        'QUAL_DK_PARAM', 'QUAL_DK_PARAM', 'QUAL_DK_TABLE', 'QUAL_DK_TABLE', $
;        'QUAL_FX_CORR', 'QUAL_FX_CORR', 'QUAL_FX_PARAM', 'QUAL_FX_PARAM' ]
;
;    FOR i = 0, n_elements(fisobs_frm_flag_members)-1 DO BEGIN
;      tmp = tsd->Get_Flag_TSD('FIS_OBS', fisobs_frm_flag_members[i])
;      tmp[*] = 0 ; clear
;      ret = tsd->Set_Flag_TSD('FIS_OBS', fisobs_frm_flag_members[i], tmp)
;    ENDFOR

;    FOR i = 0, n_elements(fisobs_pix_flag_members)-1 DO BEGIN
;      tmp = tsd->Get_Pix_Flag_TSD('FIS_OBS', fisobs_pix_flag_members[i])
;      tmp[*] = 0 ; clear
;      ret = tsd->Set_Pix_Flag_TSD('FIS_OBS', fisobs_pix_flag_members[i], tmp)
;    ENDFOR
;
;    FOR i = 0, n_elements(fisobs_quality_members)-1 DO BEGIN
;      tmp = tsd->Get_Quality_TSD('FIS_OBS', fisobs_quality_members[i])
;      tmp[*] = 0 ; clear
;      ret = tsd->Set_Quality_TSD('FIS_OBS', fisobs_quality_members[i], tmp)
;    ENDFOR

END
