;
; Supplies function for cal_fit_lin
;
; Define a return function for LMFIT:
;   f(x) = a[0]*t + a[1]
Function cal_lin_respons, t, a
  f  = a[0]*(t)+a[1]

  Return, [a[0]*(t)+a[1], t , 1.0]
End

;
; Performs linear fitting and returns y(t1)
; where t1 may be an array
;
Function gb_cal_fit_lin, y, t, t1, YFIT=YFit, PARAMS=Params, TSTART=t0, CHISQ=chisq, ERR=err, ITER=iter, ITMIN=itmin

  ;  initial guess for the coefficients
  ;
  ndata = n_elements(y)
  a = findgen(2)

  a[0] = (y[ndata-1]-y[0])/(t[ndata-1]-t[0])
  a[1] = mean(y)

  fita = [1,1]

  IF ~Keyword_Set(t0) THEN BEGIN
      t0  = t[0]
  ENDIF
  t_0 = t - t0

;   function_name = 'cal_lin_respons'
;   c = lmfit(t_0, y, a, FITA=fita, FUNCTION_NAME=function_name, CHISQ=chisq, $
;             /double, CONVERGENCE=convergence, ITMAX=300, MEASURE_ERR=err)

;   case convergence of
;     1  : convergence=1 ; be quiet ;-)
;     0  : LMESSAGE, 7, "% the algorithm '"+function_name+"' did not converge."
;     -1 : LMESSAGE, 7, "% the algorithm '"+function_name+"' encountered a singular matrix and did not converge."
;   endcase
;   b = a[0]*(t1-t0) + a[1]

  ;; "robust" least absolute deviation method
  result = ladfit(t_0, y, ABSDEV=absdev, /DOUBLE)
  c = result[1]*(t_0) + result[0]
  b = result[1]*(t1-t0) + result[0]

  IF arg_present(YFit)   NE 0 THEN YFit = c   ; Fitted result value at each point
  IF arg_present(Params) NE 0 THEN Params = a ; Parameters of the fitting function

  Return, b

End

