; $Id: gb_get_name_ver.pro,v 1.8 2007/08/13 09:01:35 makiuti Exp $
;
; Copyright (c) 2003-2007, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;       gb_get_name_ver.pto
; PURPOSE:
;
; CATEGORY:
;       Green Box lib in ASTRO-F pipeline.
; CALLING SEQUENCE:
;       Result = gb_get_name_ver(infeil='infile'
;                  [, mod_name=mod_name, ver=ver])
; INPUTS:
;       Name of the function. The function must have
;       been compiled or executed before getting name or ver..
; OUTPUTS:
;       Name and versions
; RETURN:
;       Name and versions. -1 if the name or ver. is not
;       available because of an unexpected format of
;       the header.
; RESTRICTIONS:
;       Functions must contain the name or ver. in the first
;       line of the program. The format is like e.g.,
;
;       ; $Id: gb_get_name_ver.pro,v 1.8 2007/08/13 09:01:35 makiuti Exp $
;
; PROCEDURE:
;
; EXAMPLE:
;        > name_ver=gb_get_name_ver(infile='gb_get_name_ver' $
;        >          , mod_name=mod_name, ver=ver)
;        >
;        > print,name_ver
;        > gb_get_name_ver.pro,v 1.27
;        > print,mod_name
;        > gb_get_name_ver.pro
;        > print,ver
;        > 1.27
;
; MODIFICATION HISTORY:
;   Nov. 2003     Written by A. Kawamura.
;   Aug. 2007     Last modified by Makiuti
;-

FUNCTION gb_get_name_ver, infile=infile, mod_name=mod_name, ver=ver

  dummy = routine_info(infile, /functions, /source)
  inf = dummy.path

  msg = 'Read Module: '+inf
  LMESSAGE, 7, msg

  ;; Read the 1st line of the module (infile)
  close, 1
  ;openr,lun,inf,/get_lun
  openr, 1, inf
  first_line = ''
  readf, 1, first_line
  close, 1
  msg = '1st line of the module: '+first_line
  LMESSAGE, 7, msg

  ;;
  string_length = STRLEN(first_line)
  IF (string_length LE 8) THEN BEGIN
    msg = 'Unable to get the module name and/or ver. of the module.'
    LMESSAGE, 7, msg, /CONTINUE
    return, ''
  ENDIF

  ;;
  mod_name_p = strpos(first_line, '$Id: ') ; Look for module name
  ver_p  = strpos(first_line, 'v ') ; Look for version

  IF (mod_name_p EQ -1) OR (ver_p EQ -1) THEN BEGIN
    msg = 'Unable to get the module name and/or ver. of the module.'
    LMESSAGE, 7, msg, /CONTINUE
    return, ''
  ENDIF

  ;;
  name_ver = strmid(first_line, mod_name_p+5, ver_p-(mod_name_p+5)+6)
  mod_name = strmid(first_line, mod_name_p+5, ver_p-(mod_name_p+5)-1)
  ver = strmid(first_line, ver_p+2, 4)

  name_ver = STRTRIM(name_ver, 2)
  mod_name = STRTRIM(mod_name, 2)
  ver      = STRTRIM(ver, 2)

  ;help, name_ver, mod_name, ver

  RETURN, name_ver

END
