; $Id: read_glitch_ccf.pro,v 1.5 2006/01/18 06:57:00 baba Exp $
;
; Copyright (c) 2003, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	gb_gauss_glitch.pro
; PURPOSE:
;	Aplly the GaussianProcess glitch finder to one pixel
; CATEGORY:
;	Function. Library for the procedures in ASTRO-F pipeline.
; CALLING SEQUENCE:
;	glitcal = read_glitch_ccf( ccf, pixel, theta=theta, iter=iter, $
;	    npoint=npoint, cutoff=cutoff, boxlen=boxlen, neborin=neborin, $
;	    source=source
; INPUTS:
;	ccf		calibration constants file (not yet implemented)
;	pixel		pixel number
;	theta		GP constants
;			default: [2.0, 1.0, 0.0, 0.1, 0.0]
;	iter		maximum number of internal iterations
;			default: 10
;	npoint		width of the GP process
;			default: 15
;	cutoff		level to detect glitches in S/N
;			default: 3
;	boxlen		smoothing length of no-glitch bandwidth
;			default: 81
;	neborin 	influence of neighbouring points
;			default: 0.4
;	source		influence of point sources
;			default: 0.7
;
; OUTPUTS:
; RETURN:
;	flag		glitch flags are set
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; REFERENCE:
;
; MODIFICATION HISTORY:
;	Dec. 2003		Written by Do Kester.
;	Oct. 2004		Update
;-


;; for now return hard coded values, independent of channel

FUNCTION read_glitch_ccf, ccf, pixel, theta=theta, iterations=iterations, $
	npoint=npoint, cutoff=cutoff, boxlen=boxlen, neborin=neborin, $
	source=source

if ( n_elements( theta ) EQ 0 ) then theta = [2.0, 1.0, 0.0, 0.1, 0.0]
if ( n_elements( iterations ) EQ 0 ) then iterations = 10
if ( n_elements( npoint ) EQ 0 ) then npoint = 15
if ( n_elements( cutoff ) EQ 0 ) then cutoff = 3.0
if ( n_elements( boxlen ) EQ 0 ) then boxlen = 81
if ( n_elements( neborin ) EQ 0 ) then neborin = 0.4
if ( n_elements( source ) EQ 0 ) then source = 1.0
if ( n_elements( radius ) EQ 0 ) then radius = 3.0
if ( n_elements( amplit ) EQ 0 ) then amplit = 1e-4
if ( n_elements( offset ) EQ 0 ) then offset = 0.0
if ( n_elements( noise ) EQ 0 ) then noise = 1e-4
if ( n_elements( slope ) EQ 0 ) then slope = 0.0
if ( n_elements( prior ) EQ 0 ) then prior = 1000.0
if ( n_elements( spike ) EQ 0 ) then spike = 1.0


glitcal = { theta   : theta, $	    ; gaussian parameter set
	    npoint  : npoint, $     ; size of environment
	    iter    : iterations, $ ; maximum nr of iterations
	    cutoff  : cutoff, $     ; S/N
	    boxlen  : boxlen, $     ; smoothing length of bandwidth
	    neborin : neborin, $    ; influence of neightbouring samples
	    source  : source, $     ; influence of point sources
	    radius  : radius, $     ; radius of gaussian process
	    amplit  : amplit, $     ; amplitude of gaussian process
	    offset  : offset, $     ; offset of gaussian process
	    noise   : noise, $	    ; noise of gaussian process
	    slope   : slope, $	    ; slope of gaussian process
	    prior   : prior, $	    ; prior range
	    spike   : spike }	    ; relative spike strength

return, glitcal
end

