; $Id: gb_make_dc_cal_lamp_table.pro,v 1.25 2007/06/11 06:37:34 makiuti Exp $
;
Pro write_cal_fits, A, acomment, flnm, CALVER=calver

  IF n_elements(calver) EQ 0 THEN calver = 3 ; Version of method to get Cal lamp signal strength.
                                             ; Should be same as TABLEVER
                                             ; ver3 : original method (integration of Cal On signals)
                                             ;        => Method A
                                             ; ver4 : new method (pick up peak value of 4-points fit for ramp)
                                             ;      : using getcal_fit()
                                             ;        => Method B

  ;; Create the primary header

  ; Find Time Stamp
  CALDAT, SYSTIME(/JULIAN, /UTC), mm, dd, yy, hh, ms, ss
  cdate = STRING(FORMAT='(I4.4,"-",I2.2,"-",I2.2,"T",I2.2,":",I2.2,":",I2.2)', $
            yy, mm, dd, hh, ms, ss, /PRINT)

  FXHMAKE,  header, /initialize, /extend, /date
  FXADDPAR, header, 'ORIGIN',   'ISAS/JAXA',    'Organization creating FITS file'
  FXADDPAR, header, 'DATE',     cdate,          'Date of data creation'
  FXADDPAR, header, 'DATATYPE', 'FIS CAL ',     'Data Type'
  FXADDPAR, header, 'TELESCOP', 'ASTRO-F ',     'ASTRO-F mission'
  FXADDPAR, header, 'INSTRUME', 'FIS     ',     'FIS'
  FXADDPAR, header, 'TABLEVER', calver    ,     'Version of Cal Table'
  FXADDPAR, header, 'CREATOR',  'GB_MAKE_DC_CAL_LAMP_TABLE', 'Green Box module name and version'
  FXADDPAR, header, 'COMMENT',  '## FIS DC_CAL_LAMP TABLE ##', ''

  FXWRITE, flnm, header

  ;; Create the CAL extention header and write it

  sz_cal = size(A)

  FXBHMAKE, bhdcal, sz_cal[1], 'CAL',                 'Extension name'
  FXADDPAR, bhdcal, 'EXTVER',  '1',                   'Extension version (must be an integer)'
  FXADDPAR, bhdcal, 'TIMESYS', 'UTC',                 'Explicit time scale specification'
  FXADDPAR, bhdcal, 'DATE-OBS','2006-02-21T21:28:00', 'Date of observation start'
  FXADDPAR, bhdcal, 'DATE-END','2008-12-31T00:00:00', 'Date of observation end'
  FXADDPAR, bhdcal, 'COMMENT', '', ''
  FXADDPAR, bhdcal, 'COMMENT', 'Extracted from FIS observation packet', ''
  FXADDPAR, bhdcal, 'COMMENT', acomment,  ''
  FXADDPAR, bhdcal, 'COMMENT', '', ''

  FXBADDCOL, iAFTIME,     bhdcal, 0D,  'AFTIME',    'ASTRO-F satellite time',  TUNIT = 'sec'
  FXBADDCOL, iFLUX,       bhdcal, 0E,  'FLUX',      'FIS CAL data'
  FXBADDCOL, iERR,        bhdcal, 0E,  'ERR',       'Error of CAL flux'
  FXBADDCOL, iSHUTOPEN,   bhdcal, 0B,  'SHUTOPEN',  'Shutter Open/Close'
  FXBADDCOL, iBAD,        bhdcal, 0B,  'BAD',       'Bad ON/OFF'
  FXBADDCOL, iDAC,        bhdcal, 0B,  'CALADAC',   'CalA DAC set value'
  FXBADDCOL, iPID,        bhdcal, 0B,  'PID',       'Packet ID'
  FXBADDCOL, iTM_SAA,     bhdcal, 0D,  'TM_SAA',    'Time from the last SAA(BB)',  TUNIT = 'sec'

  MWRFITS, A, flnm, bhdcal, /NO_COMMENT

  return

End
;
; Copyright (c) 2002-2007, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
;
;FUNCTION NAME:
;        GB_DC_Cal_Lamp_Table.pro
;
;PURPOSE:
;        The DC responsivity variation can be evaluated by monitoring
;        the signals of the calibration lamp. This function creates a DC
;        calibration table for the correction of the DC responsivity.
;        This table also may be used for a quick look of responsivity
;        variation over time.  The output DC calibration table is updated
;        by appending new data as scan goes on.
;
;CATEGORY:
;        Function. Green Box procedure in ASTRO-F pipeline.
;
;CALLING SEQUENCE:
;        Result = GB_DC_Cal_Lamp_Table(tsd)
;
;INPUTS:
;        tsd ; TSD object reference.
;
;OUTPUT FILE FORMAT and NAMING:
;
;        The output file name will be CAL_LW_xxxx.fits or CAL_SW_xxxx.fits,
;        depending on the band. The xxxx is associated with AFTIME when
;        data are taken. Since AFTIME is expected to range ~2x10(8) sec.
;        during the observations, we divide them with 100000L, and use these
;        four digit values for xxxx.
;          ex)  aftime = 1.238938423828x10(8)  --> xxxx = 1238
;        Approximately 1 or 2 files per day will be generated.
;
;        These files will be saved in directory ./CALTABLE. This directory
;        will be automatically generated, if it does not exist.
;
;        The output file is in FITS extension form as follows.
;
;          1st column : aftime  - DOUBLE AFTime
;          2nd column : flux    - FLOAT  ARRAY with size of 100 (SW) or 75 (LW)
;          3rd column : ferr    - FLOAT  ARRAY with size of 100 (SW) or 75 (LW)
;          4th column : shtop   - BYTE   Shutter status
;          5th column : bad     - BYTE   ARRAY with size of 100 (SW) or 75 (LW)
;          6th column : caladac - BYTE   CalA DAC set value
;          7th column : pid     - BYTE   Packet ID
;          8th column : tm_saa  - DOUBLE Time from the last SAA (Bias Boost)

;
;RETURN:
;        !SUCCEED if function ended successfully.
;        !FAIL in failure.
;
;PROCEDURE and DESCRIPTION:
;        1. Extract detector data and status mentioned above from TSD.
;        2. Extract the data with some statuses.
;        3. Assign cal. file name using AFTIME information. There does not
;           have to be only one cal. file.
;        4. Check if CALTABLE directory exists, and if none, make it.
;        5. Check if the above named cal. file exists, and if not, save cal.
;           data to a new cal. file.
;        6. If the cal. file exist, read the cal. file and sort following
;           the sequence of AFTIMEs. If there are overlaps in AFTIMEs, then
;           they are deleted.
;        7. A simple comment is also recorded in the new cal. file.
;
;        ** notes **
;        In order to read/write cal. table files which are not in TSD format,
;        several procedures in IDL ASTRONOMY LIBRARY have been used such as
;        MRDFITS, MWRFITS, FXHMAKE, FXADDPAR, FXWRITE, FXBHMAKE, and FXBADDCOL.
;
;
;REFERENCE:
;        Baba et al. "ASTRO-F/FIS Survey Data Structure"
;
;MODIFICATION HISTORY:
;        Oct. 2002    Written by A. Kawamura.
;        May  2004    by YS: output file format is 'proposed'.
;        Jun. 2007    Last modified by Makiuti
;
;-
FUNCTION GB_Make_DC_Cal_Lamp_Table, tsd, CALVER=calver, _extra=_extra
; ON_ERROR, 2

  IF n_elements(calver) EQ 0 THEN calver = 3 ; Version of method to get Cal lamp signal strength.
                                             ; ver3 : original method (integration of Cal On signals)
                                             ; ver4 : new method (pick up peak value of 4-points fit)

  ;;; Starting GB_DC_Cal_Lamp ;;;
  LMESSAGE, 5, 'Executing GB_DC_Cal_Lamp_Table...', /CONTINUE

  ; Table file division unit
  fldiv = 1.D5  ; seconds, table file is generated at this rate

  ;;; Extract values from TSD object reference ;;;
  in_flux = tsd->Get_value_tsd('FIS_OBS','Flux')
  ret     = af_finite(in_flux, flux)
  verr    = tsd->Get_value_tsd('FIS_OBS','FErr')
  ti      = tsd->Get_value_tsd('FIS_OBS','AFtime')
  ; number of channels ;;;
    si = SIZE(flux)
    num_ch   = si[1]
    num_data = si[2]

  ;;; Define duration of lighting by CalA for pulse(short) Cal.
  ;   ## This values need to be optimized. ##
  IF (num_ch EQ 75) THEN BEGIN ;; LW
    calAtime = 0.5     ; (05/11/26 since IT#4)
    idura = fix(calAtime*16.86)+1 ; Sampling rate is 16.86Hz in Survey mode
                                  ; => idura = 9
  ENDIF ELSE IF (num_ch EQ 100) THEN BEGIN ;; SW
    ;calAtime = 0.2     ; Old default
    ;calAtime = 0.1399  ; Crossing time of point source in survey obs.
    calAtime = 0.16    ; (05/11/26 since IT#4)
    idura = fix(calAtime*25.28)+1 ; Sampling rate is 25.28Hz in Survey mode
                                  ; => idura = 5
  ENDIF

  ;;-----------------------
  ;; Read status and flags
  ;;-----------------------
  op_shutter = tsd->Get_status_tsd('FIS_OBS','ShtOp')  ; '1' means shutter open
  cl_shutter = 1 - op_shutter

  pid = tsd->Get_value_tsd('FIS_OBS','PacketID')
  tm_saa = tsd->Get_value_tsd('SE','TM_SAA')

  st_bad   = tsd->Get_pix_flag_tsd('FIS_OBS','Bad')
  no_bad   = 1 - st_bad

  IF (num_ch EQ 75) THEN BEGIN
    on_calA = tsd->Get_status_tsd('FIS_OBS','CalALon')
    calA_pls_cnt = tsd->Get_value_tsd('FIS_OBS','CalPlsCntAL')
    on_boost = tsd->Get_status_tsd('FIS_OBS','LWBoostOn')
    of_boost = 1 - on_boost
    caldac = tsd->Get_value_tsd('FIS_OBS','DACCalAL')
    flcom = '_LW_'
  ENDIF ELSE BEGIN
    on_calA = tsd->Get_status_tsd('FIS_OBS','CalASon')
    calA_pls_cnt = tsd->Get_value_tsd('FIS_OBS','CalPlsCntAS')
    on_boost = tsd->Get_status_tsd('FIS_OBS','SWBoostOn')
    of_boost = 1 - on_boost
    caldac = tsd->Get_value_tsd('FIS_OBS','DACCalAS')
    flcom = '_SW_'
  ENDELSE

  on_calB = tsd->Get_status_tsd('FIS_OBS','CalBon')
  calB_pls_cnt = tsd->Get_value_tsd('FIS_OBS','CalPlsCntB')

  ; Blank & SAA statuses
  st_blank = tsd->Get_flag_tsd('FIS_OBS','Blank')
  no_blank = 1 - st_blank
  in_in_saa= tsd->Get_flag_tsd('FIS_OBS','In_SAA')
  no_in_saa= 1 - in_in_saa

  ; Quality
  qual_rp_data = tsd->Get_Quality_TSD('FIS_OBS','Qual_RP_Data')


  ;;-----------------------
  ;; Correct status bit of 'CalAS(L)on' using 'CalAPlsCnt' and duration
  ;;   - Status of pulse (short) Cal is to be set again because the timing
  ;;     is supposed to be different from the real data.
  ;;   - Status of continuous Cal is not changed.
  ;;   - Pulse Cal is regarded to be done during shutter open, while continuous Cal
  ;;     is done during shutter close.
  ;;-----------------------

  ; Find out positions of frames at which CalA turned on
  idum = where(calA_pls_cnt - shift(calA_pls_cnt, 1) NE 0, cnt)

  ;plot, on_calA, /nodata, xra=[2000,2050]   ;#for CHECK#;;;;;;;;;;;;;;;;;;;;;;;
  ;oplot, on_calA, color='0000FF'XL, psym=-2 ;#for CHECK#;;;;;;;;;;;;;;;;;;;;;;;

  IF (cnt GE 2) THEN BEGIN ; If CalA turned on more than once in the current TSD

    IF (on_calA[0]*op_shutter[0] EQ 1) THEN BEGIN ; If data begins during A Cal pulse On.
      dum = where(on_calA[0:idura-1] EQ 1, cntp)
      on_calA[0:cntp-1] = 1B
      on_calA[cntp:num_data-1L] = 0B
    ENDIF ELSE BEGIN
      tmp_idx = where(on_calA*op_shutter EQ 1, cnt_tmp) ; CalA ON but not continuous Cal
      IF (cnt_tmp GT 0) THEN on_calA[tmp_idx] = 0B
      ;on_calA[0:num_data-1L] = 0B
    ENDELSE

    ist=0 & IF (idum[0] EQ 0) THEN ist=1
    FOR i = ist, cnt-1 DO BEGIN

      ;ib = idum[i] ; until IT#4
      IF (num_ch EQ 100) THEN BEGIN
        ib = idum[i]+3 ; new for SW
        IF (ib GE num_data) THEN continue
      ENDIF ELSE BEGIN
        ib = idum[i] ; new for LW
      ENDELSE

      IF (cl_shutter[ib] EQ 1) THEN continue ; If not pulse but continuous
      ;ie = min([num_data, ib+idura-1L]) ; until IT#4
      ie = min([num_data-1, ib+idura]) ; new
      on_calA[ib:ie] = 1B
    ENDFOR

    of_calA = 1 - on_calA

  ENDIF ELSE BEGIN
    LMESSAGE, 3, 'Too small number of pulse CAL.', /CONTINUE
    return, !SUCCEED
  ENDELSE

  ;oplot, on_calA, color='FF0000'XL, psym=-4 ;#for CHECK#;;;;;;;;;;;;;;;;;;;;;;;

  ;;; To discard continuous (B) Cal data taken at transition period, special
  ;;; care is needed. for pulse (A) Cal, we can not do the same thing due to
  ;;; not many data
  ;
  idx_c = where(on_calB EQ 1, cnt)
  idx_b = where(idx_c - shift(idx_c, 1) NE  1)
  idx_e = where(idx_c - shift(idx_c,-1) NE -1)
  on_calB[idx_c[idx_b]] = 0  &  on_calB[idx_c[idx_b]+1] = 0
  on_calB[idx_c[idx_e]] = 0  &  on_calB[idx_c[idx_e]-1] = 0

  ;
  of_calA = 1 - on_calA
  of_calB = 1 - on_calB

;;-----------------------
;; Identify useful A-Cal ON data frames with flags
;;  for "Method A"
;;-----------------------
ok1_cal = op_shutter * on_calA * of_boost * no_in_saa ; '1' for Cal frame
ok2_cal = ok1_cal * no_blank                ; '1' for Cal & no_blank frame
ok3_cal = bytarr(num_ch, num_data)
FOR ch = 0, num_ch-1 DO BEGIN
  ok3_cal[ch,*] = ok2_cal[*] * no_bad[ch,*] ; '1' for Cal & no_blank & no_bad frame
ENDFOR

;;-----------------------
;; Identify useful and Cal-Off data frames with flags
;;  for "Method A"
;;-----------------------
ok1_dat = op_shutter * of_calA * of_calB * of_boost * no_blank * no_in_saa
ok3_dat = bytarr(num_ch, num_data)
FOR ch = 0, num_ch-1 do begin
     ok3_dat[ch,*] = ok1_dat[*] * no_bad[ch,*]
ENDFOR

;;-----------------------
;; Identify unusable (NG) data frames to get Cal data
;;  for "Method B"
;;-----------------------
ng_getcal_tmp = cl_shutter * on_boost * in_in_saa * st_blank ; '1' means unusable frame
ng_getcal = bytarr(num_ch, num_data)
FOR ch = 0, num_ch-1 DO BEGIN
  ng_getcal[ch,*] = ng_getcal_tmp[*] OR st_bad[ch,*] ; '1' means unusable frame for GetCal
ENDFOR

;;-----------------------
;; Make index of Cal data & Check them
;;-----------------------
idx_cal = where(ok1_cal NE 0.D, cal_cnt)
         ; idx_cal is an index of Cal data frames
         ; cal_cnt is the number of Cal data frames
idx_calg = where(flux * ok3_cal NE 0.D)
         ; idx_calg is an index of Cal data frames in good condition

IF (cal_cnt LT 3) THEN BEGIN
  LMessage, 5, 'No useful calibration data in the file.', /CONTINUE
  return, !SUCCEED
ENDIF

inum_str = where(idx_cal - shift(idx_cal, 1) NE  1, cnt_calb) ; index# of Cal start
inum_end = where(idx_cal - shift(idx_cal,-1) NE -1          ) ; index# of Cal end
                                                  ; cnt_calb is a number of Cal blocks

IF (cnt_calb LT 1) THEN BEGIN
  LMessage, 5, 'No useful calibration data in the file.', /CONTINUE
  return, !SUCCEED
ENDIF

;; Values at where each Cal lamp turned on
; AFTIME
cal_ti = ti[idx_cal[inum_str]]
; Shutter Opne/Close status
cal_shop = op_shutter[idx_cal[inum_str]]
; Cal DAC set value
cal_DAC = caldac[idx_cal[inum_str]]
; Packet ID
cal_packet_ID = pid[idx_cal[inum_str]]
; TM_SAA
cal_tmsaa = tm_saa[idx_cal[inum_str]]


;;----------------------------------------------------------
;; Derive Cal flux values
;;----------------------------------------------------------
cal_flux = dblarr(num_ch, cnt_calb) ; cnt_calb is the number of Cal data set (block) in the file
cal_err  = dblarr(num_ch, cnt_calb) ;
cal_bad  = intarr(num_ch, cnt_calb) ; -> If cal data is bad, =1

  ;;-----------------------
  ;; Loop for channel (pixel)
  ;;-----------------------
  FOR ch = 0, num_ch-1 DO BEGIN

    ;;-----------------------
    ;; Loop for Cal data
    ;;  - Get 'cal_flux' (flux value for Cal On) -
    ;;-----------------------
    cal_fl = dblarr(cnt_calb) ; cnt_calb is the number of Cal data set (block) in the file
    cal_sg = dblarr(cnt_calb)


    IF (calver EQ 4) THEN BEGIN
    ;;-----------------------
    ;; Get Cal signal value
    ;;   with 4-points fitting method
    ;;   ## Method B ##
    ;;-----------------------
      FOR j_cal = 0, cnt_calb-1 DO BEGIN
      
        IF (idx_cal[inum_str[j_cal]] GE 60) THEN BEGIN
          ii_str = idx_cal[inum_str[j_cal]] - 60 ; start frame# for 'j'th Cal calc.
        ENDIF ELSE BEGIN
          ii_str = 0
        ENDELSE
        
        IF (idx_cal[inum_end[j_cal]] LE num_data-4) THEN BEGIN
          ii_end = idx_cal[inum_end[j_cal]] + 3  ; end frame# for 'j'th Cal calc.
        ENDIF ELSE BEGIN
          ii_end = num_data - 1
        ENDELSE

        flx = reform(flux[ch, ii_str:ii_end]) ; diffed detector signal of one pixel
        ngf = reform(ng_getcal[ch, ii_str:ii_end]) ; NG frame not to be used

        ret = getcal_fit(flx, ngf, slope=slp, sigma=sgm)

        cal_fl[j_cal] = ret[0] ; Cal strength indicating responsivity
        cal_sg[j_cal] = ret[1] ; Error of Cal strength

      ENDFOR ;; End of loop for Cal

    ENDIF ELSE IF (calver EQ 3) THEN BEGIN
    ;;-----------------------
    ;; Get Cal signal value
    ;;   by integration of Cal On signals
    ;;   ## Method A ##
    ;;-----------------------
      FOR j_cal = 0, cnt_calb-1 DO BEGIN

        ii_str = idx_cal[inum_str[j_cal]] ; frame# of 'j'th cal start(On).
        ii_end = idx_cal[inum_end[j_cal]]

        ; Check if whole data in a block are bad
        ibcal = where(flux[ch, ii_str:ii_end]*ok3_cal[ch, ii_str:ii_end] NE 0.D, cnt_ok_cal_frm)

        sufc_frms = idura - 2 > 5 ; acceptable minimum frame number in a pulse cal.
                                  ; 5 for SW, 7 for LW

        IF (cnt_ok_cal_frm LE sufc_frms) THEN BEGIN
          msg = 'Not enough good Cal data in the'+strcompress(string(j_cal+1))$
                +'th cal of ch.'+strcompress(string(ch))
          cal_bad[ch, j_cal] = 1
          LMessage, 7, msg, /Continue
          Continue
        ENDIF

        ;;
        ;; Get baseline level (valtemp; mean signal value before and after Cal ON).
        ;;
        ib1 = ii_str-10  &  IF (ib1 LT 0) THEN ib1 = 0
        ie1 = ii_str-1   &  IF (ie1 LT 0) THEN ie1 = 0
        idx_ok_precal = where(flux[ch, ib1:ie1]*ok3_dat[ch, ib1:ie1] NE 0.D, cnt_ok_precal)

        IF (cnt_ok_precal GE 4) THEN BEGIN
          titemp  = ti[idx_ok_precal[cnt_ok_precal-4:cnt_ok_precal-1]+ib1]
          valtemp = flux[ch, idx_ok_precal[cnt_ok_precal-4:cnt_ok_precal-1]+ib1]

          ib2 = ii_end+5  & IF (ib2 GT num_data-1L) THEN ib2 = num_data-1L
          ie2 = ii_end+15 & IF (ie2 GT num_data-1L) THEN ie2 = num_data-1L
          idx_ok_postcal = where(flux[ch, ib2:ie2]*ok3_dat[ch, ib2:ie2] NE 0.D, cnt_ok_postcal)

          IF (cnt_ok_postcal GE 4) THEN BEGIN
            titemp  = [titemp, ti[idx_ok_postcal[0:3]+ib2]]
            temp    = flux[ch, idx_ok_postcal[0:3]+ib2]
            valtemp = transpose([transpose(valtemp),transpose(temp)])
          ENDIF
        ENDIF ELSE BEGIN
          ib2 = ii_end+5  & IF (ib2 GT num_data-1L) THEN ib2 = num_data-1L
          ie2 = ii_end+15 & IF (ie2 GT num_data-1L) THEN ie2 = num_data-1L
          idx_ok_postcal = where(flux[ch, ib2:ie2]*ok3_dat[ch, ib2:ie2] NE 0.D, cnt_ok_postcal)

          IF (cnt_ok_postcal GE 4) THEN BEGIN
            titemp  = ti[idx_ok_postcal[0:3]+ib2]
            valtemp = flux[ch, idx_ok_postcal[0:3]+ib2]
          ENDIF
        ENDELSE

        ;;
        ;; Get Cal signal and error levels
        ;;
        IF (n_elements(valtemp) GT 4) THEN BEGIN
          cal_fl[j_cal] = mean(flux[ch, ibcal+ii_str]) - mean(valtemp)
          cal_sg[j_cal] = stddev(valtemp)
          ;<- Now, error is the deviation of background (before and after the Cal)
        ENDIF

      ENDFOR ;; End of loop for Cal

    ENDIF

    ;;
    ;; Set quality of Cal lamp data
    ;;
    FOR j_cal = 0, cnt_calb-1 DO BEGIN

      ; start frame# of quality
      IF (j_cal GE 1) THEN BEGIN
        strf = (idx_cal[inum_str[j_cal]] + idx_cal[inum_str[j_cal-1]]) / 2
      ENDIF ELSE BEGIN
        strf = 0
      ENDELSE
      ; end frame# of quality
      IF (j_cal LT cnt_calb-1) THEN BEGIN
        endf = (idx_cal[inum_str[j_cal]] + idx_cal[inum_str[j_cal+1]]) / 2 - 1
      ENDIF ELSE BEGIN
        endf = num_data - 1
      ENDELSE

      dum = where(cal_sg[j_cal] LE 0, cnt_qual3)
      IF (cnt_qual3 GT 0) THEN qual_rp_data[ch, strf:endf] = 3 ; unusable
      dum = where(cal_sg[j_cal] GE 10e-5, cnt_qual2)
      IF (cnt_qual2 GT 0) THEN qual_rp_data[ch, strf:endf] = 2 ; low quality
      dum = where((cal_sg[j_cal] GE 6e-5) AND (cal_sg[j_cal] LT 10e-5), cnt_qual1)
      IF (cnt_qual1 GT 0) THEN qual_rp_data[ch, strf:endf] = 1 ; medium quality

    ENDFOR ;; End of loop for setting quality

    ;; copy
    cal_flux[ch, *] = cal_fl
    cal_err[ch, *]  = cal_sg

    ;; Set bad flag for invalid (negative) Cal data
    ng_flux_idx = where(cal_fl LE 0.D, cnt_ng_cal)
    IF (cnt_ng_cal GT 0) THEN cal_bad[ch, ng_flux_idx] = 1

  ENDFOR ;; End of channel loop

  ; Set quality of Cal lamp data
  ret = tsd->Set_Quality_TSD('FIS_OBS','Qual_RP_Data', qual_rp_data)


  ;; # for Check and Debug ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  ch = 31
;  plot, ti, flux[ch,*], yrange=[0, 0.005], psym=-1;, xra=[ti[ii_str]-1, ti[ii_str]+1]
;  ; the last Cal pulse
;  oplot, ti[ibcal+ii_str], flux[ch, ibcal+ii_str], psym=-4, color='0000FF'XL ; Red
;  ; Cal table data values
;  oplot, cal_ti[*], cal_flux[ch, *], psym=2, color='FF0000'XL ; Blue

;;----------------------------------------------------------
;;  Write into CAL table files (FITS format)
;;----------------------------------------------------------
  ; Define structure
  _A = {aftime:0D, flux:fltarr(num_ch), err:fltarr(num_ch), shutop:0B, $
        bad:bytarr(num_ch), calAdac:0B, pid:0B, tm_saa:0D}

  ; Make subdirectory to save table files
   ret = file_test('CALTABLE', /dir)
   IF (ret EQ 0) THEN BEGIN
     file_mkdir, 'CALTABLE'
     LMessage, 5,' CALTABLE directory generated.', /Continue
   ENDIF

  ; Assortment (Find ID numbers for CAL files)
  tidn = fix(cal_ti[*]/fldiv) ; cal_time is in AF-time; fldiv is an interval of the file.
                              ; tidn is the ID number of CAL files to be generated/updated
  idx_idc = where(tidn - shift(tidn, 1)  NE 0, cntds) ; index# of changing points of tidn
                                               ; cntds is the number of changing points of tidn

  ; Check if ID number of table file is valid.
  IF (total(tidn) EQ 0) THEN BEGIN ; If tidn[*] EQ 0
    LMessage, 5, 'CalTable file name is NOT VALID.', /Continue
    Return, !FAIL
  ENDIF

  IF (cntds EQ 0) THEN BEGIN ; If all data is to be in one file,
    cntds = 1          ; only one file will be generated/updated
    idx_idc = [0, N_elements(tidn)]
  ENDIF

  ; cntds is the number of CAL files to be generated/updated
  FOR i = 0, cntds-1 DO BEGIN

    fileID = tidn[idx_idc[i]]
    iis = idx_idc[i]
    IF (i LT cntds-1) THEN iie = idx_idc[i+1]-1 $
                      ELSE iie = N_elements(tidn)-1

    ;; Make contents of FITS table
    num = iie - iis + 1
    A = replicate(_A, num)
    ;
    A.aftime  = cal_ti[iis:iie]
    A.flux    = cal_flux[*, iis:iie]
    A.err     = cal_err[*, iis:iie]
    A.shutop  = cal_shop[iis:iie]
    A.bad     = cal_bad[*, iis:iie]
    A.calAdac = cal_DAC[iis:iie]
    A.pid     = cal_packet_ID[iis:iie]
    A.tm_saa  = cal_tmsaa[iis:iie]

    ; See whether CAL file already exists
    flnm = './CALTABLE/CAL'+flcom+string(fileID, format='(i4.4)')+'.fits'
    dum = file_search(flnm)

    IF (dum EQ '') THEN BEGIN    ; the file does not exist

      Acomment = '    A new CAL table file generated. '
      write_cal_fits, A, acomment, flnm, CALVER=calver

      LMessage, 5, acomment+flnm, /PRINT

    ENDIF ELSE BEGIN    ; the file does exist and thus should be updated

      CAL = MRDFITS(flnm, 1, STATUS=status)
      csz = size(CAL)
      A = [CAL, A]
      A = A[sort(A.aftime)]
      idx = where(A.aftime - shift(A.aftime, 1) NE 0.D)
      A = A[idx]
      asz = size(A)
      Acomment = '    The CAL table is updated. '
      IF (csz[1] EQ asz[1]) THEN Acomment='    The CAL table remains the same.'
      write_cal_fits, A, acomment, flnm

      LMessage, 5, acomment+flnm, /PRINT

    ENDELSE

  ENDFOR

  Return, !SUCCEED

END
