; $Id: im_detector.pro,v 1.4 2006/05/11 05:39:46 do Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	im_detector.pro
; PURPOSE:
;	process one detector scan
; CATEGORY:
;	Function.
; CALLING SEQUENCE:
;	Result = im_detector( det, flux, flag, spheric, mapset, mapinfo, $
;		    plot=plot, scannr=scannr, _extra=_extra )
;
; INPUTS:
;	det		detector number
;	flux		flux forthis det
;	flag		flag for this det
;	spheric 	positional information
;	mapinfo 	map information
;	scannr		number of this scan
;	plot		various plot options
;	_extra		for plotting
; IN/OUT:
;	mapset		update of contents
; RETURN
;	!SUCCEED if function ended successfully.
;	!FAIL in failure.
; RESTRICTIONS:
;
; PROCEDURE:
;	Determine detector position (im_detpos), destripe (im_destripe)
;	and coadd (im_detcoad).
;
; EXAMPLE:
;
; REFERENCE:
;	Baba et al. 2002 Nov. 21, ``ASTRO-F/FIS Survey Data Structure"
;
; MODIFICATION HISTORY:
;	Oct. 2004		Written by Do Kester
;-

FUNCTION im_detector, det, flux, wgt, flag, spheric, mapset, mapinfo, $
	    plot=plot, scannr=scannr, _extra=_extra

COMPILE_OPT IDL2

;   ON_ERROR 2

    result = !SUCCEED

    result = result || im_detpos( det, spheric, mapinfo, x, y )

;if ( det EQ 1 OR det EQ 15  ) then begin
; q = indgen( n_elements( x) / 100 ) * 100
; q4 = indgen( n_elements( x) / 400 ) * 400
; oplot, x[q4], y[q4], ps=1, color = det EQ 1 ? 2 : 3
; oplot, x[q4], y[q4], ps=2, color = det EQ 1 ? 2 : 3
; if ( det EQ 1 ) then begin
;  rr = im_detpos( 0, spheric, mapinfo, x0, y0 )
;  oplot, x0[q], y0[q], ps=1
;  oplot, x0[q4], y0[q4], ps=1
;  for k=1,n_elements(x)/400-1 do begin
;      oplot, [x0[q4[k]-200],x0[q4[k]]], [y0[q4[k]-200],y0[q4[k]]]
;  endfor
;endif
;endif

    b = where( flag EQ 0, nb )
    IF ( nb EQ 0 ) THEN begin
;	print, string( "Nothing to do for det =", det )
	return, result		    ;; nothing to do
    endif

    if ( plot EQ 3 ) THEN oplot, x, y, ps=3

    if ( mapinfo.destr ) then $
	result = result || $
		 im_destripe( flux, flag, x, y, mapset, mapinfo, det, $
			      scannr, plot=plot )

    ff = flux[b]
    wt = wgt[b]
    xx = x[b]
    yy = y[b]
    result = result || keyword_set( mapinfo.drizzle ) ? $
		im_detcoad_pointdrizzle( ff, wt, xx, yy, mapset, mapinfo ) : $
		im_detcoad( ff, wt, xx, yy, mapset, mapinfo )

    return, result

END


