;+
; NAME:
;   apfactor
;
; PIRPOSE:
;   output the aperture correction factor
;
; MODIFICATION HISTORY:
;   written by M.Shirahata, 2007/06/04
;      2007/08/01 M.Shirahata  bug fix
;-

PRO apfactor, band, radius, factor, factor_error

; Display the usage
    if (n_params() eq 0) then begin
       print, "# USAGE #"
       print, "apfactor, band, radius, factor"
       print, "   band    : 0=N60, 1=Wide-S, 2=Wide-L, 3=N160"
       print, "   radius  : [arcmin]"
       return
    endif


; Band selection and Aperture correction file Open
    root = GETENV('FISDR_ROOT')
    if (band eq 0) then begin
        print, '--- N60 band ---'
        line = 69
        OpenR, lun_in, root+'/slowscan/lib/COG_N60.txt', /get_lun, Error=oper
    endif else if (band eq 1) then begin
        print, '--- Wide-S band ---'
        line = 69
        OpenR, lun_in, root+'/slowscan/lib/COG_WideS.txt', /get_lun, Error=oper
    endif else if (band eq 2) then begin
        print, '--- Wide-L band ---'
        line = 49
        OpenR, lun_in, root+'/slowscan/lib/COG_WideL.txt', /get_lun, Error=oper
    endif else if (band eq 3) then begin
        print, '--- N160 band ---
        line = 59
        OpenR, lun_in, root+'/slowscan/lib/COG_N160.txt', /get_lun, Error=oper
    endif else begin
        print, 'Please select the photometric band !'
        print, '   1: N60'
        print, '   2: Wide-S'
        print, '   3: Wide-L'
        print, '   4: N160'
        return
    endelse

    if(oper eq 0) then begin
        print, 'Read Aperture correction table'
    endif else begin
        print, 'Cannot read the Aperture correction table !!!'
        return
    endelse

; Make array
    aptable = dblarr(3,line)
    x = dblarr(line)
    y1 = dblarr(line)
    y2 = dblarr(line)

; Read Data from input table
    ReadF, lun_in, aptable

; File Close
    Free_lun, lun_in

; Fit with spline
    x = aptable[0,*]
    y1 = aptable[1,*]
    y2 = aptable[2,*]

    factor = SPLINE(x, y1, double(radius), /double)
    factor_error = SPLINE(x, y2, double(radius), /double)

    print, 'Aperture correction factor at r=', Strcompress(radius), $
      ' :', Strcompress(factor), ' +-', Strcompress(factor_error)

end
