;+
; NAME:
;   ss_eg_lpf
;
; PURPOSE:
;   median/boxcar-smooth low-pass filtering to extract DC/slow components
;
; CALLING SEQUENCE:
;
; MODIFICATION HISTORY:
;   Written by S.Matsuura, 2007/09/06 : based on ss_eg_slrmv.pro
;
;   2007/09/06 S.Matsuura, edge handling for boxcar smooth. 
;   2007/09/09 S.Matsuura, modified edge handling, increased loop count limit, 
;		and median filter is used for the initial loop for smoothing. 
;   2007/09/11 S.Matsuura, changed position of loop count limit. 
;
;-
pro ss_eg_lpf, x, det, err, w_sm, flt_type, n_line, n_pix, w_pix, bad_flg, $
		det_sm, err_sm


    if (flt_type ne 0 and flt_type ne 1) then begin
	print, 'No filter type is specified. Boxcar smoothing will be used by default.'
	flt_type = 0
    endif


    ; boxcar smoothing and sigma clipping

    if (flt_type eq 0) then begin
    print,'Stray light removal: boxcar smoothing and sigma clipping method is selected.'

    det_sm = dblarr(n_pix,n_line+w_sm)
    err_sm = dblarr(n_pix,n_line+w_sm)
    det_p = dblarr(n_line+w_sm)
    err_p = dblarr(n_line+w_sm)
    det_res = dblarr(n_line+w_sm)
    hw_sm = fix(w_sm/2)

    for pix = 0, n_pix-1 do begin
    if (bad_flg[pix] eq 0) then begin

        det_p[hw_sm:n_line+hw_sm-1] = det[pix,0:n_line-1]
        det_p[0:hw_sm-1] = det[pix,0]
        det_p[n_line+hw_sm:n_line+2*hw_sm-1] = det[pix,n_line-1]
        err_p[hw_sm:n_line+hw_sm-1] = err[pix,0:n_line-1]
        err_p[0:hw_sm-1] = err[pix,0]
        err_p[n_line+hw_sm:n_line+2*hw_sm-1] = err[pix,n_line-1]

	clip_cnt = 1
	loop_cnt = 0
	while (clip_cnt gt 0) do begin
        nz_idx = where(det_p ne 0 and err_p ne 0, nz_idx_cnt)
        if (nz_idx_cnt gt 0) then begin

	    if (loop_cnt eq 0) then begin
		; median filter to get initial smooth curve
		det_sm[pix,nz_idx] = median(det_p[nz_idx],w_sm, /double)
                err_sm[pix,nz_idx] = 0
	    endif else begin
		; smooth filter for the next loop
	        det_sm[pix,nz_idx] = smooth(det_p[nz_idx],w_sm,/edge_truncate,/NaN)
                err_sm[pix,nz_idx] = 0
	    endelse

	    ; 3-sigma clipping
	    det_res = det_p - det_sm[pix,*]
	    det_res_std = stddev(det_res,/NaN)
	    clip_idx = where(abs(det_res) gt 3*det_res_std, clip_cnt)
	    if (clip_cnt gt 0) then begin
		det_p[clip_idx] = det_sm[pix,clip_idx]
	    endif

        endif
	loop_cnt = loop_cnt + 1
	if (loop_cnt gt 50) then clip_cnt = 0
	endwhile
	; print, 'Loop counts: ', loop_cnt

    endif
    det_p[*] = 0D
    err_p[*] = 0D
    endfor

    det_sm = det_sm[*,hw_sm:n_line+hw_sm-1]
    err_sm = err_sm[*,hw_sm:n_line+hw_sm-1]
    det_sm = det_sm[*,0:n_line-1]
    err_sm = err_sm[*,0:n_line-1]

    endif


    ; median smoothing

    if (flt_type eq 1) then begin
    print,'Stray light removal: boxcar smoothing and sigma clipping method is selected.'

    det_sm = dblarr(n_pix,n_line+w_sm)
    err_sm = dblarr(n_pix,n_line+w_sm)
    det_p = dblarr(n_line+w_sm)
    err_p = dblarr(n_line+w_sm)
    hw_sm = fix(w_sm/2)

    for pix = 0, n_pix-1 do begin
    if (bad_flg[pix] eq 0) then begin
        det_p[hw_sm:n_line+hw_sm-1] = det[pix,0:n_line-1]
        det_p[0:hw_sm-1] = det[pix,0]
        det_p[n_line+hw_sm:n_line+2*hw_sm-1] = det[pix,n_line-1]
        err_p[hw_sm:n_line+hw_sm-1] = err[pix,0:n_line-1]
        err_p[0:hw_sm-1] = err[pix,0]
        err_p[n_line+hw_sm:n_line+2*hw_sm-1] = err[pix,n_line-1]
        nz_idx = where(det_p ne 0 and err_p ne 0, nz_idx_cnt)
        if (nz_idx_cnt gt 0) then begin
        det_sm[pix,nz_idx] = median(det_p[nz_idx],w_sm,/double)
        err_sm[pix,nz_idx] = 0
        endif
    endif
    det_p[*] = 0D
    err_p[*] = 0D
    endfor

    det_sm = det_sm[*,hw_sm:n_line+hw_sm-1]
    err_sm = err_sm[*,hw_sm:n_line+hw_sm-1]
    det_sm = det_sm[*,0:n_line-1]
    err_sm = err_sm[*,0:n_line-1]

    endif


end
