; Copyright(C) 2005, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;   fisdr
;
; PURPOSE:
;   Initialize System variables relevant for FIS data reduction
;
; CATEGORY:
;   Data definition
;
; CALLING SEQUENCE:
;   fisinit
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None
;
; RESTRICTIONS:
;   None
;
; COMMENTS:
;   None
;
; DESCRIPTION:
;   None
;
; EXAMPLE:
;   None
;
; SIDE EFFECTS:
;   None
;
; REFERENCE:
;   None
;
; MODIFICATION HISTORY:
;   Version 1.0   2005/06/08  IY First draft
;   Version 1.1   2005/06/15  IY !MSG_LEVEL (= 5) is implemented.
;   Version 1.2   2007/02/20  IY Device, retain=2 is included.
;   Version 1.3   2007/08/21  IY !PATH reset to overridge users preference
;                             IY prompt changed
;   Version 1.4   2007/09/09  IY Can include users IDL setup via
;                             IY FISDR_USER_SETUP implemented
;   Version 1.5   2007/09/11  IY !PATH is set up more properly
;   Version 1.5.1 2007/09/11  IY Bug fix to keep compatibility on Unix/Windows
;   Version 1.5.2 2007/09/11  IY Further minor correction.
;
;-

PRO fisdr
    ; Not intended to be called directly by regular users
    COMPILE_OPT idl2, hidden
    ON_ERROR, 2                 ; Return to caller if an error occurs

    ; Include users setup
    user_setup_file = GETENV('FISDR_USER_SETUP')
    if (FILE_TEST(user_setup_file)) then begin
       nl = FILE_LINES(user_setup_file)
       user_setup_command = STRARR(nl)
       OpenR, lun, user_setup_file, /Get_LUN
          ReadF, lun, user_setup_command
       Free_Lun, lun
       for i = 0, nl - 1 do ret = EXECUTE(user_setup_command[i])
    endif

    ; Define some system variables.
    ; This part can not move to util::INIT procedure...
    !QUIET = 1B                 ; Be quiet if !QUIET is non zero.
    DEFSYSV, '!DEBUG', EXISTS = e
    IF NOT e THEN DEFSYSV, '!DEBUG', 0B
    DEFSYSV, '!SUCCEED', EXISTS = e
    IF NOT e THEN DEFSYSV, '!SUCCEED', 0S, 1 ; Read_Only
    DEFSYSV, '!FAIL', EXISTS = e
    IF NOT e THEN DEFSYSV, '!FAIL', -1S, 1 ; Read_Only
    DEFSYSV, '!TRUE', EXISTS = e
    IF NOT e THEN DEFSYSV, '!TRUE', 1B, 1 ; Read_Only
    DEFSYSV, '!FALSE', EXISTS = e
    IF NOT e THEN DEFSYSV, '!FALSE', 0B, 1 ; Read_Only
    DEFSYSV, '!IDL_TRUE', EXISTS = e
    IF NOT e THEN DEFSYSV, '!IDL_TRUE', 1B, 1 ; Read_Only
    DEFSYSV, '!IDL_FALSE', EXISTS = e
    IF NOT e THEN DEFSYSV, '!IDL_FALSE', 0B, 1 ; Read_Only

    DEFSYSV, '!STDIN', EXISTS = e
    IF NOT e THEN DEFSYSV, '!STDIN', 0S, 1 ; Read_Only
    DEFSYSV, '!STDOUT', EXISTS = e
    IF NOT e THEN DEFSYSV, '!STDOUT', -1S, 1 ; Read_Only
    DEFSYSV, '!STDERR', EXISTS = e
    IF NOT e THEN DEFSYSV, '!STDERR', -2S, 1 ; Read_Only

; Read FISDR root directory
    root = GETENV('FISDR_ROOT')

; Set IDL_PATH again (overridge preference setting)
    !PATH = EXPAND_PATH('+' + root) + $
      ':' + EXPAND_PATH(PREF_GET('IDL_PATH')) + $
      ':' + EXPAND_PATH('<IDL_DEFAULT>')

; CCF related variables   
    DEFSYSV, '!CCF_PATH', EXISTS = e
    IF NOT e THEN DEFSYSV, '!CCF_PATH', root+'/CCF/data/', 0 ; Read/Write

    DEFSYSV, '!CCF_MODE', EXISTS = e
    IF NOT e THEN DEFSYSV, '!CCF_MODE', 'TEST', 0 ; Read/Write

; Message level
    DEFSYSV, '!MSG_LEVEL', EXISTS = e
    IF NOT e THEN DEFSYSV, '!MSG_LEVEL', 5, 0 ; Read/Write

; Device set up to recover the plots when active
    Device, retain=2

; Command prompt
    !PROMPT = 'FISDR> '

; State that this procedure is called
    MESSAGE, 'Environment setup done.', /CONTINUE

END
