; $Id: bte__define.pro,v 1.22 2007/06/27 07:09:06 cyamauch Exp $
;
; Copyright(C) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; CLASS_NAME:
;   BTE - Binary Table Extension of FITS, inherited from HDU
;
; PURPOSE:
;   Define the structure for HDU, which is a part of TSD.
;
; CATEGORY:
;   Data definition
;
; CALLING SEQUENCE:
;   None
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None
;
; RESTRICTIONS:
;   None
;
; COMMENTS:
;   None
;
; DESCRIPTION:
;   See 'ASTRO-F FIS/Survey Data Structure' in detail.
;
; EXAMPLE:
;   None
;
; SIDE EFFECTS:
;   None
;
; MODIFICATION HISTORY:
;   Written by: Hajime Baba, June 2002.
;
;-

;----------------------------------------------------------------------------
; Functions which are internal use only
;----------------------------------------------------------------------------

FUNCTION bte::getModuleName
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    IF STRCMP(self.extname, 'FIS_OBS', /FOLD_CASE) THEN BEGIN
        RETURN, self.datatype
    ENDIF ELSE BEGIN
        RETURN, self.extname
    ENDELSE
END

FUNCTION bte::GetTableDef
    COMPILE_OPT hidden
    modname = self->getModuleName()
;     cmd = "COMMON " + modname + "_block"
;     IF EXECUTE(cmd) THEN BEGIN
;         cmd = "tmp=STRUPCASE(TRANSPOSE(" + modname + "[" + i + ",*]))"
;         IF NOT EXECUTE(cmd) THEN MESSAGE, 'Exec error: ' + cmd
;         RETURN, TEMPORARY(tmp)
;     ENDIF
    CASE STRUPCASE(modname) OF
        'FIS_SW': COMMON fis_sw_block
        'FIS_LW': COMMON fis_lw_block
        'ISO':    COMMON iso_block
        'FIS_HK': COMMON fis_hk_block
        'IRC_HK': COMMON irc_hk_block
        'HK_2':   COMMON hk_2_block
        'AOCU':   COMMON aocu_block
        'GADS':   COMMON gads_block
        'ADS':    COMMON ads_block
        'PR':     COMMON pr_block
        'SE':     COMMON se_block
        ELSE: MESSAGE, 'Unknown modname: ' + modname
    ENDCASE
    CASE STRUPCASE(modname) OF
        'FIS_SW': tmp = fis_sw
        'FIS_LW': tmp = fis_lw
        'ISO':    tmp = iso
        'FIS_HK': tmp = fis_hk
        'IRC_HK': tmp = irc_hk
        'HK_2':   tmp = hk_2
        'AOCU':   tmp = aocu
        'GADS':   tmp = gads
        'ADS':    tmp = ads
        'PR':     tmp = pr
        'SE':     tmp = se
        ELSE: MESSAGE, 'Unknown modname: ' + modname
    ENDCASE
    RETURN, TEMPORARY(tmp)
END

FUNCTION bte::TagNames
    COMPILE_OPT hidden
    tmp = self->GetTableDef()
    RETURN, STRUPCASE(tmp[*].ttype)
END
FUNCTION bte::TTypes
    COMPILE_OPT hidden
    tmp = self->GetTableDef()
    RETURN, STRUPCASE(tmp[*].ttype)
END
FUNCTION bte::TForms
    COMPILE_OPT hidden
    tmp = self->GetTableDef()
    RETURN, STRUPCASE(tmp[*].tform)
END
FUNCTION bte::TUnits
    COMPILE_OPT hidden
    tmp = self->GetTableDef()
    RETURN, STRUPCASE(tmp[*].tunit)
END
FUNCTION bte::TDims
    COMPILE_OPT hidden
    tmp = self->GetTableDef()
    RETURN, STRUPCASE(tmp[*].tdim)
END
FUNCTION bte::Comments
    tmp = self->GetTableDef()
    RETURN, STRUPCASE(tmp[*].comment)
END
FUNCTION bte::TElem2d
    tmp = self->GetTableDef()
    RETURN, tmp[*].telem
END
FUNCTION bte::FValues
    COMPILE_OPT hidden
    tforms = self->TForms()
    fvalues = STRARR(N_ELEMENTS(tforms))
    FOR i=0,N_ELEMENTS(tforms)-1 DO BEGIN
        fvalues[i] = STRING(StructForm(tforms[i]))
    ENDFOR
    RETURN, fvalues
END

FUNCTION bte::Get_ArraySizeType, var
    COMPILE_OPT hidden
    sz = SIZE(var)
    CASE sz[sz[0]+1] OF
        1:    tname = 'BYT'
        2:    tname = 'INT'
        3:    tname = 'LON'
        4:    tname = 'FLT'
        5:    tname = 'DBL'
        6:    tname = 'COMPLEX'
        7:    tname = 'STR'
        8:    tname = 'Structure' ; Note!!!
        9:    tname = 'DCOMPLEX'
        10:   tname = 'PTR'
        11:   tname = 'OBJ'
        12:   tname = 'UINT'
        13:   tname = 'ULON'
        14:   tname = 'LON64'
        15:   tname = 'ULON64'
        ELSE: tname = '(Undefined)'
    ENDCASE
    type = tname + 'ARR('
    FOR i=1,sz[0] DO BEGIN
        type = type + STRTRIM(STRING(sz[i]),2)
        IF i NE sz[0] THEN type = type + ',' $
        ELSE               type = type + ')'
    ENDFOR
    RETURN, type
END

; multi-bits, start: i, width: n
;
; MSB  01234567  LSB
;     +--------+
;         ^       <- i (0..7)
;         +-+     <- n (0..8)
;
; Really? Check it.
;
FUNCTION bte::Get_ith_Bit_OF_Bytarr, array, i, n
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    IF (i LT 0 OR i GT 7) THEN BEGIN
        MESSAGE, 'index should be between 0 and 7.'
        RETURN, !FAIL
    ENDIF
    IF NOT KEYWORD_SET(n) THEN n = 1
    IF (n LT 0 OR n GT 8) THEN BEGIN
        MESSAGE, 'width should be between 0 and 8.'
        RETURN, !FAIL
    ENDIF
;    DPRINT, "n=" + string(n)
    RETURN, BYTE(ISHFT(array, -(8-1-i-n+1)) AND BYTE(2^n-1))
END

; multi-bits, start: i, width: n
;
; MSB  01234567  LSB
;     +--------+
;         ^       <- i
;         +-+     <- n
;
; Really? Check it.
;
FUNCTION bte::Set_ith_Bit_OF_Bytarr, array, val, i, n
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    ; check size of argument array (val)
    IF N_ELEMENTS(val) NE N_ELEMENTS(array) THEN BEGIN
        MESSAGE, 'Error: Mismatch argument type: val ', /CONTINUE
        MESSAGE, 'Type of arg must be: ' + self->Get_ArraySizeType(array)
        RETURN, !FAIL
    ENDIF
    IF (i LT 0 OR i GT 7) THEN BEGIN
        MESSAGE, 'index should be between 0 and 7.'
        RETURN, !FAIL
    ENDIF
    IF NOT KEYWORD_SET(n) THEN n = 1
    IF (n LT 0 OR n GT 8) THEN BEGIN
        MESSAGE, 'width should be between 0 and 8.'
        RETURN, !FAIL
    ENDIF

    ; make bitmask
    bitmask = NOT(ISHFT(BYTE(2^n-1), 8-1-i-n+1))
    ; (1) zero-clear (8-1-i)-th bit (array AND bitmask)
    ; (2) set val to (8-1-i)-th bit (OR ISHFT(BYTE(val AND 2^n-1), 8-1-i-n+1))
    newarr = BYTE((array AND bitmask) OR ISHFT(BYTE(val AND 2^n-1), 8-1-i-n+1))

    ; return two-dimensional array
    RETURN, newarr
END


;----------------------------------------------------------------------------
; Below is the definitions of BTE specific functions.
; Some methods are overrided by the methods of decendant class
; inherited from  BTE.
;----------------------------------------------------------------------------

FUNCTION bte::read, lun, tsdver, RANGE=range
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    LMESSAGE, 7, "Reading BTE HDU(" + self.extname + ")...", /PRINT

    start = SYSTIME(/SECONDS)
    ; Read the data
    data = MRDFITS(lun, 0, header, RANGE=range, STATUS=status, /SILENT)
    IF status NE 0 THEN RETURN, !FAIL
;    PRINT, "Elapsed time: ", SYSTIME(/SECONDS) - start, " s"

    self.header = PTR_NEW(header)
    IF SIZE(data, /TNAME) EQ 'STRUCT' THEN BEGIN
        self.data = PTR_NEW(data)
    ENDIF
    self.extver = self->Get_Header_Keyword('EXTVER')
    IF self.extver EQ 0 THEN self.extver = tsdver
;    IF N_TAGS(data) GT 0 THEN DPRINT, STRJOIN(TAG_NAMES(data), ', ')
    RETURN, !SUCCEED
END

FUNCTION bte::write, outfile
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    LMESSAGE, 7, "Writing BTE HDU(" + self.extname + ")...", /PRINT

    ; Write the data
    start = SYSTIME(/SECONDS)
    IF PTR_VALID(self.data) THEN BEGIN
        ; set indices of bits column: bit_cols and nbit_cols
        ; tforms = self->TForms()
        tforms = STRTRIM(FXPAR(*self.header,"TFORM*"),2)
        bit_cols = WHERE(STRMATCH(tforms, "*X"),count) + 1
    IF count NE 0 THEN BEGIN
            nbit_cols = STRARR(N_ELEMENTS(bit_cols))
            tmp = tforms[bit_cols-1]
            FOR i=0,N_ELEMENTS(tmp)-1 DO BEGIN
                nbit_cols[i] = STRMID(tmp[i], 0, STRLEN(tmp[i])-1)
            ENDFOR
    ENDIF
        logical_cols = WHERE(STRMATCH(tforms, "*L"),count) + 1
        IF count EQ 0 THEN logical_cols = [0]

        ; write BTE header and data
        MWRFITS, *self.data, outfile, *self.header, /NO_COMMENT, /SILENT, BIT_COLS=bit_cols, NBIT_COLS=nbit_cols, LOGICAL_COLS=logical_cols
    ENDIF ELSE BEGIN
        ; write BTE header only
        FXBCREATE, lun, outfile, *self.header
        FXBFINISH, lun
    ENDELSE
;    PRINT, "Elapsed time: ", SYSTIME(/SECONDS) - start, " s"

    RETURN, !SUCCEED
END

FUNCTION bte::create, naxis2
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    DPRINT, "Creating BTE HDU(" + self.extname + ")..."
    ttypes = self->TTypes()
    tforms = self->TForms()
    tunits = self->TUnits()
    tdims  = self->TDims()
    comments = self->Comments()
    telem2d = self->TElem2d()

    ; Create BTE header
    FXBHMAKE, header, naxis2, self.extname, /INITIALIZE, /DATE, EXTVER=self.extver
    self.header = PTR_NEW(header)

    ; Create Data structure
    IF naxis2 GT 0 THEN BEGIN
        fnames = self->TagNames()
        fvalues = self->FValues()
        tmp_struct = MRD_STRUCT(fnames, fvalues, naxis2, /SILENT)
        self.data = PTR_NEW(tmp_struct)

        FOR i=0,N_ELEMENTS(ttypes)-1 DO BEGIN
            IF STRCMP(tunits[i], '') THEN FXBADDCOL, index, header, tmp_struct[0].(i), ttypes[i], comments[i] $
            ELSE FXBADDCOL, index, header, tmp_struct[0].(i), ttypes[i], comments[i], TUNIT=tunits[i]
        ENDFOR

        FOR i=0,N_ELEMENTS(tdims)-1 DO BEGIN
            IF NOT STRCMP(tdims[i], '') THEN FXADDPAR, header, 'TDIM' + STRTRIM(STRING(i+1),2), tdims[i], ''
        ENDFOR

        FOR i=0,N_ELEMENTS(telem2d)-1 DO BEGIN
            IF ( PTR_VALID(telem2d[i]) ) THEN BEGIN
                telements = ''
                FOR j=0,N_ELEMENTS(*(telem2d[i]))-1 DO BEGIN
                    telements = telements + STRUPCASE( (*(telem2d[i]))[j] )
                    IF ( j NE N_ELEMENTS(*(telem2d[i]))-1 ) THEN telements = telements + ','
                ENDFOR
                FXADDPAR, header, 'TELEM' + STRTRIM(STRING(i+1),2), telements, ''
            ENDIF
        ENDFOR

        tmp_tforms = STRTRIM(FXPAR(header,"TFORM*"),2)
        FOR i=0,N_ELEMENTS(tforms)-1 DO BEGIN
            IF NOT STRCMP(tforms[i],tmp_tforms[i]) THEN BEGIN
                FXADDPAR, header, 'TFORM' + STRTRIM(STRING(i+1),2), tforms[i], ''
            ENDIF
        ENDFOR

        self.header = PTR_NEW(header)

    ENDIF ELSE BEGIN
        self.data = PTR_NEW()
    ENDELSE

    RETURN, !SUCCEED
END

FUNCTION bte::Get_Value_Members
    COMPILE_OPT hidden
    RETURN, self->TTypes()
END

FUNCTION bte::_Get_Status_Members
    COMPILE_OPT hidden
    members = [ '' ]
    ttypes = self->TTypes()
    tmp = WHERE(STRMATCH(ttypes, 'STATUS', /FOLD_CASE) EQ 1)
    IF tmp NE -1 THEN BEGIN
        i = tmp[0]
        telem2d = self->TElem2d()
        IF PTR_VALID(telem2d[i]) THEN members = *(telem2d[i])
    ENDIF
    RETURN, members
END
FUNCTION bte::Get_Status_Members
    COMPILE_OPT hidden
    RETURN, self->_Get_Status_Members()
END

FUNCTION bte::_Get_Flag_Members
    COMPILE_OPT hidden
    members = [ '' ]
    ttypes = self->TTypes()
    tmp = WHERE(STRMATCH(ttypes, 'FLAG', /FOLD_CASE) EQ 1)
    IF tmp NE -1 THEN BEGIN
        i = tmp[0]
        telem2d = self->TElem2d()
        IF PTR_VALID(telem2d[i]) THEN members = *(telem2d[i])
    ENDIF
    RETURN, members
END
FUNCTION bte::Get_Flag_Members
    COMPILE_OPT hidden
    RETURN, self->_Get_Flag_Members()
END

FUNCTION bte::_Get_Pix_Flag_Members
    COMPILE_OPT hidden
    members = [ '' ]
    ttypes = self->TTypes()
    tmp = WHERE(STRMATCH(ttypes, 'PIX_FLAG', /FOLD_CASE) EQ 1)
    IF tmp NE -1 THEN BEGIN
        i = tmp[0]
        telem2d = self->TElem2d()
        IF PTR_VALID(telem2d[i]) THEN members = *(telem2d[i])
    ENDIF
    RETURN, members
END
FUNCTION bte::Get_Pix_Flag_Members
    COMPILE_OPT hidden
    RETURN, self->_Get_Pix_Flag_Members()
END

FUNCTION bte::_Get_Quality_Members
    COMPILE_OPT hidden
    members = REPLICATE({s0, name:'', bits:0}, 1)
    ttypes = self->TTypes()
    tmp = WHERE(STRMATCH(ttypes, 'QUALITY', /FOLD_CASE) EQ 1)
    IF tmp NE -1 THEN BEGIN
        i = tmp[0]
        telem2d = self->TElem2d()
        IF PTR_VALID(telem2d[i]) THEN BEGIN
            telements = *(telem2d[i])
            n = N_ELEMENTS(telements)
            f = 1
            IF n GE 2 THEN BEGIN
                ; cutting corner :-p
                IF STRMATCH(telements[0],telements[1]) THEN f=2
            ENDIF
            n = N_ELEMENTS(telements) / f
            members = REPLICATE({s0, name:'', bits:0}, n)
            FOR i=0,n-1 DO BEGIN
                members[i] = {s0,telements[f*i],f}
            ENDFOR
        ENDIF
    ENDIF
    RETURN, members
END
FUNCTION bte::Get_Quality_Members
    COMPILE_OPT hidden
    RETURN, self->_Get_Quality_Members()
END

FUNCTION bte::Get_Value, key, POINTER=pointer
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    ; Check whether key is included in the members of structure
    members = TAG_NAMES(*self.data)
    tmp = WHERE(STRMATCH(members, key, /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN 
        rkey = key
        IF STRMATCH(key, 'RANG', /FOLD_CASE) THEN rkey='ROLL'
        IF STRMATCH(key, 'ROLL', /FOLD_CASE) THEN rkey='RANG'
        IF STRMATCH(key, 'DRANG', /FOLD_CASE) THEN rkey='D_ROLL'
        IF STRMATCH(key, 'D_ROLL', /FOLD_CASE) THEN rkey='DRANG'
        IF STRMATCH(key, 'DRA', /FOLD_CASE) THEN rkey='D_RA'
        IF STRMATCH(key, 'D_RA', /FOLD_CASE) THEN rkey='DRA'
        IF STRMATCH(key, 'DDEC', /FOLD_CASE) THEN rkey='D_DEC'
        IF STRMATCH(key, 'D_DEC', /FOLD_CASE) THEN rkey='DDEC'
        IF STRMATCH(key, 'SAT_X', /FOLD_CASE) THEN rkey='SAT_POSX'
        IF STRMATCH(key, 'SAT_Y', /FOLD_CASE) THEN rkey='SAT_POSY'
        IF STRMATCH(key, 'SAT_Z', /FOLD_CASE) THEN rkey='SAT_POSZ'
        IF STRMATCH(key, 'SAT_POSX', /FOLD_CASE) THEN rkey='SAT_X'
        IF STRMATCH(key, 'SAT_POSY', /FOLD_CASE) THEN rkey='SAT_Y'
        IF STRMATCH(key, 'SAT_POSZ', /FOLD_CASE) THEN rkey='SAT_Z'
        IF STRMATCH(key, 'QUATERNION', /FOLD_CASE) THEN rkey='AOCU_ADS_Q'
        IF STRMATCH(key, 'AOCU_ADS_Q', /FOLD_CASE) THEN rkey='QUATERNION'
        IF STRMATCH(key, 'BODY_RATE', /FOLD_CASE) THEN rkey='AOCU_BODY_RATE'
        IF STRMATCH(key, 'AOCU_BODY_RATE', /FOLD_CASE) THEN rkey='BODY_RATE'
        IF STRMATCH(key, 'ATTITUDE', /FOLD_CASE) THEN rkey='QUATERNION'
        IF STRMATCH(key, 'POS_ERR', /FOLD_CASE) THEN rkey='ATTITUDE_ER'
        IF STRMATCH(key, 'ATTITUDE_ER', /FOLD_CASE) THEN rkey='POS_ERR'
        IF STRMATCH(key, 'ANGL_VEL', /FOLD_CASE) THEN rkey='BODY_RATE'
        tmp = WHERE(STRMATCH(members, rkey, /FOLD_CASE) EQ 1)
        IF tmp EQ -1 THEN BEGIN 
            rkey = key
            IF STRMATCH(key, 'QUATERNION', /FOLD_CASE) THEN rkey='ATTITUDE'
            IF STRMATCH(key, 'BODY_RATE', /FOLD_CASE) THEN rkey='ANGL_VEL'
            tmp = WHERE(STRMATCH(members, rkey, /FOLD_CASE) EQ 1)
        ENDIF
    ENDIF
    IF tmp EQ -1 THEN BEGIN
        str = 'TSD' + STRTRIM(STRING(self.extver),2)
        MESSAGE, 'No such member in ' + str + ': ' + key, /CONTINUE
        MESSAGE, 'Available values in ' + self.extname + ' are: ' + $
                 STRJOIN(members,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF
    i = tmp[0]

    IF KEYWORD_SET(pointer) THEN BEGIN
        RETURN, PTR_NEW((*self.data).(i))
    ENDIF ELSE BEGIN
        tmp = (*self.data).(i)
    ; Cast flux into double
    IF STRCMP(key, 'FLUX', /FOLD_CASE) THEN RETURN, DOUBLE(tmp) $
    ELSE RETURN, tmp
    ENDELSE
END

FUNCTION bte::Set_Value, key, val
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    ; Check whether key is included in the members of structure
    members = TAG_NAMES(*self.data)
    tmp = WHERE(STRMATCH(members, key, /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN 
        rkey = key
        IF STRMATCH(key, 'RANG', /FOLD_CASE) THEN rkey='ROLL'
        IF STRMATCH(key, 'ROLL', /FOLD_CASE) THEN rkey='RANG'
        IF STRMATCH(key, 'DRANG', /FOLD_CASE) THEN rkey='D_ROLL'
        IF STRMATCH(key, 'D_ROLL', /FOLD_CASE) THEN rkey='DRANG'
        IF STRMATCH(key, 'DRA', /FOLD_CASE) THEN rkey='D_RA'
        IF STRMATCH(key, 'D_RA', /FOLD_CASE) THEN rkey='DRA'
        IF STRMATCH(key, 'DDEC', /FOLD_CASE) THEN rkey='D_DEC'
        IF STRMATCH(key, 'D_DEC', /FOLD_CASE) THEN rkey='DDEC'
        IF STRMATCH(key, 'SAT_X', /FOLD_CASE) THEN rkey='SAT_POSX'
        IF STRMATCH(key, 'SAT_Y', /FOLD_CASE) THEN rkey='SAT_POSY'
        IF STRMATCH(key, 'SAT_Z', /FOLD_CASE) THEN rkey='SAT_POSZ'
        IF STRMATCH(key, 'SAT_POSX', /FOLD_CASE) THEN rkey='SAT_X'
        IF STRMATCH(key, 'SAT_POSY', /FOLD_CASE) THEN rkey='SAT_Y'
        IF STRMATCH(key, 'SAT_POSZ', /FOLD_CASE) THEN rkey='SAT_Z'
        IF STRMATCH(key, 'QUATERNION', /FOLD_CASE) THEN rkey='AOCU_ADS_Q'
        IF STRMATCH(key, 'AOCU_ADS_Q', /FOLD_CASE) THEN rkey='QUATERNION'
        IF STRMATCH(key, 'BODY_RATE', /FOLD_CASE) THEN rkey='AOCU_BODY_RATE'
        IF STRMATCH(key, 'AOCU_BODY_RATE', /FOLD_CASE) THEN rkey='BODY_RATE'
        IF STRMATCH(key, 'ATTITUDE', /FOLD_CASE) THEN rkey='QUATERNION'
        IF STRMATCH(key, 'POS_ERR', /FOLD_CASE) THEN rkey='ATTITUDE_ER'
        IF STRMATCH(key, 'ATTITUDE_ER', /FOLD_CASE) THEN rkey='POS_ERR'
        IF STRMATCH(key, 'ANGL_VEL', /FOLD_CASE) THEN rkey='BODY_RATE'
        tmp = WHERE(STRMATCH(members, rkey, /FOLD_CASE) EQ 1)
        IF tmp EQ -1 THEN BEGIN 
            rkey = key
            IF STRMATCH(key, 'QUATERNION', /FOLD_CASE) THEN rkey='ATTITUDE'
            IF STRMATCH(key, 'BODY_RATE', /FOLD_CASE) THEN rkey='ANGL_VEL'
            tmp = WHERE(STRMATCH(members, rkey, /FOLD_CASE) EQ 1)
        ENDIF
    ENDIF
    IF tmp EQ -1 THEN BEGIN
        str = 'TSD' + STRTRIM(STRING(self.extver),2)
        MESSAGE, 'No such member in ' + str + ': ' + key, /CONTINUE
        MESSAGE, 'Available values in ' + self.extname + ' are: ' + $
                 STRJOIN(members,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF
    i = tmp[0]

    ; Check type of argument
    sz_key = SIZE((*self.data).(i))
    IF sz_key[0] EQ 1 THEN BEGIN
        ; supress ARRAY[1,n] to ARRAY[n]
        val = REFORM(val, N_ELEMENTS(val), /OVERWRITE)
    ENDIF
    type_error=0
    sz_val = SIZE(val)
    IF N_ELEMENTS(sz_key) NE N_ELEMENTS(sz_val) THEN type_error=1
    IF STRCMP(key, 'FLUX', /FOLD_CASE) THEN BEGIN
        FOR j=0,sz_key[0] DO BEGIN
            IF sz_key[j] NE sz_val[j] THEN type_error=1
        ENDFOR
        IF sz_val[sz_val[0]+1] NE 5 THEN type_error=1 ; Check double or not
    ENDIF ELSE BEGIN
        FOR j=0,N_ELEMENTS(sz_key)-1 DO BEGIN
            IF sz_key[j] NE sz_val[j] THEN type_error=1
        ENDFOR
    ENDELSE
    IF type_error THEN BEGIN
        type = self->Get_ArraySizeType((*self.data).(i))
        MESSAGE, 'Error: Mismatch argument type: ' + key, /CONTINUE
        MESSAGE, 'Type of ' + key + ' must be: ' + type
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF

    (*self.data).(i) = val
    RETURN, !SUCCEED
END

FUNCTION bte::Get_Status, key
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    members = self->Get_Status_Members()
    tmp = WHERE(STRMATCH(members, key, /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN
        str = 'TSD' + STRTRIM(STRING(self.extver),2)
        MESSAGE, 'No such status in ' + str + ': ' + key, /CONTINUE
        MESSAGE, 'Available status in ' + self.extname + ' are: ' + $
                 STRJOIN(members,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF
    i = tmp[0]

    CASE self.extver OF
        4: ptr = PTR_NEW((*self.data).status)
        3: ptr = PTR_NEW((*self.data).status)
        2: ptr = PTR_NEW((*self.data).status)
        1: ptr = PTR_NEW((*self.data).fis_status)
        ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
    ENDCASE

    ; set value to i-th status
    j = FIX(i/8)
    k = FIX(i MOD 8)
    ; get byte array of j-th column of status[4,*]
    orig = REFORM((*ptr)[j,*])
    ; return k-th bit of orig
    ret = self->Get_ith_Bit_OF_Bytarr(orig, k)
    ; Free all heap variables.
    ; See http://www.dfanning.com/fileio_tips/memleak.html for more detail.
    PTR_FREE, ptr

    RETURN, ret
END

FUNCTION bte::Set_Status, key, val
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    members = self->Get_Status_Members()
    tmp = WHERE(STRMATCH(members, key, /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN
        str = 'TSD' + STRTRIM(STRING(self.extver),2)
        MESSAGE, 'No such status in ' + str + ': ' + key, /CONTINUE
        MESSAGE, 'Available status in ' + self.extname + ' are: ' + $
                 STRJOIN(members,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF
    i = tmp[0]

    CASE self.extver OF
        4: ptr = PTR_NEW((*self.data).status)
        3: ptr = PTR_NEW((*self.data).status)
        2: ptr = PTR_NEW((*self.data).status)
        1: ptr = PTR_NEW((*self.data).fis_status)
        ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
    ENDCASE

    ; set value to i-th status
    j = FIX(i/8)
    k = FIX(i MOD 8)
    ; get byte array of j-th column of status[4,*]
    orig = REFORM((*ptr)[j,*])
    ; make a new array
    newarr = self->Set_ith_Bit_OF_Bytarr(orig, val, k)

    ; Reform BYTARR[n] -> BYTARR[1,n]
    IF SIZE(newarr, /N_DIMENSION) EQ 1 THEN BEGIN
    newarr = REFORM(newarr, 1, N_ELEMENTS(newarr), /OVERWRITE)
    ENDIF
    ; This is an inconsistency between IDL 5.6 and 6.0.
    ; IDL5.6: BYTARR[n],  IDL6: BYTARR[1,n]
    IF FLOAT(!VERSION.RELEASE) LT 6.0 THEN BEGIN
    newarr = REFORM(newarr, /OVERWRITE)
    ENDIF

    ; write back to original status
    ; '(*ptr)[j,*] = newarr' does not work.
    CASE self.extver OF
        4: (*self.data).status[j,*] = newarr
        3: (*self.data).status[j,*] = newarr
        2: (*self.data).status[j,*] = newarr
        1: (*self.data).fis_status[j,*] = newarr
        ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
    ENDCASE
    ; Free all heap variables.
    ; See http://www.dfanning.com/fileio_tips/memleak.html for more detail.
    PTR_FREE, ptr

    RETURN, !SUCCEED
END

FUNCTION bte::Get_Flag, key
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    members = self->Get_Flag_Members()
    tmp = WHERE(STRMATCH(members, key, /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN
        str = 'TSD' + STRTRIM(STRING(self.extver),2)
        MESSAGE, 'No such flag in ' + str + ': ' + key, /CONTINUE
        MESSAGE, 'Available flag in ' + self.extname + ' are: ' + $
                 STRJOIN(members,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF
    i = tmp[0]

    ; get byte array of flag
    CASE self.extver OF
        4: orig = REFORM((*self.data).flag)
        3: orig = REFORM((*self.data).flag)
        2: orig = REFORM((*self.data).flag)
        1: orig = REFORM((*self.data).fis_flag)
        ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
    ENDCASE

    ; return k-th bit of orig
    ret = self->Get_ith_Bit_OF_Bytarr(orig, i)
    RETURN, ret
END

FUNCTION bte::Set_Flag, key, val
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    members = self->Get_Flag_Members()
    tmp = WHERE(STRMATCH(members, key, /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN
        str = 'TSD' + STRTRIM(STRING(self.extver),2)
        MESSAGE, 'No such flag in ' + str + ': ' + key, /CONTINUE
        MESSAGE, 'Available flag in ' + self.extname + ' are: ' + $
                 STRJOIN(members,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF
    i = tmp[0]

    ; get byte array of flag
    CASE self.extver OF
        4: orig = (*self.data).flag
        3: orig = (*self.data).flag
        2: orig = (*self.data).flag
        1: orig = (*self.data).fis_flag
        ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
    ENDCASE
    ; make a new array
    newarr =  self->Set_ith_Bit_OF_Bytarr(orig, val, i)
    ; write back to original flag
    CASE self.extver OF
        4: (*self.data).flag = newarr
        3: (*self.data).flag = newarr
        2: (*self.data).flag = newarr
        1: (*self.data).fis_flag = newarr
        ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
    ENDCASE

    RETURN, !SUCCEED
END

FUNCTION bte::Get_Pix_Flag, key, PIXNO=pixno
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    members = self->Get_Pix_Flag_Members()
    tmp = WHERE(STRMATCH(members, key, /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN
        str = 'TSD' + STRTRIM(STRING(self.extver),2)
        MESSAGE, 'No such pix_flag in ' + str + ': ' + key, /CONTINUE
        MESSAGE, 'Available pix_flag in ' + self.extname + ' are: ' + $
                 STRJOIN(members,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF
    i = tmp[0]

    CASE self.extver OF
        4: ptr = PTR_NEW((*self.data).pix_flag)
        3: ptr = PTR_NEW((*self.data).pix_flag)
        2: ptr = PTR_NEW((*self.data).pix_flag)
        1: ptr = PTR_NEW((*self.data).fis_pix_flag)
        ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
    ENDCASE

    ; set value to i-th status
    nbytes_per_pixel = (SIZE(*ptr))[1] / self.ndet
    j = FIX(i/8)
    k = FIX(i MOD 8)
    index = nbytes_per_pixel * INDGEN(self.ndet) + j
    IF N_ELEMENTS(pixno) NE 0 THEN index = nbytes_per_pixel * pixno + j
    ; get integer array of j-th column of pix_flag[index,*]
    orig = (*ptr)[index,*]
    ; return k-th bit of orig
    ret = self->Get_ith_Bit_OF_Bytarr(orig, k)
    ; Free all heap variables recursively.
    ; See http://www.dfanning.com/fileio_tips/memleak.html for more detail.
    PTR_FREE, ptr

    RETURN, ret
END

FUNCTION bte::Set_Pix_Flag, key, val, PIXNO=pixno
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    members = self->Get_Pix_Flag_Members()
    tmp = WHERE(STRMATCH(members, key, /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN
        str = 'TSD' + STRTRIM(STRING(self.extver),2)
        MESSAGE, 'No such pix_flag in ' + str + ': ' + key, /CONTINUE
        MESSAGE, 'Available pix_flag in ' + self.extname + ' are: ' + $
                 STRJOIN(members,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF
    i = tmp[0]

    CASE self.extver OF
        4: ptr = PTR_NEW((*self.data).pix_flag)
        3: ptr = PTR_NEW((*self.data).pix_flag)
        2: ptr = PTR_NEW((*self.data).pix_flag)
        1: ptr = PTR_NEW((*self.data).fis_pix_flag)
        ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
    ENDCASE

    ; set value to i-th status
    nbytes_per_pixel = (SIZE(*ptr))[1] / self.ndet
    j = FIX(i/8)
    k = FIX(i MOD 8)
    index = nbytes_per_pixel * INDGEN(self.ndet) + j
    IF N_ELEMENTS(pixno) NE 0 THEN index = nbytes_per_pixel * pixno + j
    ; get integer array of i-th column of pix_flag[index,*]
    orig = (*ptr)[index,*]
    ; make a new array
    newarr = self->Set_ith_Bit_OF_Bytarr(orig, val, k)

    ; Reform BYTARR[n] -> BYTARR[1,n]
    IF SIZE(newarr, /N_DIMENSION) EQ 1 THEN BEGIN
    newarr = REFORM(newarr, 1, N_ELEMENTS(newarr), /OVERWRITE)
    ENDIF
    ; This is an inconsistency between IDL 5.6 and 6.0.
    ; IDL5.6: BYTARR[n],  IDL6: BYTARR[1,n]
    IF FLOAT(!VERSION.RELEASE) LT 6.0 THEN BEGIN
    newarr = REFORM(newarr, /OVERWRITE)
    ENDIF

    ; write back to original fis_pix_flag
    CASE self.extver OF
        4: (*self.data).pix_flag[index,*] = newarr
        3: (*self.data).pix_flag[index,*] = newarr
        2: (*self.data).pix_flag[index,*] = newarr
        1: (*self.data).fis_pix_flag[index,*] = newarr
        ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
    ENDCASE
    ; Free all heap variables recursively.
    ; See http://www.dfanning.com/fileio_tips/memleak.html for more detail.
    PTR_FREE, ptr

    RETURN, !SUCCEED
END

FUNCTION bte::Get_Quality, key, PIXNO=pixno, BITS=bits
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    members = self->Get_Quality_Members()
    tmp = WHERE(STRMATCH(members.name, key, /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN
        str = 'TSD' + STRTRIM(STRING(self.extver),2)
        MESSAGE, 'No such quality in ' + str + ': ' + key, /CONTINUE
        MESSAGE, 'Available quality in ' + self.extname + ' are: ' + $
                 STRJOIN(members.name,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF
    i = tmp[0]
    bits = members[i].bits

    CASE self.extver OF
        4: ptr = PTR_NEW((*self.data).quality)
        3: ptr = PTR_NEW((*self.data).quality)
        2: MESSAGE, 'not supported in TSD version ' + STRING(self.extver)
        1: MESSAGE, 'not supported in TSD version ' + STRING(self.extver)
        ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
    ENDCASE

    ; set value to i-th status
    n_det = self.ndet
    IF n_det le 0 THEN n_det = 1
    nbytes_per_pixel = (SIZE(*ptr))[1] / n_det
    start = TOTAL((members.bits)[0:i]) - bits
    j = FIX(start/8)
    k = FIX(start MOD 8)
    index = nbytes_per_pixel * INDGEN(n_det) + j
    IF N_ELEMENTS(pixno) NE 0 THEN index = nbytes_per_pixel * pixno + j
    ; get integer array of j-th column of quality[index,*]
    orig = (*ptr)[index,*]
    ; return k-th bit of orig
    ret = self->Get_ith_Bit_OF_Bytarr(orig, k, bits)
    ; Free all heap variables recursively.
    ; See http://www.dfanning.com/fileio_tips/memleak.html for more detail.
    PTR_FREE, ptr

    RETURN, ret
END

FUNCTION bte::Set_Quality, key, val, PIXNO=pixno, BITS=bits
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    members = self->Get_Quality_Members()
    tmp = WHERE(STRMATCH(members.name, key, /FOLD_CASE) EQ 1)
    IF tmp EQ -1 THEN BEGIN
        str = 'TSD' + STRTRIM(STRING(self.extver),2)
        MESSAGE, 'No such quality in ' + str + ': ' + key, /CONTINUE
        MESSAGE, 'Available quality in ' + self.extname + ' are: ' + $
                 STRJOIN(members.name,','), /CONTINUE
;        HELP, /TRACEBACK
        RETURN, !FAIL
    ENDIF
    i = tmp[0]
    bits = members[i].bits

    CASE self.extver OF
        4: ptr = PTR_NEW((*self.data).quality)
        3: ptr = PTR_NEW((*self.data).quality)
        2: MESSAGE, 'not supported in TSD version ' + STRING(self.extver)
        1: MESSAGE, 'not supported in TSD version ' + STRING(self.extver)
        ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
    ENDCASE

    ; set value to i-th status
    n_det = self.ndet
    IF n_det le 0 THEN n_det = 1
    nbytes_per_pixel = (SIZE(*ptr))[1] / n_det
    start = TOTAL((members.bits)[0:i]) - bits
    j = FIX(start/8)
    k = FIX(start MOD 8)
    index = nbytes_per_pixel * INDGEN(n_det) + j
    IF N_ELEMENTS(pixno) NE 0 THEN index = nbytes_per_pixel * pixno + j
    ; get integer array of i-th column of quality[index,*]
    orig = (*ptr)[index,*]
    ; make a new array
    newarr = self->Set_ith_Bit_OF_Bytarr(orig, val, k, bits)

    ; Reform BYTARR[n] -> BYTARR[1,n]
    IF SIZE(newarr, /N_DIMENSION) EQ 1 THEN BEGIN
    newarr = REFORM(newarr, 1, N_ELEMENTS(newarr), /OVERWRITE)
    ENDIF
    ; This is an inconsistency between IDL 5.6 and 6.0.
    ; IDL5.6: BYTARR[n],  IDL6: BYTARR[1,n]
    IF FLOAT(!VERSION.RELEASE) LT 6.0 THEN BEGIN
    newarr = REFORM(newarr, /OVERWRITE)
    ENDIF

    ; write back to original fis_quality
    CASE self.extver OF
        4: (*self.data).quality[index,*] = newarr
        3: (*self.data).quality[index,*] = newarr
        2: MESSAGE, 'not supported in TSD version ' + STRING(self.extver)
        1: MESSAGE, 'not supported in TSD version ' + STRING(self.extver)
        ELSE: MESSAGE, 'Unknown TSD version: ' + STRING(self.extver)
    ENDCASE
    ; Free all heap variables recursively.
    ; See http://www.dfanning.com/fileio_tips/memleak.html for more detail.
    PTR_FREE, ptr

    RETURN, !SUCCEED
END


; INIT should return a scalar TRUE value if the initialization
; is successful, and FALSE if the initialization fails.
FUNCTION bte::INIT, extname, EXTVER=extver ;, DATATYPE=datatype
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    IF NOT KEYWORD_SET(extname) THEN BEGIN
        MESSAGE, "Usage: bte = OBJ_NEW('bte', 'extname', EXTVER=extver)"
        RETURN, !FALSE
    ENDIF
    self.exttype = "BINTABLE"
    self.extname = extname
    self.extver  = 4
    IF KEYWORD_SET(extver) THEN self.extver = extver
;    self.datatype = ''
;    IF KEYWORD_SET(datatype) THEN self.datatype = datatype
;    CASE STRUPCASE(STRTRIM(self.datatype,2)) OF
;        'FIS_SW': self.ndet = 100
;        'FIS_LW': self.ndet = 75
;        'ISO':    self.ndet = 4
;        ELSE:     self.ndet = 0
;    ENDCASE
    RETURN, self->hdu::INIT()
END

PRO bte::CLEANUP
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

;     modname = self->getModuleName()

;     IF STRMATCH(modname, 'FIS_SW', /FOLD_CASE) OR $
;       STRMATCH(modname, 'FIS_LW', /FOLD_CASE) OR $
;       STRMATCH(modname, 'ISO', /FOLD_CASE) THEN BEGIN
;         COMMON fis_sw_block
;         tmp = fis_sw
;         telem2d = tmp[*].telem
;         FOR i=0,N_ELEMENTS(telem2d)-1 DO BEGIN
;             ; HELP, telem2d[i]
;             PTR_FREE, telem2d[i]
;         ENDFOR
;         COMMON fis_lw_block
;         tmp = fis_lw
;         telem2d = tmp[*].telem
;         FOR i=0,N_ELEMENTS(telem2d)-1 DO BEGIN
;             ; HELP, telem2d[i]
;             PTR_FREE, telem2d[i]
;         ENDFOR
;         COMMON iso_block
;         tmp = iso
;         telem2d = tmp[*].telem
;         FOR i=0,N_ELEMENTS(telem2d)-1 DO BEGIN
;             ; HELP, telem2d[i]
;             PTR_FREE, telem2d[i]
;         ENDFOR
;     ENDIF ELSE BEGIN
;         telem2d = self->TElem2d()
;         FOR i=0,N_ELEMENTS(telem2d)-1 DO BEGIN
;             ; HELP, telem2d[i]
;             PTR_FREE, telem2d[i]
;         ENDFOR
;     ENDELSE

    self->hdu::CLEANUP
END

;----------------------------------------------------------------------------
;Routine to define the class structure
;
PRO bte__define
    ; Not intended to be called directly by regular users
    COMPILE_OPT idl2, hidden
    ON_ERROR, 2                 ; Return to caller if an error occurs

    STRUCT_HIDE, { bte, $       ; structure name
          datatype: '', $       ; datatype (for FIS_OBS)
          ndet: 0, $            ; ndet (for FIS_OBS)
          INHERITS hdu }        ; inheritance

END

