; $Id: hdu__define.pro,v 1.7 2006/03/23 06:23:22 baba Exp $
;
; Copyright(C) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; CLASS_NAME:
;   HDU - Header and Data Unit
;
; PURPOSE:
;   Define the structure for HDU, which is a part of TSD.
;
; CATEGORY:
;   Data definition
;
; CALLING SEQUENCE:
;   None
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None
;
; RESTRICTIONS:
;   None
;
; COMMENTS:
;   None
;
; DESCRIPTION:
;   See 'ASTRO-F FIS/Survey Data Structure' in detail.
;
; EXAMPLE:
;   None
;
; SIDE EFFECTS:
;   None
;
; MODIFICATION HISTORY:
;   Written by: Hajime Baba, June 2002.
;
;-

;; anonymous functions
FUNCTION hdu::read
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    MESSAGE, "This method should be overrided."
    RETURN, !SUCCEED
END
FUNCTION hdu::write
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    MESSAGE, "This method should be overrided."
    RETURN, !SUCCEED
END
FUNCTION hdu::create
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    MESSAGE, "This method should be overrided."
    RETURN, !SUCCEED
END

FUNCTION hdu::Get_Header_Keyword, key
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    IF NOT PTR_VALID(self.header) THEN BEGIN
        IF !DEBUG THEN MESSAGE, 'Detects NULL pointer (maybe not initialized yet).', /INFORMATIONAL
        RETURN, !FAIL
    ENDIF
    RETURN, FXPAR(*self.header, key)
END

FUNCTION hdu::Set_Header_Keyword, key, val
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    IF NOT PTR_VALID(self.header) THEN BEGIN
        IF !DEBUG THEN MESSAGE, 'Detects NULL pointer (maybe not initialized yet).', /INFORMATIONAL
        RETURN, !FAIL
    ENDIF
    FXADDPAR, *self.header, key, val[0]
    RETURN, !SUCCEED
END

FUNCTION hdu::Get_Header, POINTER=pointer
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    IF NOT PTR_VALID(self.header) THEN BEGIN
        IF !DEBUG THEN MESSAGE, 'Detects NULL pointer (maybe not initialized yet).', /INFORMATIONAL
        RETURN, !FAIL
    ENDIF
    IF KEYWORD_SET(pointer) THEN BEGIN
        RETURN, self.header
    ENDIF ELSE BEGIN
        RETURN, *self.header
    ENDELSE
END

FUNCTION hdu::Set_Header, header
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    self.header = PTR_NEW(header)
    RETURN, !SUCCEED
END

FUNCTION hdu::Get_Data, POINTER=pointer
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    IF NOT PTR_VALID(self.data) THEN BEGIN
        IF !DEBUG THEN MESSAGE, 'Detects NULL pointer (maybe not initialized yet).', /INFORMATIONAL
        RETURN, !FAIL
    ENDIF
    IF KEYWORD_SET(pointer) THEN BEGIN
        RETURN, self.data
    ENDIF ELSE BEGIN
        RETURN, *self.data
    ENDELSE
END

FUNCTION hdu::Set_Data, data
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    self.data = PTR_NEW(data)
    RETURN, !SUCCEED
END



; INIT should return a scalar TRUE value if the initialization
; is successful, and FALSE if the initialization fails.
FUNCTION hdu::INIT
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    RETURN, !TRUE
END

PRO hdu::CLEANUP
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
    ; Free all heap variables recursively.
    ; See http://www.dfanning.com/fileio_tips/memleak.html for more detail.
    PTR_FREE, self.header
    PTR_FREE, self.data
END

;----------------------------------------------------------------------------
;Routine to define the class structure
;
PRO hdu__define
    ; Not intended to be called directly by regular users
    COMPILE_OPT idl2, hidden
    ON_ERROR, 2                 ; Return to caller if an error occurs

    STRUCT_HIDE, { hdu, $       ; structure name
          exttype: '', $        ; Extension type (PRIMARY or BINTABLE)
          extname: '', $        ; Extension name
          extver:  0, $         ; Extension version number
          header: PTR_NEW(), $  ; Pointer to STRARR
          data: PTR_NEW()}      ; Pointer to anonymous structure array

END

